'use strict';

define(function() {
	function Chat (template, config, options) {
		var self = this;

		self.config = config;
		self.options = options;

		if (template) {
			self.template = template;
		} else {
			self.template = '\
			<div class="c-chat" id="chat-window">\
				<div class="c-chat-header">\
					<div class="c-chat_panel-col">\
						<img src="./scripts/chat/icons/users.png" height="35" width="45" id="chat-users"></i>\
					</div>\
					<div class="c-chat_panel-col">\
						<span class="" id="chat-room-name"></span>\
					</div>\
					<div class="c-chat_panel-col u-ta-r u-pos-r">\
						<span class="chat_msg_count" id="chat-msg-count" style="display: none;"></span>\
						<img src="./scripts/chat/icons/msg.png" height="35" width="45" id="chat-trigger"></i>\
					</div>\
				</div>\
				<div class="c-chat_screen" id="chat-screen">\
				</div>\
				<div class="c-chat_screen" id="users-screen" style="display: none;">\
				</div>\
				<div class="c-chat-footer" id="c-chat-footer">\
					<div class="c-chat-footer-wrap u-va-m">\
						<textarea class="u-bgc--grey-8 c-chat-input u-fw-500 u-w-100" id="chat-msg" placeholder=""></textarea>\
					</div>\
					<div class="u-dtc u-ta-c u-va-m u-fz-28">\
						<input type="button" class="c-chat-btn" id="chat-send" value="">\
					</div>\
				</div>\
			</div>';
		}

		self.connectUrl = '//vircgame.dev:8010';
		if (config && config.webSocketUrl) {
			self.connectUrl = config.webSocketUrl;
		}

		self.socket = null;

		self.el = null;
		self.chatWindow = null;
		self.usersIsShow = false;
		self.chatIsShow = false;
		self.chatMsg = null;
		self.chatUsers = null;
		self.chatScreen = null;
		self.chatRoomName = null;
		self.lastMsgUserId = null;
		self.lastMsg = {
			id: null,
			element: null,
			number: 0
		};
		self.userInfo = null;
		self.unreadMsgCount = 0;
		self.userNames = [];
		self.msgStore = [];

		self.maxMsgLength = 1000;
		self.keyboardHeight = 0;
		self.banTime = null;
		self.textKeypressed = [];

		self.start = function() {
			try {
				self.socket = new WebSocket(self.connectUrl);
				self.socketDelegateEvents();
			} catch (e) {
				setTimeout(function() {
					self.socket = new WebSocket(self.connectUrl);
					self.socketDelegateEvents();
				}, 2000);
			}
		};

		self.login = function (cb) {
			var xmlhttp;
			try {
				xmlhttp = new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
				} catch (E) {
					xmlhttp = false;
				}
			}
			if (!xmlhttp && typeof XMLHttpRequest != 'undefined') {
				xmlhttp = new XMLHttpRequest();
			}

			var url = self.config.serverURLProtocol + '://'+self.config.serverURL + '/users/get_session.json?_=' + (new Date().getTime());
			xmlhttp.open('GET', url, true);
			xmlhttp.onreadystatechange = function() {
				if (xmlhttp.readyState == 4) {
					if(xmlhttp.status == 200) {
						var resp = JSON.parse(xmlhttp.responseText);
						self.socket.send(JSON.stringify({
							type: 'login',
							login: resp.users.session
						}));
						cb();
					}
				}
			};
			xmlhttp.send(null);
		};

		self.device = function () {
			try {
				return device.platform.toLowerCase();
			} catch (e) {
				return 'unknown';
			}
		};

		self.onShow = function () {
			if (self.device() === 'android') {
				self.login(onShow);
			} else {
				onShow();
			}
		};

		function onShow () {
			console.log(self.connectUrl);
			self.chatIsShow = true;
			self.chatWindow = document.getElementById('chat-window');
			self.chatMsg = document.getElementById('chat-msg');
			self.chatSend = document.getElementById('chat-send');
			self.chatUsers = document.getElementById('chat-users');
			self.chatScreen = document.getElementById('chat-screen');
			self.usersScreen = document.getElementById('users-screen');
			self.chatRoomName = document.getElementById('chat-room-name');
			self.chatTrigger = document.getElementById('chat-trigger');

			self.chatSend.value = self.options.tr('Send');

			self.chatUsers.addEventListener('click', self.getUsers);
			self.chatTrigger.addEventListener('click', self.close);
			self.chatSend.addEventListener('click', self.sendMessage);
			self.chatMsg.addEventListener('keydown', self.inputKeyDown);
			self.chatMsg.addEventListener('keyup', self.inputKeyUp);

			self.chatMsg.focus();

			//self.socketDelegateEvents();

			if (self.msgStore.length > 0) {
				for (var i = 0; i < self.msgStore.length; i++) {
					if (self.msgStore[i].type == 'service') {
						self.addServiceMessage(self.msgStore[i]);
					} else {
						self.addMessage(self.msgStore[i]);
					}
				}
			}

			setTimeout(function() {
				self.socket.send(JSON.stringify({type: 'history'}));
			});

			setTimeout(function() {
				self.socket.send(JSON.stringify({type: 'info'}));
			});

			if (self.msgStore.length > 0) {
				self.chatScreen.scrollTop = self.chatScreen.scrollHeight;
			} else {
				setTimeout(function() {
					self.chatScreen.scrollTop = self.chatScreen.scrollHeight;
				}, 500);
			}

			self.clearMsgCount();

			var testData = {
				/*from: {
				 id: 123,
				 username: 'asfasdfas21314',
				 avatar: '1.png'
				 },
				 text: "asdfasfsadf"*/
				type: 'ban',
				ban_time: 3600,
				username: 'Test'
			};

			//self.addServiceMessage(testData);
			//self.addMessage(testData, true);

		};

		self.closeSocket = function () {
			self.socket.close();
		};

		self.reconnect = function () {
			self.msgStore = [];
			self.lastMsgUserId = null;
			self.lastMsg.number = 0;
			self.lastMsg.element = null;
			self.closeSocket();
		};

		self.socketDelegateEvents = function () {

			self.socket.onopen = function () {
				var auth = JSON.parse(localStorage.getItem('auth'));
				auth.type = 'login';
				setTimeout(function() {
					self.socket.send(JSON.stringify(auth));
				});
				setTimeout(function() {
					self.socket.send(JSON.stringify({type: 'info'}));
				});
			};

			self.socket.onerror = function (error) {
				console.log(error);
				var text = self.options.tr('Sorry, but there\'s some problem with your socket or the server is down.');
				self.addMessage({text: text});
			};

			self.socket.onmessage = function (inputMessage) {
				setTimeout(function() {
					try {
						var message = JSON.parse(inputMessage.data);
					} catch (e) {
						console.log('Not a valid JSON: ', inputMessage.data);
						return;
					}

					switch (message.type) {
						case 'message':
							if (self.chatIsShow) {
								if (self.msgStore.length > 0 && self.msgStore[self.msgStore.length - 1].id < message.id) {
									self.msgStore.push(message);
									self.addMessage(message);
								}
								if (self.msgStore.length == 0) {
									self.msgStore.push(message);
									self.addMessage(message);
								}
							} else {
								if (message.from.id != message.to.id) {
									self.incMsgCount();
								}
							}
							break;

						case 'banned':
							if (self.chatScreen) {
								var banTime = null;

								if (isNaN(message.ban_time)) {
									banTime = message.ban_time;
								} else {
									banTime = self.formatBanTime(message.ban_time);
								}
								var text = self.options.tr('You are banned untill');
								if (self.chatIsShow) {
									self.addServiceMessage({text: text, ban_time: banTime, type: 'service'});
									self.msgStore.push({text: text, ban_time: banTime, type: 'service', id: 0});
								} else {
									self.msgStore.push({text: text, ban_time: banTime, type: 'service', id: 0});
									self.incMsgCount();
								}
								self.chatMsg.style.border = '2px solid red';
							}
							break;

						case 'users':
							self.showUsers(message.users);
							break;

						case 'info':
							self.userInfo = message;
							if (self.userInfo.is_banned) {
								self.banTime = self.formatBanTime(message.banned_until);
								self.userInfo.banned_until = self.banTime;
								if (self.chatMsg) {
									self.chatMsg.style.border = '2px solid red';
								}
							} else {
								if (self.chatMsg) {
									self.chatMsg.style.border = '';
									if (self.usersIsShow) {
										var userElement = document.getElementById('chat-user-'+self.userInfo.id).parentElement;
										userElement.style.color = '';
									}
								}
							}
							break;

						case 'service':
							if (self.chatIsShow) {
								if (self.msgStore.length > 0 && self.msgStore[self.msgStore.length - 1].id < message.id) {
									if (!isNaN(message.ban_time)) {
										message.ban_time = self.formatBanTime(message.ban_time);
									}
									self.addServiceMessage(message);
									self.msgStore.push(message);
								}
								if (self.msgStore.length == 0) {
									if (!isNaN(message.ban_time)) {
										message.ban_time = self.formatBanTime(message.ban_time);
									}
									self.addServiceMessage(message);
									self.msgStore.push(message);
								}
							} else {
								self.incMsgCount();
							}
							if (message.subtype == 'ban' && self.usersIsShow) {
								var element = document.getElementById('chat-user-'+message.user_id);
								element.className = 'ban-slider';
								var userElement = element.parentElement;
								userElement.style.color = 'red';
							}
							break;
					}
				});
			};

			self.socket.onclose = function () {
				self.start();
			};
		};

		self.show = function () {
			self.el = document.createElement('div');
			self.el.className = 'c-chat-wrap';
			self.el.innerHTML = self.template;
			document.body.appendChild(self.el);
			self.onShow();
		};

		self.showUsersOrChat = function () {
			(self.usersIsShow ? self.usersScreen : self.chatScreen).style.display = 'block';
			(self.usersIsShow ? self.chatScreen : self.usersScreen).style.display = 'none';
		};

		self.getUsers = function () {
			if (!self.usersIsShow) {
				self.usersIsShow = true;
				self.chatIsShow = false;
				setTimeout(function() {
					self.socket.send(JSON.stringify({type: 'users'}));
				});
			} else {
				self.usersIsShow = false;
				self.chatIsShow = true;
				self.clearMsgCount();
			}
			self.showUsersOrChat();
		};

		self.showUsers = function (users) {
			self.chatIsShow = false;
			self.usersScreen.innerHTML = '';
			for(var i = 0; i < users.length; i++){
				self.addUser(users[i]);
			}
		};

		self.close = function () {
			self.chatIsShow = false;
			self.usersIsShow = false;
			self.lastMsgUserId = null;
			self.lastMsg.number = 0;
			self.lastMsg.element = null;
			document.body.removeChild(self.el);
		};

		self.sendMessage = function (event) {
			if (event) {
				event.preventDefault();
			}

			var message = {
				type: 'message',
				text: (self.chatMsg.value.length > self.maxMsgLength) ? self.chatMsg.value.substr(0, self.maxMsgLength - 1) : self.chatMsg.value
			};
			if (self.usersIsShow) {
				self.usersIsShow = false;
				self.chatIsShow = true;
				self.clearMsgCount();
				self.showUsersOrChat();
			}
			if (self.userInfo.is_banned && message.text.length > 0) {
				var text = self.options.tr('You are banned untill');
				self.addServiceMessage({text: text, ban_time: self.banTime || self.userInfo.banned_until, type: 'service'});
			}
			if (!self.userInfo.is_banned && message.text.length > 0) {
				setTimeout(function() {
					self.socket.send(JSON.stringify(message));
				});
				self.chatMsg.value = '';
			}
			self.chatMsg.focus();
		};

		self.addUser = function (userData) {
			var user = document.createElement('div');
			user.className = 'c-chat-user';
			if (self.userInfo.is_admin && !userData.is_banned && userData.id != self.userInfo.id) {
				user.addEventListener('click', self.openBanSelector);
			}

			if (userData.is_banned) {
				user.style.color = 'red';
			}

			user.innerHTML = '\
				<div class="c-user-avatar u-fl-l" data-user-id="'+userData.id+'">\
					<img class="c-avatar-img" src="' + self.minAvatar(userData.avatar) + '">\
				</div>\
				<div class="c-user-name u-fl-l" data-user-id="'+userData.id+'">'
				+ userData.username +
				'</div>\
				<div class="ban-slider" id="chat-user-'+userData.id+'">\
					<div class="c-user-ban">\
						<div class="c-user-ban_canceled">\
							<input type="button" class="c-ban-cancel-btn" data-user-id="'+ userData.id +'" value="X">\
						</div>\
						<div class="c-user-ban_confirm">\
							<input type="button" class="c-ban-confirm-btn" data-user-id="'+ userData.id +'" value="Ban">\
						</div>\
					</div>\
				</div>';

			self.usersScreen.appendChild(user);
			user.children[0].addEventListener('click', self.addUserName);
			user.children[1].addEventListener('click', self.addUserName);
		};

		self.addMessage = function (data, incoming) {
			if (incoming == null) {
				incoming = true;
			}
			self.lastMsg.id = data.id;
			var message = document.createElement('div');
			if (data.from && data.to) {
				incoming = data.from.id != data.to.id;
			}

			var currUser = self.lastMsgUserId == data.from.id;
			var msgClass = 'c-message-in';
			message.className = 'c-message';

			data.text = self.escapeHtml(data.text);

			var myNickname = self.userInfo.username;
			var regExp = new RegExp('^' + myNickname);

			var boldedText = data.text.replace(regExp, '<b>$&</b>');

			if (incoming) {
				if (currUser) {
					message.className = msgClass;
					//message.className += ' u-pl-msg';
				}

				if (data.from && data.from.avatar && !currUser) {
					self.userNames[data.from.id] = data.from.username;

					message.innerHTML = '\
					<div class="c-user-avatar">\
						<img class="c-avatar-img" src="' + self.minAvatar(data.from.avatar) + '">\
					</div>';
				}
				if (!currUser) {
					message.className += ' u-pb-10';
					message.setAttribute('data-user-id', data.from.id);
					self.lastMsg.number = 0;
					message.innerHTML += '<div class="c-msg-username">' + data.from.username + '</div>'
					msgClass += ' msg-brdrs';
					message.innerHTML += '\
						<div class="'+msgClass+'">'
						+ boldedText +
						'</div>';
				} else {
					message.className += ' msg-brdrs-bottom';

					if (self.lastMsg.number > 1) {
						self.lastMsg.element.children[self.lastMsg.number+1].className = 'c-message-in';
					} else {
						self.lastMsg.element.children[2].className = 'c-message-in msg-brdrs-top';
					}
					message.innerHTML = boldedText;
				}
			} else {
				msgClass = 'c-message-out';
				if (currUser) {
					message.className = msgClass;
					//message.className += ' u-pl-msg';
				}
				if (!currUser) {
					message.className += ' u-pb-10';
					self.lastMsg.number = 0;
					msgClass += ' msg-brdrs';
					message.innerHTML = '\
						<div class="'+msgClass+'">'
						+ boldedText +
						'</div>';
				} else {
					message.className += ' msg-brdrs-bottom';

					if (self.lastMsg.number > 1) {
						self.lastMsg.element.children[self.lastMsg.number-1].className = 'c-message-out';
					} else {
						self.lastMsg.element.children[0].className = 'c-message-out msg-brdrs-top';
					}
					message.innerHTML = boldedText;
				}
			}

			if (data.from) {
				self.lastMsgUserId = data.from.id;
			}
			if (currUser) {
				self.lastMsg.element.appendChild(message);
			} else {
				self.chatScreen.appendChild(message);
				message.addEventListener('click', self.addUserName);
				self.lastMsg.element = message;
			}
			self.lastMsg.number += 1;
			self.chatScreenScroll(message);
		};

		self.addServiceMessage = function (msg) {
			var message = document.createElement('div');
			var msgClass = 'c-message-in u-mt-15 msg-brdrs';
			message.className = 'c-message u-pb-10';
			message.innerHTML = '\
				<div class="c-user-avatar">\
					<img class="c-avatar-img" src="'+self.config.serverURLProtocol+'://'+self.config.serverURL+'/img_origin/avatar-small.jpg">\
				</div>';

			var text = (msg.text) ? msg.text : '';

			if (msg.subtype == 'ban') {
				text = msg.username +' '+ options.tr('banned untill');
				text += ' '+ msg.ban_time;
				if (msg.operator_name) {
					text += ' '+ options.tr('by user') +' '+ msg.operator_name;
				} else {
					text += ' '+ options.tr('by administration');
				}
			}

			if (msg.subtype != 'ban' && msg.ban_time) {
				text += ' '+ msg.ban_time;
			}

			message.innerHTML += '\
				<div class="'+msgClass+'">'
				+ text +
				'</div>';

			self.chatScreen.appendChild(message);
			self.chatScreenScroll(message);

			self.lastMsgUserId = 0;
			self.lastMsg.number = 0;
			self.lastMsg.element = null;
		};

		self.chatScreenScroll = function (newElement) {
			var diff = self.chatScreen.scrollHeight - self.chatScreen.scrollTop - newElement.clientHeight;
			var def = 5;
			if (diff >= self.chatScreen.offsetHeight - def && diff <= self.chatScreen.offsetHeight + def) {
				self.chatScreen.scrollTop = self.chatScreen.scrollHeight;
			}
		}

		self.minAvatar = function (str) {
			var avatarMin = '0_min.png';
			if (str)
				avatarMin = String(str).split('.')[0].split('_')[0] + '_avatar.png';
			var remoteAvatar = self.config.serverURLProtocol + '://' + self.config.serverURL + '/img_origin/avatars/' + avatarMin;
			return remoteAvatar;
		};

		self.openBanSelector = function (event) {
			if (self.userInfo.is_banned) {
				return;
			}
			var element = event.currentTarget;
			for (var i = 0; i < element.childNodes.length; i++) {
				if (element.childNodes[i].className == 'ban-slider') {
					element.childNodes[i].className = 'ban-slider.close';
					element.childNodes[i].firstElementChild.children[0].firstElementChild.addEventListener('click', self.closeBanSelector);
					element.childNodes[i].firstElementChild.children[1].firstElementChild.addEventListener('click', self.setBan);
				}
			}
			element.removeEventListener(event.type, self.openBanSelector);
		};

		self.setBan = function (event) {
			event.preventDefault();
			var userId = event.currentTarget.getAttribute('data-user-id');
			var banMsg = {
				type: 'ban',
				userId: userId,
				banSeconds: 60 * 60
			};
			if (self.userInfo.is_admin) {
				setTimeout(function() {
					self.socket.send(JSON.stringify(banMsg));
				});
			}
		};

		self.closeBanSelector = function (event) {
			event.preventDefault();
			var userId = event.currentTarget.getAttribute('data-user-id');
			var element = document.getElementById('chat-user-'+userId);
			element.className = 'ban-slider';
			event.stopPropagation();
			element.parentNode.addEventListener('click', self.openBanSelector);
		}

		self.inputKeyDown = function (e) {
			/* 
				8  - backspace
				13 - enter
				17 - ctrl
			*/
			if (e.keyCode === 8) {
				return;
			}
			
			if (options.platform == 'browser') {
				if (self.textKeypressed.indexOf(e.keyCode) == -1) {
					self.textKeypressed.push(e.keyCode);
				}
				if (self.textKeypressed.length == 2) {
					if (self.textKeypressed.indexOf(13) != -1 && self.textKeypressed.indexOf(17) != -1) {
						self.chatMsg.value = self.chatMsg.value + '\n';
						self.textKeypressed = [];
					} else if (self.textKeypressed.indexOf(13) != -1) {
						self.sendMessage();
					}
				}

				if (self.textKeypressed.length > 2) {
					if (e.keyCode == 13) {
						e.preventDefault();
						self.sendMessage();
					}
				}

				if (self.textKeypressed.length == 1 && e.keyCode == 13) {
					e.preventDefault();
					self.sendMessage();
				}
			}

			if (self.chatMsg.value.length >= self.maxMsgLength-1) {
				var cropText = self.chatMsg.value.substr(0, self.maxMsgLength - 1);
				self.chatMsg.value = cropText + '...';
			}
		};

		self.inputKeyUp = function (e) {
			var index = self.textKeypressed.indexOf(e.keyCode);
			if (index != -1) {
				self.textKeypressed.splice(index, 1);
			}
		}

		self.clearMsgCount = function () {
			self.unreadMsgCount = 0;
			var chatMsgCount = document.getElementById('chat-msg-count');
			var elements = document.getElementsByClassName('chat_outer_msg_count');
			for(var i=0, l=elements.length; i<l; i++){
				elements[i].style.display = 'none';
				elements[i].innerHTML = '';
			}

			chatMsgCount.style.display = 'none';
			chatMsgCount.innerHTML = '';
		}

		self.updateMsgCounter = function () {
			var chatMsgCount = document.getElementById('chat-msg-count');
			var elements = document.getElementsByClassName('chat_outer_msg_count');
			if (chatMsgCount && self.usersIsShow && self.unreadMsgCount > 0) {
				chatMsgCount.style.display = '';
				chatMsgCount.innerHTML = ' ';
			} else if (elements.length && !self.usersIsShow && !self.chatIsShow && self.unreadMsgCount > 0) {
				for(var i=0, l=elements.length; i<l; i++){
					elements[i].style.display = '';
					elements[i].innerHTML = ' ';
				}
			}
		}

		self.incMsgCount = function () {
			self.unreadMsgCount++;
			self.updateMsgCounter();
		}

		self.addUserName = function (event) {
			var userId = event.currentTarget.getAttribute('data-user-id');
			if (self.userNames[userId] != null) {
				self.chatMsg.value += self.userNames[userId] + ', ';
			}
		}

		self.setKeyboardHeight = function (value) {
			self.keyboardHeight = value;
		}

		self.triggerKeyBoard = function (keyboardHeight) {
			var keyboardheight = (keyboardHeight) ? keyboardHeight : self.keyboardHeight;
			if (options.platform == 'android') {
				keyboardheight += 30;
				self.chatWindow.setAttribute('style', 'top: auto !important; bottom: '+keyboardheight+'px;' + 
				' -webkit-transform: translate(0px, 0px) !important;' +
				' transform: translate(0px, 0px) !important; height: 100% !important;');
			}
		}

		self.unTriggerKeyBoard = function () {
			if (options.platform == 'android') {
				self.chatWindow.setAttribute('style', '');
			}
		}

		self.escapeHtml = function (str) {
			var div = document.createElement('div');
			div.appendChild(document.createTextNode(str));
			return div.innerHTML;
		}

		self.isChatOpen = function () {
			return self.chatIsShow || self.usersIsShow;
		}

		self.formatBanTime = function (timestamp) {
			var bTime = new Date(timestamp);
			var banTime = (bTime.getMonth()+1 > 9) ? bTime.getMonth()+1 : '0'+(bTime.getMonth()+1)+'.';
			banTime += (bTime.getDate() > 9) ? bTime.getDate()+'-' : '0'+bTime.getDate()+'-';
			banTime += (bTime.getHours() > 9) ? bTime.getHours()+':' : '0'+bTime.getHours()+':';
			banTime += (bTime.getMinutes() > 9) ? bTime.getMinutes() : '0'+bTime.getMinutes();
			return banTime;
		}
	}

	return Chat;
});
