var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['marionette', 'jquery'], function(Marionette, $) {
  var QuestsController;
  QuestsController = (function(superClass) {
    extend(QuestsController, superClass);

    function QuestsController() {
      this.doIfCompleted = bind(this.doIfCompleted, this);
      this.checkCompleted = bind(this.checkCompleted, this);
      this._onQuestCompleted = bind(this._onQuestCompleted, this);
      this._onQuestFinished = bind(this._onQuestFinished, this);
      this._onQuestTaken = bind(this._onQuestTaken, this);
      this._onQuestAvailable = bind(this._onQuestAvailable, this);
      this.clearCache = bind(this.clearCache, this);
      this._bindEvents = bind(this._bindEvents, this);
      return QuestsController.__super__.constructor.apply(this, arguments);
    }

    QuestsController.prototype._cache = {};

    QuestsController.prototype._availableQuestsDeferred = {};

    QuestsController.prototype._takenQuestsDeferred = {};

    QuestsController.prototype._finishedQuestsDeferred = {};

    QuestsController.prototype._completedQuestsDeferred = {};

    QuestsController.prototype.initialize = function(arg) {
      this.app = arg.app;
      return this._bindEvents();
    };

    QuestsController.prototype._bindEvents = function() {
      this.app.vent.on('quests:quest_available', this._onQuestAvailable);
      this.app.vent.on('quests:quest_taken', this._onQuestTaken);
      this.app.vent.on('quests:quest_finished', this._onQuestFinished);
      return this.app.vent.on('quests:quest_completed', this._onQuestCompleted);
    };

    QuestsController.prototype.clearCache = function() {
      return this._cache = {};
    };

    QuestsController.prototype._onQuestAvailable = function(quest) {
      var deferred;
      if (!quest || !quest.cid) {
        return;
      }
      deferred = this.whenQuestAvailable(quest.cid);
      if (deferred.state() === 'pending') {
        return deferred.resolve(quest);
      }
    };

    QuestsController.prototype._onQuestTaken = function(quest) {
      var deferred;
      if (!quest || !quest.cid) {
        return;
      }
      deferred = this.whenQuestTaken(quest.cid);
      if (deferred.state() === 'pending') {
        return deferred.resolve(quest);
      }
    };

    QuestsController.prototype._onQuestFinished = function(quest) {
      var deferred;
      if (!quest || !quest.cid) {
        return;
      }
      deferred = this.whenQuestFinished(quest.cid);
      if (deferred.state() === 'pending') {
        return deferred.resolve(quest);
      }
    };

    QuestsController.prototype._onQuestCompleted = function(quest) {
      var deferred;
      if (!quest || !quest.cid) {
        return;
      }
      deferred = this.whenQuestCompleted(quest.cid);
      if (deferred.state() === 'pending') {
        return deferred.resolve(quest);
      }
    };

    QuestsController.prototype.whenQuestState = function(state, cid) {
      switch (state) {
        case 'available':
          return this.whenQuestAvailable(cid);
        case 'taken':
          return this.whenQuestTaken(cid);
        case 'finished':
          return this.whenQuestFinished(cid);
        case 'completed':
          return this.whenQuestCompleted(cid);
        default:
          throw new Error('invalid state');
      }
    };

    QuestsController.prototype.whenQuestAvailable = function(cid) {
      if (!this._availableQuestsDeferred[cid]) {
        this._availableQuestsDeferred[cid] = $.Deferred();
      }
      return this._availableQuestsDeferred[cid];
    };

    QuestsController.prototype.whenQuestTaken = function(cid) {
      if (!this._takenQuestsDeferred[cid]) {
        this._takenQuestsDeferred[cid] = $.Deferred();
      }
      return this._takenQuestsDeferred[cid];
    };

    QuestsController.prototype.whenQuestFinished = function(cid) {
      if (!this._finishedQuestsDeferred[cid]) {
        this._finishedQuestsDeferred[cid] = $.Deferred();
      }
      return this._finishedQuestsDeferred[cid];
    };

    QuestsController.prototype.whenQuestCompleted = function(cid) {
      if (!this._completedQuestsDeferred[cid]) {
        this._completedQuestsDeferred[cid] = $.Deferred();
      }
      return this._completedQuestsDeferred[cid];
    };

    QuestsController.prototype.checkCompleted = function(cid) {
      var deferred;
      deferred = $.Deferred();
      if (this._cache.hasOwnProperty(cid)) {
        if (this._cache[cid].completed) {
          deferred.resolve(true);
        } else {
          deferred.reject(this._cache[cid]);
        }
      } else {
        app.server.quests.check_completed({
          cid: cid,
          success: (function(_this) {
            return function(data) {
              if (data) {
                _this._cache[cid] = {
                  completed: (data != null ? data.completed : void 0) || 0,
                  message: (data != null ? data.message : void 0) || app.tr('Access Denied')
                };
              }
              if (!data || !(data != null ? data.completed : void 0)) {
                return deferred.reject({
                  message: (data != null ? data.message : void 0) || app.tr('Access Denied')
                });
              } else {
                return deferred.resolve(true);
              }
            };
          })(this),
          error: function() {
            return deferred.reject({
              message: app.tr('Access Denied')
            });
          }
        });
      }
      return deferred;
    };

    QuestsController.prototype.doIfCompleted = function(cid, fn) {
      return this.checkCompleted(cid).then(function() {
        return fn();
      }, function(data) {
        return app.screens.modal('common/info-popup', {
          msg: data.message
        });
      });
    };

    return QuestsController;

  })(Marionette.Controller);
  return QuestsController;
});
