var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['marionette', 'jquery'], function(Marionette, $) {
  var ShopController;
  ShopController = (function(superClass) {
    extend(ShopController, superClass);

    function ShopController() {
      this._buyQuery = bind(this._buyQuery, this);
      this._buy = bind(this._buy, this);
      this.buyItem = bind(this.buyItem, this);
      return ShopController.__super__.constructor.apply(this, arguments);
    }

    ShopController.prototype._purchaseAttempts = {};

    ShopController.prototype.CURRENCY_VGOLD_ID = 1;

    ShopController.prototype.CURRENCY_VDOLLARS_ID = 2;

    ShopController.prototype.CURRENCY_PVP_BADGES_ID = 3;

    ShopController.prototype.initialize = function(arg) {
      this.app = arg.app;
    };

    ShopController.prototype.gotoVGoldPurchase = function() {
      var app;
      app = this.app;
      if (app.paymentsService.getEnabledProviders().length < 1) {
        app.screens.modal('common/info-popup', {
          msg: app.tr("In-app purchases disabled")
        });
        return;
      }
      app.showLoader();
      return app.paymentsService.readyAll().then(function() {
        app.hideLoader();
        return app.screens.main('shop/vgold');
      }, function() {
        app.hideLoader();
        return app.screens.modal('common/info-popup', {
          msg: app.tr("In-app purchases disabled")
        });
      });
    };

    ShopController.prototype.buyItem = function(itemId, quantity, currency) {
      if (arguments.length < 2) {
        quantity = 1;
      }
      return this._checkCanBuy({
        item_id: itemId,
        quantity: quantity,
        currency: currency
      }).then(this._buy);
    };

    ShopController.prototype._checkCanBuy = function(params) {
      var deferred, self;
      deferred = $.Deferred();
      self = this;
      this.app.server.shop.check_can_buy({
        id: params.item_id,
        quantity: params.quantity,
        currency: params.currency,
        success: function(data) {
          var resultData;
          resultData = {
            status: 'error'
          };
          if (data && (data != null ? data.status : void 0) === 'success') {
            resultData.status = 'success';
            resultData.item_id = data.id;
            resultData.quantity = data.quantity;
            resultData.currency = params.currency;
          } else {
            resultData.error = self._setErrorFromData(data);
          }
          if (resultData.status === 'error') {
            return deferred.reject(resultData);
          } else {
            return deferred.resolve(resultData);
          }
        },
        error: function(data) {
          var resultData;
          resultData = {
            status: 'error',
            error: self._setErrorFromData(data)
          };
          return deferred.reject(resultData);
        }
      });
      return deferred;
    };

    ShopController.prototype._buy = function(params) {
      var itemId, self;
      itemId = params.item_id;
      self = this;
      this._purchaseAttempts[itemId] = 3;
      return self._buyQuery(params).always(function(data) {
        if ((data != null ? data.error : void 0) && data.error.code === 'LOCK_ERROR' && self._purchaseAttempts[itemId] > 0) {
          self._purchaseAttempts[itemId]--;
          return self._buyQuery(params);
        } else {
          return data;
        }
      });
    };

    ShopController.prototype._buyQuery = function(params) {
      var deferred, self;
      self = this;
      deferred = $.Deferred();
      this.app.server.shop.buy({
        id: params.item_id,
        quantity: params.quantity,
        currency: params.currency,
        success: function(data) {
          var resultData;
          console.log('shop.buy success');
          resultData = {
            status: 'error'
          };
          if (data && (data != null ? data.status : void 0) === 'success') {
            resultData.status = 'success';
            resultData.item_id = data.id;
            resultData.quantity = data.quantity;
            resultData.price = data.price;
          } else {
            resultData.error = self._setErrorFromData(data);
          }
          if (resultData.status === 'error') {
            return deferred.reject(resultData);
          } else {
            return deferred.resolve(resultData);
          }
        },
        error: function(data) {
          var resultData;
          resultData = {
            status: 'error',
            error: self._setErrorFromData(data)
          };
          return deferred.reject(resultData);
        }
      });
      return deferred;
    };

    ShopController.prototype._onError = function(data) {
      return app.screens.modal('common/info-popup', {
        msg: data.error.message
      });
    };

    ShopController.prototype._setErrorFromData = function(data) {
      if ((data != null) && ((data != null ? data.setFlash[0] : void 0) != null)) {
        return {
          code: data.error_code || 'FATAL_ERROR',
          message: data.setFlash[0].msg
        };
      } else {
        return {
          code: 'FATAL_ERROR',
          'Server error': 'Server error'
        };
      }
    };

    return ShopController;

  })(Marionette.Controller);
  return ShopController;
});
