define(['jquery', 'services/payments/platform', 'services/payments/xsolla'], function($, PlatformPayments, XsollaPayments) {
  var PaymentsService;
  PaymentsService = (function() {
    PaymentsService.prototype._providers = {};

    function PaymentsService(app, configs, serverHost, serverProtocol) {
      var config, platform, ref, ref1;
      platform = this._detectPlatform();
      if ((configs != null ? (ref = configs.platform) != null ? ref.devices : void 0 : void 0) && configs.platform.devices.indexOf(platform) !== -1) {
        config = (configs != null ? configs.platform : void 0) || {};
        config.platform = platform;
        config.host = serverHost;
        config.protocol = serverProtocol || 'http';
        this._providers['platform'] = new PlatformPayments(config);
      }
      if ((configs != null ? (ref1 = configs.xsolla) != null ? ref1.devices : void 0 : void 0) && configs.xsolla.devices.indexOf(platform) !== -1) {
        config = (configs != null ? configs.xsolla : void 0) || {};
        config.platform = platform;
        config.host = serverHost;
        config.protocol = serverProtocol || 'http';
        this._providers['xsolla'] = new XsollaPayments(config);
      }
    }

    PaymentsService.prototype.getEnabledProviders = function() {
      var name, provider, ref, result;
      result = [];
      ref = this._providers;
      for (name in ref) {
        provider = ref[name];
        if (provider.isEnabled()) {
          result.push(name);
        }
      }
      return result;
    };

    PaymentsService.prototype.provider = function(provider) {
      return this._providers[provider];
    };

    PaymentsService.prototype.isEnabled = function(provider) {
      return !!(this._providers.hasOwnProperty(provider) && this._providers[provider].isEnabled());
    };

    PaymentsService.prototype.readyAll = function() {
      var deferred, platformDeferred, xsollaDeffered;
      deferred = $.Deferred();
      xsollaDeffered = this._providers.hasOwnProperty('xsolla') ? this._providers.xsolla.ready() : false;
      platformDeferred = this._providers.hasOwnProperty('platform') ? this._providers.platform.ready() : false;
      $.when(xsollaDeffered, platformDeferred).then(function(xsolla, platform) {
        if (xsolla || platform) {
          return deferred.resolve(true);
        } else {
          return deferred.resolve(false);
        }
      });
      return deferred;
    };

    PaymentsService.prototype.setPendingPurchaseCallbacks = function(success, error) {
      var name, provider, ref, results;
      ref = this._providers;
      results = [];
      for (name in ref) {
        provider = ref[name];
        results.push(provider.setPendingPurchaseCallbacks(success, error));
      }
      return results;
    };

    PaymentsService.prototype._detectPlatform = function() {
      var ua;
      if ((typeof device !== "undefined" && device !== null) && device.platform) {
        return device.platform.toLowerCase();
      }
      ua = navigator.userAgent;
      if (window.isCordova === true) {
        if (ua.match(/iPad|iPod|iPhone/)) {
          return 'ios';
        }
        if (ua.match(/Android/i)) {
          return 'android';
        }
      }
      if (ua.match(/Windows Phone|WPDesktop/i)) {
        return 'windows phone';
      }
      return 'browser';
    };

    return PaymentsService;

  })();
  return PaymentsService;
});
