var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

define(['jquery'], function($) {
  var LightBox, XsollaPayments;
  LightBox = (function() {
    LightBox.prototype._elements = {};

    LightBox.prototype._handlers = {};

    function LightBox() {
      this._createDOMElements();
      this._bindHandlers();
    }

    LightBox.prototype.show = function(url) {
      var iframe;
      iframe = document.createElement('IFRAME');
      iframe.width = '100%';
      iframe.height = '100%';
      iframe.id = 'xsolla-iframe';
      iframe.name = 'xsolla-iframe';
      iframe.src = url;
      this._elements.body.appendChild(iframe);
      return document.body.appendChild(this._elements.container);
    };

    LightBox.prototype.onClose = function(fn) {
      return this._handlers.onClose = fn;
    };

    LightBox.prototype.close = function() {
      return _close();
    };

    LightBox.prototype._createDOMElements = function() {
      this._elements.container = document.createElement('DIV');
      this._elements.container.setAttribute('class', 'c-xsolla-lightbox');
      this._elements.top = document.createElement('DIV');
      this._elements.top.setAttribute('class', 'c-xsolla-lightbox__top');
      this._elements.closeBtn = document.createElement('DIV');
      this._elements.closeBtn.setAttribute('class', 'c-xsolla-lightbox__close');
      this._elements.body = document.createElement('DIV');
      this._elements.body.setAttribute('class', 'c-xsolla-lightbox__body');
      this._elements.top.appendChild(this._elements.closeBtn);
      this._elements.container.appendChild(this._elements.top);
      return this._elements.container.appendChild(this._elements.body);
    };

    LightBox.prototype._bindHandlers = function() {
      var self;
      self = this;
      return this._elements.closeBtn.addEventListener('click', function() {
        return self._close();
      });
    };

    LightBox.prototype._close = function() {
      this._elements.container.parentNode.removeChild(this._elements.container);
      if (this._handlers.onClose) {
        return this._handlers.onClose();
      }
    };

    return LightBox;

  })();
  XsollaPayments = (function() {
    XsollaPayments.prototype._xsollaProductionUri = '//secure.xsolla.com/paystation2/';

    XsollaPayments.prototype._xsollaTestUri = '//sandbox-secure.xsolla.com/paystation2/';

    XsollaPayments.prototype._tokenPath = '/payments/xsolla/getVirtualCurrencyToken';

    XsollaPayments.prototype._paymentStatusPath = '/payments/xsolla/paymentStatus';

    XsollaPayments.prototype._currencyListPath = '/payments/xsolla/getVirtualCurrencyList';

    XsollaPayments.prototype._readyDeferred = null;

    XsollaPayments.prototype._currencyList = {};

    XsollaPayments.prototype._isReady = false;

    XsollaPayments.prototype._isEnabled = true;

    function XsollaPayments(config) {
      this._openPopup = bind(this._openPopup, this);
      this._checkPaymentCompleted = bind(this._checkPaymentCompleted, this);
      this._loadVirtualCurrencyList = bind(this._loadVirtualCurrencyList, this);
      this._loadToken = bind(this._loadToken, this);
      this.ready = bind(this.ready, this);
      this._isTest = (config != null ? config.sandbox : void 0) ? true : false;
      this._currentPaymentId = 0;
      this._lastUrlHost = null;
      this._serverHost = config.host;
      this._serverUri = (config.protocol || 'http') + '://' + config.host;
      this._tokenUrl = this._serverUri + this._tokenPath;
      this._paymentStatusUrl = this._serverUri + this._paymentStatusPath;
      this._currencyListUrl = this._serverUri + this._currencyListPath;
      if (this._isTest) {
        this._xsollaUri = this._xsollaTestUri;
      } else {
        this._xsollaUri = this._xsollaProductionUri;
      }
      this._xsollaHost = this._getHostFromUrl(this._xsollaUri);
      this._isReady = false;
      this._readyDeferred = $.Deferred();
    }

    XsollaPayments.prototype.getName = function() {
      return 'Xsolla';
    };

    XsollaPayments.prototype.isEnabled = function() {
      return this._isEnabled;
    };

    XsollaPayments.prototype.ready = function() {
      if (this._isReady === false) {
        this._loadVirtualCurrencyList();
      }
      return this._readyDeferred;
    };

    XsollaPayments.prototype.setPendingPurchaseCallbacks = function(success, error) {};

    XsollaPayments.prototype.refresh = function() {};

    XsollaPayments.prototype.loadCurrencyList = function() {
      return this._currencyList;
    };

    XsollaPayments.prototype.getCurrencyVgold = function(id) {
      var i, item, len, ref, vgold;
      vgold = 0;
      ref = this._currencyList;
      for (i = 0, len = ref.length; i < len; i++) {
        item = ref[i];
        if (item.id == id) {
          vgold = item.vgold;
          break;
        }
      }
      return vgold;
    };

    XsollaPayments.prototype.order = function(itemId) {
      var deferred, self;
      deferred = $.Deferred();
      self = this;
      this._loadToken(itemId).then(this._openPopup).always(function(data) {
        return self._checkPaymentCompleted(data).then(deferred.resolve, deferred.reject);
      });
      return deferred;
    };

    XsollaPayments.prototype._loadToken = function(itemId) {
      return $.ajax({
        url: this._tokenUrl + (this._tokenUrl.indexOf('?') !== -1 ? '&' : '?') + 'item_id=' + itemId,
        xhrFields: {
          withCredentials: true
        }
      });
    };

    XsollaPayments.prototype._loadVirtualCurrencyList = function() {
      var self;
      self = this;
      return $.ajax({
        url: this._currencyListUrl,
        success: function(data) {
          self._isReady = true;
          if (data && (data != null ? data.items : void 0) && data.items.length) {
            self._currencyList = data.items;
            return self._readyDeferred.resolve(true);
          } else {
            return self._readyDeferred.resolve(false);
          }
        },
        error: function() {
          self._isEnabled = false;
          self._isReady = true;
          return self._readyDeferred.resolve(false);
        }
      });
    };

    XsollaPayments.prototype._checkPaymentCompleted = function(data) {
      var deferred, paymentId, xsollaStatus;
      deferred = $.Deferred();
      if (!data || !data.payment_id) {
        deferred.reject(false);
        return deferred;
      }
      paymentId = data.payment_id;
      xsollaStatus = data.status || 'error';
      $.ajax({
        url: this._paymentStatusUrl + '?payment_id=' + paymentId,
        method: 'GET'
      }).then(function(response) {
        var resultData;
        resultData = {
          status: 'error'
        };
        if (!response || !(response != null ? response.payment_status : void 0)) {
          resultData.error = {
            code: 'FATAL_ERROR',
            message: 'Server error'
          };
          deferred.resolve(resultData);
          return;
        }
        if (response.payment_status === 3) {
          resultData.status = 'success';
          resultData.amount = response.amount;
          resultData.transaction_id = response.transaction_id;
          resultData.payment_id = response.payment_id;
          deferred.resolve(resultData);
          return;
        } else if (xsollaStatus === 'semi-success' || xsollaStatus === 'success') {
          resultData.status = 'uncompleted';
          resultData.transaction_id = paymentId;
          deferred.resolve(resultData);
          return;
        }
        resultData.error = {
          code: 'PAYMENT_FAILED',
          message: 'Payment failed or not completed correctly'
        };
        return deferred.reject(resultData);
      }, function() {
        var resultData;
        resultData = {
          status: 'error',
          error: {
            code: 'FATAL_ERROR',
            message: 'Server error'
          }
        };
        return deferred.reject(resultData);
      });
      return deferred;
    };

    XsollaPayments.prototype._openPopup = function(data) {
      var child, deferred, lightBox, messageHandler, resultData, self, url;
      this._lastUrlHost = null;
      deferred = $.Deferred();
      if (!data || !data.token) {
        deferred.reject(false);
        return deferred;
      }
      resultData = {
        payment_id: data.payment_id,
        status: 'error'
      };
      url = this._xsollaUri + '?' + $.param({
        access_token: data.token
      });
      self = this;
      if (window.isCordova === false) {
        lightBox = new LightBox;
        messageHandler = function(event) {
          var message, ref;
          message = event.data;
          if (message && typeof message === 'string') {
            message = JSON.parse(message);
            message = message.data;
          }
          if (message && ((ref = message.paymentInfo) != null ? ref.status : void 0) === 'done') {
            resultData.status = 'success';
          }
          if ((message != null ? message.status : void 0) === 'success') {
            resultData.status = 'success';
            return lightBox.close();
          }
        };
        window.addEventListener('message', messageHandler);
        lightBox.onClose(function() {
          window.removeEventListener('message', messageHandler);
          if (deferred.state() === 'pending') {
            if (resultData.status === 'success') {
              return deferred.resolve(resultData);
            } else {
              return deferred.reject(resultData);
            }
          }
        });
        lightBox.show(url);
      } else if (window.isCordova === true) {
        child = window.open(url, '_blank');
        child.addEventListener('loadstop', function(event) {
          var currentHost;
          child.executeScript({
            code: 'navigator.mimeTypes = navigator.mimeTypes || {}'
          });
          currentHost = self._getHostFromUrl(event.url);
          self._lastUrlHost = currentHost;
          if (currentHost === self._serverHost) {
            return child.executeScript({
              code: "localStorage.getItem('postMessage')"
            }, function(data) {
              var error, message;
              message = data[0];
              try {
                message = JSON.parse(message);
              } catch (_error) {
                error = _error;
                message = null;
              }
              if (message && (message != null ? message.status : void 0) === 'success') {
                resultData.status = 'success';
                deferred.resolve(resultData);
              }
              return child.close();
            });
          }
        });
        child.addEventListener('exit', function() {
          if (deferred.state() === 'pending') {
            if (self._lastUrlHost === self._serverHost || self._lastUrlHost === self._xsollaHost) {
              resultData.status === 'semi-success';
              return deferred.resolve(resultData);
            } else {
              return deferred.reject(resultData);
            }
          }
        });
      } else {
        deferred.reject(resultData);
      }
      return deferred;
    };

    XsollaPayments.prototype._getHostFromUrl = function(url) {
      var a;
      a = document.createElement('a');
      a.href = url;
      return a.hostname;
    };

    return XsollaPayments;

  })();
  return XsollaPayments;
});
