var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

define(['jquery'], function($) {
  var BaseProvider;
  BaseProvider = (function() {
    function BaseProvider(type, config) {
      this._onError = bind(this._onError, this);
      this._genericXHRErrorHandler = bind(this._genericXHRErrorHandler, this);
      this._afterAuth = bind(this._afterAuth, this);
      this._verifyToken = bind(this._verifyToken, this);
      this._showPopup = bind(this._showPopup, this);
      this._storeUserData = bind(this._storeUserData, this);
      this._getFriendIds = bind(this._getFriendIds, this);
      this.getFriendIds = bind(this.getFriendIds, this);
      this._getCurrentUserData = bind(this._getCurrentUserData, this);
      this._getTokenRequestData = bind(this._getTokenRequestData, this);
      this._tokenRequest = bind(this._tokenRequest, this);
      this.grantScopeRequest = bind(this.grantScopeRequest, this);
      this.reauth = bind(this.reauth, this);
      this.auth = bind(this.auth, this);
      this.getCurrentUserData = bind(this.getCurrentUserData, this);
      this.getCurrentToken = bind(this.getCurrentToken, this);
      this.getCurrentUserId = bind(this.getCurrentUserId, this);
      this.isAuthenticated = bind(this.isAuthenticated, this);
      this.getType = bind(this.getType, this);
      this._init = bind(this._init, this);
      this.SCOPE_ID = 0x1;
      this.SCOPE_NAME = 0x2;
      this.SCOPE_EMAIL = 0x4;
      this.SCOPE_GENDER = 0x8;
      this.SCOPE_BIRTHDAY = 0x10;
      this.SCOPE_FRIENDS = 0x20;
      this.SCOPE_WALL = 0x40;
      this.SCOPE_MESSAGES = 0x80;
      this.SCOPE_OFFLINE = 0x100;
      this._scopeDelimiter = ',';
      this._protocol = config.serverProtocol || http;
      this._serverHost = config.serverUrl;
      this._serverUrl = this._protocol + '://' + this._serverHost;
      this._tokenRedirectUri = this._serverUrl + '/social/api/clientCallback';
      this._tokenDisplay = 'popup';
      this._apiVersion = '';
      this._type = type;
      this._needAuth = true;
      this._tokenVerified = false;
      this._addTokenData = {};
      this._forceTokenData = {};
    }

    BaseProvider.prototype._init = function(config) {
      var ref, ref1, timestamp;
      this._appId = config['app_id'];
      this._session = this._loadSessionData();
      this._userData = null;
      this._token = this._session ? this._session.token : null;
      this._baseScope = this.SCOPE_OFFLINE | this.SCOPE_ID | this.SCOPE_NAME | this.SCOPE_EMAIL | this.SCOPE_GENDER | this.SCOPE_FRIENDS;
      timestamp = Math.floor(Date.now() / 1000);
      if (((ref = this._session) != null ? ref.token : void 0) && (this._session.expires === 0 || this._session.expires > timestamp) && ((ref1 = this._session) != null ? ref1.user_id : void 0)) {
        return this._needAuth = false;
      }
    };

    BaseProvider.prototype.getType = function() {
      return this._type;
    };

    BaseProvider.prototype.isAuthenticated = function() {
      return !this._needAuth;
    };

    BaseProvider.prototype.getCurrentUserId = function() {
      if (this._userData) {
        return this._userData.user_id;
      } else {
        return null;
      }
    };

    BaseProvider.prototype.getCurrentToken = function() {
      if (this._token) {
        return this._token;
      } else {
        return this._session.token;
      }
    };

    BaseProvider.prototype.getError = function() {
      return {
        code: this._errCode,
        msg: this._errMsg
      };
    };

    BaseProvider.prototype.getCurrentUserData = function() {
      return this.auth();
    };

    BaseProvider.prototype.auth = function() {
      var ref, self;
      if (!this._needAuth && this._tokenVerified) {
        return this._getCurrentUserData();
      } else if (!this._tokenVerified && ((ref = this._session) != null ? ref.token : void 0)) {
        self = this;
        return this._verifyToken().then(function() {
          return self._getCurrentUserData();
        }, function() {
          return self.reauth(false);
        });
      } else {
        return this._tokenRequest(this._baseScope, false);
      }
    };

    BaseProvider.prototype.reauth = function(withDialog) {
      var scope;
      scope = this._session && this._session.scopeMask ? this._session.scopeMask : this._baseScope;
      return this._tokenRequest(scope, withDialog ? true : false);
    };

    BaseProvider.prototype.grantScopeRequest = function(bits) {
      return this._tokenRequest(this._addScopeBits(bits), true);
    };

    BaseProvider.prototype._tokenRequest = function(scopeMask, force) {
      var data;
      data = this._getTokenRequestData(scopeMask, force);
      return this._showPopup(this._tokenUri, data).then(this._afterAuth, this._onError).then(this._getCurrentUserData);
    };

    BaseProvider.prototype._getTokenRequestData = function(scopeMask, force) {
      var data;
      data = {
        scope: this._scopeMaskToList(scopeMask).join(this._scopeDelimiter),
        client_id: this._appId,
        display: this._tokenDisplay,
        response_type: 'token',
        redirect_uri: this._tokenRedirectUri
      };
      data = $.extend(data, this._addTokenData);
      if (force) {
        data = $.extend(data, this._forceTokenData);
      }
      return data;
    };

    BaseProvider.prototype._getCurrentUserData = function() {
      var deferred, ref;
      deferred = $.Deferred();
      if ((this._userData != null)) {
        if (!((ref = this._session) != null ? ref.user_id : void 0)) {
          this._session.user_id = this._userData.user_id;
          this._session.email = this._userData.email;
        }
        deferred.resolve({
          status: 'success',
          user: this._userData
        });
        return deferred;
      }
      this._loadUserData(deferred);
      return deferred;
    };

    BaseProvider.prototype.getFriendIds = function() {
      return this.auth().then(this._getFriendIds);
    };

    BaseProvider.prototype._getFriendIds = function() {
      var deferred;
      deferred = $.Deferred();
      this._loadFriendIds(deferred);
      return deferred;
    };

    BaseProvider.prototype._storeUserData = function(userData) {
      this._session.user_id = userData.user_id;
      this._session.email = userData.email;
      this._saveSessionData(this._session);
      return this._userData = userData;
    };

    BaseProvider.prototype._showPopup = function(uri, data) {
      var child, deferred, self, url;
      self = this;
      deferred = $.Deferred();
      url = uri + (uri.indexOf('?') !== -1 ? '&' : '?') + $.param(data);
      child = window.open(url, '_blank');
      if (child && window.isCordova === false) {
        window.addEventListener('message', function(event) {
          if ((self != null ? self.__popupIntervalId : void 0)) {
            clearInterval(self.__popupIntervalId);
          }
          return deferred.resolve((event != null ? event.data : void 0) ? event.data : null);
        });
        self.__popupIntervalId = window.setInterval(function() {
          if (child.closed === true) {
            clearInterval(self.__popupIntervalId);
            if (deferred.state() === 'pending') {
              return deferred.reject(null);
            }
          }
        }, 1000);
      } else if (child && window.isCordova === true) {
        child.addEventListener('loadstop', function(event) {
          var currentHost;
          currentHost = self._getHostFromUrl(event.url);
          if (currentHost !== self._serverHost) {
            return true;
          }
          return child.executeScript({
            code: "localStorage.getItem('postMessage')"
          }, function(data) {
            var error, message;
            message = data[0];
            try {
              message = JSON.parse(message);
            } catch (_error) {
              error = _error;
              message = null;
            }
            deferred.resolve(message);
            return child.close();
          });
        });
        child.addEventListener('exit', function(event) {
          if (deferred.state() === 'pending') {
            return child.executeScript({
              code: "localStorage.getItem('postMessage')"
            }, function(data) {
              var error, message;
              message = data[0];
              try {
                message = JSON.parse(message);
              } catch (_error) {
                error = _error;
                message = null;
              }
              if (message) {
                return deferred.resolve(message);
              } else {
                return deferred.reject(null);
              }
            });
          }
        });
      } else {
        deferred.reject(null);
      }
      return deferred;
    };

    BaseProvider.prototype._methodCall = function(method, data) {
      return this._prepareRequest(this._methodUri + method, data);
    };

    BaseProvider.prototype._prepareRequest = function(url, data, method) {
      return $.ajax({
        url: url + '?' + $.param(data)
      });
    };

    BaseProvider.prototype._jsonpCall = function(method, data) {
      return $.ajax({
        url: this._methodUri + method + '?' + $.param(data),
        dataType: 'jsonp'
      });
    };

    BaseProvider.prototype._verifyToken = function() {
      var deferred;
      deferred = $.Deferred();
      this._loadGrantedScope(deferred);
      return deferred;
    };

    BaseProvider.prototype._afterAuth = function(data) {
      var deferred;
      this._storeAuthData(data);
      deferred = $.Deferred();
      this._loadGrantedScope(deferred);
      return deferred;
    };

    BaseProvider.prototype._storeAuthData = function(data) {
      var storageData;
      if (data && (data != null ? data.access_token : void 0)) {
        storageData = {
          token: data.access_token,
          refresh_token: data.refresh_token || null,
          expires: Math.floor(Date.now() / 1000) + data.expires_in,
          user_id: (data != null ? data.user_id : void 0) ? data.user_id : null,
          email: (data != null ? data.email : void 0) ? data.email : null
        };
        this._needAuth = false;
        this._token = data.access_token;
        this._session = storageData;
        localStorage.setItem(this._storageKey, JSON.stringify(storageData));
        return storageData;
      }
      return null;
    };

    BaseProvider.prototype._getXHRErrorHandler = function(deferred) {
      var errorFun;
      errorFun = this._genericXHRErrorHandler;
      return function(data) {
        return deferred.reject(errorFun(data));
      };
    };

    BaseProvider.prototype._genericXHRErrorHandler = function(xhr) {
      var data, e, resultData;
      if ((xhr != null) && (xhr.responseText != null)) {
        try {
          data = JSON.parse(xhr.responseText);
        } catch (_error) {
          e = _error;
          data = {};
        }
      } else {
        data = {};
      }
      resultData = {
        status: 'error'
      };
      if (xhr != null) {
        resultData.httpCode = xhr.statusCode();
      }
      if ((data != null) && (data != null ? data.error : void 0)) {
        this._setError(data.error.code, data.error.message);
      } else {
        this._setError('FATAL_ERROR', 'XHR error');
      }
      resultData.error = this.getError();
      return resultData;
    };

    BaseProvider.prototype._onError = function(data) {
      if ((data != null) && (data != null ? data.error_code : void 0)) {
        this._setError(data.error_code, data.error_msg);
      } else {
        this._setError('FATAL_ERROR', 'Fatal error');
      }
      return {
        status: 'error',
        error: this.getError()
      };
    };

    BaseProvider.prototype._setError = function(code, msg) {
      this._errCode = code;
      return this._errMsg = msg;
    };

    BaseProvider.prototype.getError = function() {
      return {
        code: this._errCode,
        msg: this._errMsg
      };
    };

    BaseProvider.prototype._saveSessionData = function(data) {
      return localStorage.setItem(this._storageKey, JSON.stringify(data));
    };

    BaseProvider.prototype._loadSessionData = function() {
      var data;
      data = localStorage.getItem(this._storageKey);
      if (data) {
        return JSON.parse(data);
      } else {
        return {};
      }
    };

    BaseProvider.prototype._scopeListToMask = function(list) {
      var bit, mask, ref, scope;
      mask = 0;
      if (!(list != null)) {
        return mask;
      }
      ref = this._scopeMapping;
      for (bit in ref) {
        scope = ref[bit];
        if (list.indexOf(scope) !== -1) {
          mask = mask | parseInt(bit, 10);
        }
      }
      return mask;
    };

    BaseProvider.prototype._scopeMaskToList = function(mask) {
      var bit, list, ref, scope;
      list = [];
      ref = this._scopeMapping;
      for (bit in ref) {
        scope = ref[bit];
        if ((mask & parseInt(bit, 10)) && list.indexOf(scope) === -1) {
          list.push(scope);
        }
      }
      return list;
    };

    BaseProvider.prototype._checkScopeGranted = function(scopeBits) {
      var ref;
      if ((this._session != null) && ((ref = this._session) != null ? ref.scopeMask : void 0)) {
        return !!(this._session.scopeMask & scopeBits);
      } else {
        return false;
      }
    };

    BaseProvider.prototype._addScopeBits = function(scopeBits) {
      var ref;
      if ((this._session != null) && ((ref = this._session) != null ? ref.scopeMask : void 0)) {
        return this._session.scopeMask | scopeBits;
      } else {
        return scopeBits;
      }
    };

    BaseProvider.prototype._getHostFromUrl = function(url) {
      var a;
      a = document.createElement('a');
      a.href = url;
      return a.hostname;
    };

    return BaseProvider;

  })();
  return BaseProvider;
});
