var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['underscore', 'app', 'views/item-view', 'text!templates/blueprints/logotypes/create.tmpl', 'views/components/file-uploader'], function(_, app, ItemView, template, fileUploader) {
  var BlueprintsCreateView;
  BlueprintsCreateView = (function(superClass) {
    extend(BlueprintsCreateView, superClass);

    function BlueprintsCreateView() {
      this._preprocessConfig = bind(this._preprocessConfig, this);
      return BlueprintsCreateView.__super__.constructor.apply(this, arguments);
    }

    BlueprintsCreateView.prototype.template = template;

    BlueprintsCreateView.prototype.className = 'no-bg-image';

    BlueprintsCreateView.prototype.data = {
      config: 'blueprints.logo.get_config'
    };

    BlueprintsCreateView.prototype.events = {
      'click .js-create': 'onCreateBtnClick',
      'click .js-select-image': 'onImageBtnClick',
      'click .js-select-group': 'onGroupSelectBtnClick',
      'change .js-user-data': 'onUserDataChange'
    };

    BlueprintsCreateView.prototype.initialize = function(options) {
      app.hideLoader();
      BlueprintsCreateView.__super__.initialize.call(this, options);
      this._reset();
      return this.on('data:loaded', this._preprocessConfig);
    };

    BlueprintsCreateView.prototype.onDestroy = function() {
      return this._reset();
    };

    BlueprintsCreateView.prototype._reset = function() {
      this._groups = null;
      this._minPrice = 0;
      this._minComission = 0;
      this._comission = 0;
      this._createPrice = 0;
      this._createCurrency = 1;
      this._createMaxQuantity = 0;
      if (this._file) {
        this._file.destroy();
      }
      this._file = null;
      return this._userData = {};
    };

    BlueprintsCreateView.prototype.onUserDataChange = function(event) {
      this._updateUserData();
      return this.render();
    };

    BlueprintsCreateView.prototype.onGroupSelectBtnClick = function(event) {
      var group;
      this._updateUserData();
      group = event.currentTarget.dataset.group;
      group = name.trim();
      return app.screens.modal('common/simple-select-dialog', {
        values: this._groups,
        selected: group,
        callback: (function(_this) {
          return function(group) {
            _this._userData.category = group;
            return _this.render();
          };
        })(this)
      });
    };

    BlueprintsCreateView.prototype.onImageBtnClick = function() {
      var imageElement, self;
      this._updateUserData();
      imageElement = this.el.querySelector('#js-logo-image');
      self = this;
      return app.fileService.selectFile().then(function(file) {
        if (self._file) {
          self._file.destroy();
        }
        self._file = file;
        self._userData.imageUrl = file.getImageDataUrl();
        if (!self._userData.imageUrl) {
          self._userData.imageUrl = file.getUrl();
        }
        return self.render();
      });
    };

    BlueprintsCreateView.prototype.onCreateBtnClick = function() {
      var data, errors, self;
      self = this;
      this._updateUserData();
      errors = this.validateUserValues();
      if (errors.length) {
        this.displayErrors(errors);
        return;
      }
      data = {};
      data.price = this._userData.price;
      data.quantity = this._userData.quantity;
      data.category = this._userData.category;
      data.code = this._userData.code || '';
      return app.fileService.upload(this._file, '/blueprints/logo/create.json', {
        fields: data,
        fileName: 'image',
        responseType: 'json'
      }).then(function(data) {
        var userParams;
        if (self._createCurrency === 1) {
          userParams = {
            vg_balance: -self._createPrice
          };
        } else {
          userParams = {
            vd_balance: -self._createPrice
          };
        }
        app.shortInfo.changeUserParams(userParams);
        app.shortInfo.refresh();
        app.screens.main('blueprints/logotypes/user-list');
        return app.screens.modal('common/success-popup', {
          msg: app.tr('Logotype created')
        });
      }, function(data) {
        var message;
        message = app.tr('Server error');
        if (data && data.setFlash && data.setFlash[0].msg) {
          message = data.setFlash[0].msg;
        }
        return app.screens.modal('common/info-popup', {
          msg: message
        });
      });
    };

    BlueprintsCreateView.prototype.validateUserValues = function() {
      var categories, errors, maxFileSize, regExp;
      errors = [];
      if (!this._file || !this._file.isImage()) {
        errors.push(app.tr('Select valid image for logotype'));
      }
      maxFileSize = 1048576;
      if (this._file && this._file.getSize() > maxFileSize) {
        errors.push(app.tr('Image file is too big'));
      }
      categories = Object.keys(this._groups);
      if (!this._userData.category || categories.indexOf(this._userData.category) === -1) {
        errors.push(app.tr('Select valid category for logotype'));
      }
      if (this._userData.price < this._minPrice) {
        errors.push(app.tr('Set valid price for logotype'));
      }
      if (this._userData.quantity < 0 || (this._createMaxQuantity > 0 && (this._userData.quantity > this._createMaxQuantity || this._userData.quantity < 1))) {
        errors.push(app.tr('Set valid quantity for logotype'));
      }
      if (this._userData.code) {
        if (this._userData.code.length < 4) {
          errors.push(app.tr('Code must be at least 4 characters long'));
        }
        if (this._userData.code.length > 8) {
          errors.push(app.tr('Code must be less than 8 characters long'));
        }
        regExp = new RegExp('[^A-z0-9]+');
        if (regExp.test(this._userData.code)) {
          errors.push(app.tr('Code must contain only digits and latin characters'));
        }
      }
      return errors;
    };

    BlueprintsCreateView.prototype.displayErrors = function(errors) {
      if (errors.length > 0) {
        return app.screens.modal('common/info-popup', {
          msg: errors[0]
        });
      }
    };

    BlueprintsCreateView.prototype._updateUserData = function() {
      var form, price, quantity, regExp;
      form = this.el.querySelector('#js-logo-form');
      if (!form) {
        return;
      }
      regExp = new RegExp('[^0-9\\.\\-]+');
      price = form.elements['price'].value;
      if (regExp.test(price)) {
        price = 0;
      }
      quantity = form.elements['quantity'].value;
      if (regExp.test(quantity) || quantity === "") {
        quantity = -1;
      }
      quantity = parseInt(quantity, 10);
      this._userData.price = Number(price) || 0;
      this._userData.quantity = quantity < 0 ? -1 : quantity;
      return this._userData.code = form.elements['code'].value;
    };

    BlueprintsCreateView.prototype.serializeData = function() {
      var data;
      data = BlueprintsCreateView.__super__.serializeData.call(this);
      data.userData = this._userData;
      data.groups = this._groups;
      data.minPrice = this._minPrice;
      data.comission = this._comission;
      data.minComission = this._minComission;
      data.createPrice = this._createPrice;
      data.createCurrency = this._createCurrency;
      data.createMaxQuantity = this._createMaxQuantity;
      return data;
    };

    BlueprintsCreateView.prototype._preprocessConfig = function(data) {
      if (data.config) {
        this._groups = data.config.categories;
        this._minPrice = Number(data.config.min_price) || 1;
        this._comission = Number(data.config.comission) || 0.3;
        this._minComission = Number(data.config.comission) || 0.01;
        this._createPrice = Number(data.config.create_price) || 0;
        this._createCurrency = Number(data.config.create_currency) || 1;
        return this._createMaxQuantity = Number(data.config.create_max_quantity) || 0;
      }
    };

    return BlueprintsCreateView;

  })(ItemView);
  return BlueprintsCreateView;
});
