var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'underscore', 'text!templates/business/capital.tmpl', 'text!templates/business/blocks/capital-company-one-element.tmpl', 'views/components/currency', 'text!templates/business/blocks/capital-company-element.tmpl', 'views/common/lazy-loading-view'], function(app, _, template, corporationCompanyTemplate, formatCurrency, listTemplate, CommonLazyView) {
  var CorporationCompaniesView;
  CorporationCompaniesView = (function(superClass) {
    extend(CorporationCompaniesView, superClass);

    function CorporationCompaniesView() {
      this._loadData = bind(this._loadData, this);
      return CorporationCompaniesView.__super__.constructor.apply(this, arguments);
    }

    CorporationCompaniesView.prototype.template = template;

    CorporationCompaniesView.prototype.corporationCompanyTemplate = _.template(corporationCompanyTemplate);

    CorporationCompaniesView.prototype.listTemplate = _.template(listTemplate);

    CorporationCompaniesView.prototype.pointer = 'corporationInfo';

    CorporationCompaniesView.prototype.reloadOnScreensBack = true;

    CorporationCompaniesView.prototype.data = {
      corporationInfo: {
        pointer: 'business.corporationInfo',
        corporationId: function() {
          return this.options.corporationId;
        },
        page: 1,
        limit: 50
      },
      corporationDividends: {
        pointer: 'business.corporationDividends',
        corporationId: function() {
          return this.options.corporationId;
        },
        limit: 3
      }
    };

    CorporationCompaniesView.prototype.events = {
      'click .js-goto-company': 'gotoCompany',
      'click .js-goto-management': 'gotoManagement',
      'click .js-goto-corporation-storage': 'gotoCorporationStorage',
      'click .js-goto-shareholders': 'gotoShareholders',
      'click .js-create-company': 'createCompany',
      'click .js-invest': 'invest',
      'click .js-all-dividends': 'showAllDividends',
      'click @ui.close': 'dropdownBehaviour',
      'click .js-back': 'jsBack'
    };

    CorporationCompaniesView.prototype.ui = {
      close: '.dropdown__title'
    };

    CorporationCompaniesView.prototype.loadData = function(showLoader) {
      if (showLoader == null) {
        showLoader = true;
      }
      this.data.corporationInfo.page = 1;
      return CorporationCompaniesView.__super__.loadData.call(this, showLoader, '#js-main-list-content-capital');
    };

    CorporationCompaniesView.prototype.onRefreshBalance = function(balance) {
      var currentVd, currentVg;
      if (balance.vd) {
        currentVd = this._data.corporationInfo.currentCorporation.vd_balance || 0;
        currentVd = this._data.corporationInfo.currentCorporation.vd_balance = +currentVd + balance.vd;
        this.el.querySelector('.js-currency').innerHTML = formatCurrency(+currentVd, void 0, 2);
      }
      if (balance.vg) {
        currentVg = this._data.corporationInfo.currentCorporation.vg_balance || 0;
        currentVg = this._data.corporationInfo.currentCorporation.vg_balance = +currentVg + balance.vg;
        return this.el.querySelector('.js-currency-vg').innerHTML = currentVg;
      }
    };

    CorporationCompaniesView.prototype.jsBack = function() {
      return app.screens.back(null, {
        needReload: false
      });
    };

    CorporationCompaniesView.prototype.onRefreshCompany = function(companyId) {
      app.server.business.corporationInfo({
        corporationId: this.options.corporationId,
        success: (function(_this) {
          return function(data) {
            _this.el.querySelector('.js-currency').innerHTML = formatCurrency(+data.currentCorporation.vd_balance, void 0, 2);
            return _this.el.querySelector('.js-currency-vg').innerHTML = data.currentCorporation.vg_balance;
          };
        })(this),
        error: (function(_this) {
          return function(err) {
            throw err;
          };
        })(this)
      });
      return app.server.business.companyInfo({
        companyId: companyId,
        success: (function(_this) {
          return function(data) {
            var company, companyElement, currentCompanyElement;
            data.company.corporationInfo = _this._data.corporationInfo;
            company = data.company;
            company.tr = app.tr;
            currentCompanyElement = _this.el.querySelector('.js-company-' + companyId);
            companyElement = Marionette.Renderer.render(_this.corporationCompanyTemplate, company);
            return $(currentCompanyElement).replaceWith(companyElement);
          };
        })(this),
        error: (function(_this) {
          return function(err) {
            return console.error(err);
          };
        })(this)
      });
    };

    CorporationCompaniesView.prototype.onCompanyCreated = function() {
      return this._loadData();
    };

    CorporationCompaniesView.prototype.preprocessData = function(data) {
      var dividends, i, len, ref;
      if (data != null) {
        ref = data.corporationDividends.dividends_info;
        for (i = 0, len = ref.length; i < len; i++) {
          dividends = ref[i];
          dividends.CorporationSolution.solution.vd_amount_reserved = this.numberFormat(dividends.CorporationSolution.solution.vd_amount_reserved);
        }
      }
      return data;
    };

    CorporationCompaniesView.prototype.notManagerAlert = function(event) {
      return app.screens.modal('common/info-popup', {
        msg: app.tr("You are not a company's manager")
      });
    };

    CorporationCompaniesView.prototype.gotoCompany = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('corporative_companies/common', {
        companyId: companyId
      });
    };

    CorporationCompaniesView.prototype.gotoManagement = function(event) {
      var corporationId;
      this.lockUI();
      corporationId = this.options.corporationId;
      return app.screens.tab('business/management', {
        corporationId: corporationId
      });
    };

    CorporationCompaniesView.prototype.gotoCorporationStorage = function(event) {
      var corporationId;
      this.lockUI();
      corporationId = this.options.corporationId;
      return app.screens.tab('business/corp-storage', {
        corporationId: corporationId
      });
    };

    CorporationCompaniesView.prototype.gotoShareholders = function(event) {
      var corporationId;
      this.lockUI();
      corporationId = this.options.corporationId;
      return app.screens.tab('business/shareholders', {
        corporationId: corporationId
      });
    };

    CorporationCompaniesView.prototype.createCompany = function(event) {
      var corporationId;
      this.lockUI();
      corporationId = this.options.corporationId;
      return app.screens.dialog('business/create-company', {
        corporationId: corporationId
      });
    };

    CorporationCompaniesView.prototype.invest = function(event) {
      var corporationId, vd_balance, vg_balance;
      this.lockUI();
      corporationId = this.options.corporationId;
      vd_balance = this._data.corporationInfo.currentCorporation.vd_balance;
      vg_balance = this._data.corporationInfo.currentCorporation.vg_balance;
      return app.screens.dialog('business/corp-invest', {
        corporationId: corporationId,
        vd_balance: vd_balance,
        vg_balance: vg_balance
      });
    };

    CorporationCompaniesView.prototype.showAllDividends = function(event) {
      var corporationId;
      this.lockUI();
      corporationId = this.options.corporationId;
      return app.screens.dialog('business/corporation-dividends', {
        corporationId: corporationId,
        limit: 0,
        corporation: {
          name: this._data.corporationInfo.currentCorporation.name
        }
      });
    };

    CorporationCompaniesView.prototype.dropdownBehaviour = function(e) {
      var dropdown;
      dropdown = $(e.currentTarget).closest('.dropdown');
      dropdown.toggleClass('is-active');
      dropdown.find('.dropdown__title icon').toggleClass('o-icon--down-arrow o-icon--up-arrow');
      return dropdown.find('.dropdown__details').toggle();
    };

    CorporationCompaniesView.prototype.onDomRefresh = function() {
      return [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        if (el.dataset.precision) {
          return el.innerHTML = formatCurrency(el.innerHTML, void 0, el.dataset.precision);
        } else {
          return el.innerHTML = formatCurrency(el.innerHTML);
        }
      });
    };

    CorporationCompaniesView.prototype._loadData = function(page) {
      var deferred, self;
      deferred = $.Deferred();
      self = this;
      app.server.business.corporationInfo({
        page: page || 1,
        limit: this.data.corporationInfo.limit,
        corporationId: this.options.corporationId,
        success: (function(_this) {
          return function(data) {
            var ref, ref1, ref2, ref3, resultData;
            resultData = data;
            resultData.page = page;
            resultData.hasNext = ((ref = data.paging) != null ? (ref1 = ref.Company) != null ? ref1.nextPage : void 0 : void 0) || false;
            self.hasNext = ((ref2 = data.paging) != null ? (ref3 = ref2.Company) != null ? ref3.nextPage : void 0 : void 0) || false;
            _this.data.corporationInfo.page++;
            return deferred.resolve(resultData);
          };
        })(this),
        error: function() {
          return deferred.resolve({
            companies: [],
            page: 1,
            hasNext: false
          });
        }
      });
      return deferred;
    };

    return CorporationCompaniesView;

  })(CommonLazyView);
  return CorporationCompaniesView;
});
