define(['app', 'views/item-view', 'underscore', 'text!templates/business/corp-invest.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    investAmount: {
      vd: 0,
      vg: 0
    },
    events: {
      'click .o-switcher': 'changeCurrency',
      'change .js-amount': 'changeAmount',
      'click .js-apply': 'addFunds'
    },
    changeCurrency: function(event) {
      $(event.currentTarget).toggleClass('o-switcher--active');
      this.$el.find('.js-currency').toggleClass('o-icon--vdollar o-icon--vgold');
      return this.$el.find('.js-apply').toggleClass('c-btn--green c-btn--yellow');
    },
    changeAmount: function(event) {
      var amount;
      amount = this.$el.find('.js-amount').val();
      if (isNaN(amount)) {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("The amount may consist only of digits")
        });
      } else {
        return this.$el.find('.saved-amount').html(amount);
      }
    },
    jsBack: function() {
      return app.screens.back(null, {
        needReload: false
      }, 'refresh:balance', this.investAmount);
    },
    addFunds: function(event) {
      var amount, corporationId, currency, loader;
      this.lockUI();
      this.investAmount = {
        vd: 0,
        vg: 0
      };
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Adding'));
      amount = this.$el.find('.js-amount').val();
      corporationId = this.options.corporationId;
      currency = this.$el.find('.o-switcher').hasClass('o-switcher--active') ? 'vgold' : 'vdollars';
      if (amount === "") {
        app.screens.modal('common/info-popup', {
          msg: app.tr("Specify the sum")
        });
        return loader.remove();
      } else if (isNaN(amount)) {
        app.screens.modal('common/info-popup', {
          msg: app.tr("The amount can only consist of digits")
        });
        return loader.remove();
      } else if (amount <= 0) {
        app.screens.modal('common/info-popup', {
          msg: app.tr("The amount must be greater than zero")
        });
        return loader.remove();
      } else {
        if (currency === "vdollars") {
          return app.server.business.corporationInvestVd({
            corporationId: corporationId,
            amount: amount,
            success: (function(_this) {
              return function(data) {
                _this.investAmount.vd += +amount;
                loader.remove();
                return _this.unlockUI();
              };
            })(this),
            error: function(data) {
              loader.remove();
              return app.screens.modal('common/info-popup', {
                msg: data.setFlash[0].msg || app.tr("Something went wrong")
              });
            }
          });
        } else {
          return app.server.business.corporationInvestVg({
            corporationId: corporationId,
            amount: amount,
            success: (function(_this) {
              return function(data) {
                _this.investAmount.vg += +amount;
                loader.remove();
                return _this.unlockUI();
              };
            })(this),
            error: function(data) {
              loader.remove();
              return app.screens.modal('common/info-popup', {
                msg: data.setFlash[0].msg || app.tr("Something went wrong")
              });
            }
          });
        }
      }
    }
  });
});
