define(['app', 'views/item-view', 'underscore', 'text!templates/inventory/select.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    required: ['itemId'],
    data: {
      item: {
        pointer: 'corporation.corporationItemInfo',
        itemId: function() {
          return this.options.itemId;
        }
      }
    },
    events: {
      'click .js-use': 'useItem',
      'click .js-goto-choose-city': 'gotoChooseCity',
      'click .js-goto-choose-type': 'gotoChooseType'
    },
    useItem: function(event) {
      var corporationId, itemId;
      this.lockUI();
      itemId = this.options.itemId;
      corporationId = this.options.corporationId;
      return app.server.corporation.use({
        itemId: itemId,
        cityId: this.cityId,
        typeId: this.typeId,
        corporationId: corporationId,
        success: (function(_this) {
          return function(data) {
            return app.screens.close(function() {
              return app.screens.modal('common/success-popup', {
                msg: data.setFlash[0].msg || app.tr("Item received")
              });
            });
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            return app.screens.close(function() {
              return app.screens.modal('common/info-popup', {
                msg: data.setFlash[0].msg || app.tr("Something went wrong!")
              });
            });
          };
        })(this)
      });
    },
    gotoChooseCity: function(event) {
      this.lockUI();
      return this.app.screens.modal('inventory/choose-city', {
        callback: (function(_this) {
          return function(arg) {
            _this.cityId = arg.cityId, _this.cityName = arg.cityName;
            return _this.updateCityInfo();
          };
        })(this)
      });
    },
    updateCityInfo: function() {
      return this.el.querySelector('.js-city').innerHTML = this.cityName;
    },
    gotoChooseType: function(event) {
      var corporationId, itemId;
      this.lockUI();
      itemId = this.options.itemId;
      corporationId = this.options.corporationId;
      return this.app.screens.modal('business/corp-choose-type', {
        itemId: itemId,
        corporation: true,
        callback: (function(_this) {
          return function(arg) {
            _this.typeId = arg.typeId, _this.typeName = arg.typeName;
            return _this.updateTypeInfo();
          };
        })(this)
      });
    },
    updateTypeInfo: function() {
      return this.el.querySelector('.js-type').innerHTML = this.typeName;
    }
  });
});
