var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/business/create-corporation.tmpl', 'views/components/currency', 'views/components/file-uploader'], function(app, ItemView, _, template, formatCurrency, fileUploader) {
  var CreateCorporationView;
  CreateCorporationView = (function(superClass) {
    extend(CreateCorporationView, superClass);

    function CreateCorporationView() {
      this.selectAndUploadImage = bind(this.selectAndUploadImage, this);
      return CreateCorporationView.__super__.constructor.apply(this, arguments);
    }

    CreateCorporationView.prototype.template = template;

    CreateCorporationView.prototype.className = 'no-bg-image';

    CreateCorporationView.prototype.events = {
      'click .js-create-company': 'createCompany',
      'click .js-select-image': 'selectAndUploadImage'
    };

    CreateCorporationView.prototype.data = {
      business: 'business.corporationOpenPrice'
    };

    CreateCorporationView.prototype.onDomRefresh = function() {
      var ref;
      console.log('on dom refresh');
      [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        return el.innerHTML = formatCurrency(el.innerHTML);
      });
      if (((ref = this._data) != null ? ref.business : void 0) != null) {
        return $('#party-logo').data("src", app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/corporations/' + this._data.business.new_corporation_info.default_picture);
      }
    };

    CreateCorporationView.prototype.createCompany = function(event) {
      var logo, name_en, name_ru, self;
      this.lockUI();
      logo = this.$el.find('#party-logo').data('src');
      name_ru = this.$el.find('.name_ru').val();
      name_en = this.$el.find('.name_en').val();
      self = this;
      if (name_ru === "" || name_en === "") {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("All fields are required")
        });
      } else {
        return app.server.business.createCorporation({
          name_ru: name_ru,
          name_en: name_en,
          logo: logo,
          success: (function(_this) {
            return function(data) {
              var price;
              if (app.gaEnabled) {
                price = self.$el.find('.js-currency').val();
                app.ga('send', 'event', 'corporation', 'vgoldbuy', 'create_corporation', price);
              }
              app.screens.back(null, {
                needReload: false
              }, 'corporation:created');
              return app.screens.modal('common/success-popup', {
                msg: data.setFlash[0].msg || app.tr("Corporation was created")
              });
            };
          })(this),
          error: function(data) {
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          }
        });
      }
    };

    CreateCorporationView.prototype.selectAndUploadImage = function(event) {
      var uploadPhoto, uploadUrl;
      uploadPhoto = function(imageURI) {
        var ft, options, to;
        options = new FileUploadOptions;
        options.fileKey = 'file';
        options.fileName = imageURI.substr(imageURI.lastIndexOf('/') + 1);
        options.mimeType = 'image/jpeg';
        options.chunkedMode = false;
        to = encodeURI(app.config.serverURLProtocol + '://' + app.config.serverURL + '/corporations/upload.json');
        ft = new FileTransfer;
        return ft.upload(imageURI, to, function(r) {
          var json;
          json = JSON.parse(r.response);
          if ((json != null ? json.image : void 0) != null) {
            $('#party-logo').attr('src', app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/corporations/' + json.image);
            return $('#party-logo').data("src", app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/corporations/' + json.image);
          }
        }, function(error) {
          return app.screens.modal('common/info-popup', {
            msg: app.tr("Something went wrong")
          });
        }, options);
      };
      if (navigator.camera != null) {
        return navigator.camera.getPicture(uploadPhoto, (function(message) {
          return app.screens.modal('common/info-popup', {
            msg: app.tr("Image is not chosen")
          });
        }), {
          quality: 50,
          destinationType: navigator.camera.DestinationType.FILE_URI,
          sourceType: navigator.camera.PictureSourceType.PHOTOLIBRARY
        });
      } else {
        uploadUrl = encodeURI(app.config.serverURLProtocol + '://' + app.config.serverURL + '/corporations/upload.json');
        return fileUploader('.js-file-select', uploadUrl, function(error, data) {
          var json;
          if (error) {
            return app.screens.modal('common/info-popup', {
              msg: app.tr("Something went wrong")
            });
          } else {
            json = JSON.parse(data);
            if ((json != null ? json.image : void 0) != null) {
              $('#party-logo').attr('src', app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/corporations/' + json.image);
              return $('#party-logo').data("src", app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/corporations/' + json.image);
            }
          }
        });
      }
    };

    return CreateCorporationView;

  })(ItemView);
  return CreateCorporationView;
});
