define(['app', 'views/item-view', 'underscore', 'text!templates/business/employees-list.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    data: {
      сompanyWorkersList: {
        pointer: 'business.сompanyWorkersList',
        companyId: function() {
          return this.options.companyId;
        }
      }
    },
    events: {
      'click .js-delete-worker': 'deleteWorker',
      'click .js-delete-foreign-worker': 'deleteForeignWorker'
    },
    deleteWorker: function(event) {
      var thisView, workerId;
      this.lockUI();
      workerId = event.currentTarget.getAttribute('data-worker-id');
      thisView = this;
      return app.server.business.deleteWorker({
        workerId: event.currentTarget.getAttribute('data-worker-id'),
        success: function(data) {
          thisView.removeWorkerFromView(workerId);
          return thisView.unlockUI();
        },
        error: function(data) {
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        },
        doesNotReload: true
      });
    },
    removeWorkerFromView: function(workerId) {
      var index, workers;
      workerId = parseInt(workerId);
      workers = this._data.сompanyWorkersList.currentCompany.CompanyWorker;
      index = _.findIndex(workers, {
        id: workerId
      });
      if (index !== -1) {
        workers.splice(index, 1);
        return this.$el.find('.worker_' + workerId).remove();
      }
    },
    removeForeignWorkerFromView: function(workerId) {
      var index, workerEl, workers;
      workerId = parseInt(workerId);
      workers = this._data.сompanyWorkersList.currentCompany.UserForeignWorker;
      index = _.findIndex(workers, {
        id: workerId
      });
      if (index !== -1) {
        workers.splice(index, 1);
        workerEl = this.el.querySelector('#foreign_worker_' + workerId);
        if (workerEl) {
          return workerEl.parentNode.removeChild(workerEl);
        }
      }
    },
    hideForeignWorker: function(workerId) {
      var workerEl;
      workerId = parseInt(workerId);
      workerEl = this.el.querySelector('#foreign_worker_' + workerId);
      if (workerEl) {
        return workerEl.classList.add('is-hidden');
      }
    },
    showForeginWorker: function(workerId) {
      var workerEl;
      workerId = parseInt(workerId);
      workerEl = this.el.querySelector('#foreign_worker_' + workerId);
      if (workerEl) {
        return workerEl.classList.remove('is-hidden');
      }
    },
    deleteForeignWorker: function(event) {
      var companyId, thisView, workerId, workerTypeId;
      companyId = this.options.companyId;
      thisView = this;
      workerId = event.currentTarget.getAttribute('data-worker-id');
      workerTypeId = event.currentTarget.getAttribute('data-foreign-worker-type-id');
      this.lockUI();
      return app.server.business.deleteForeignWorker({
        companyId: companyId,
        workerTypeId: workerTypeId,
        workerId: workerId,
        quantity: 1,
        success: function(data) {
          thisView.unlockUI();
          if (app.gaEnabled) {
            thisView.sendForeginWorkerSallaryStats(companyId, workerId, workerTypeId);
          }
          return thisView.removeForeignWorkerFromView(workerId);
        },
        error: function(data) {
          thisView.unlockUI();
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        },
        doesNotReload: true
      });
    },
    sendForeginWorkerSallaryStats: function(companyId, workerId, workerTypeId) {
      return app.server.business.ForeignWorkerTotalSallary({
        companyId: companyId,
        workerTypeId: workerTypeId,
        workerId: workerId,
        success: function(data) {
          return app.ga('send', 'event', 'business', 'fire_npc', 'company ' + companyId, parseInt(data.sum_spent));
        },
        error: function(data) {
          if (app.config.debug) {
            return console.error(new Error("Failed to send sallary stats for " + workerId));
          }
        },
        doesNotReload: true
      });
    }
  });
});
