define(['app', 'views/item-view', 'underscore', 'text!templates/business/recruitment.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    events: {
      'click .js-save-vacancy': 'saveVacancy',
      'click .o-switcher-small': 'toggleSwither'
    },
    data: {
      companyWorkersInfo: {
        pointer: 'business.сompanyWorkersInfo',
        companyId: function() {
          return this.options.companyId;
        }
      },
      companyForeignWorkersMaxSalary: {
        pointer: 'business.companyForeignWorkersMaxSalary',
        companyId: function() {
          return this.options.companyId;
        }
      }
    },
    renderRecommendations: function() {
      var cityMinSalary, companyForeignWorkersMaxSalary, recommendedSalary, recommendedSalaryElement, salary, salaryInput, salaryInputStyleClass, salaryTipIcon, salaryWarningElement, salaryWarningStyleClass, warningIconStyleClass;
      recommendedSalaryElement = document.querySelector('.js-recommended-salary');
      salaryWarningElement = document.querySelector('.js-salary-warning');
      salaryTipIcon = document.querySelector('.js-warning-icon');
      salaryInput = document.querySelector('.js-salary');
      salary = +salaryInput.value || 0;
      salaryInputStyleClass = 'c-input-warning';
      salaryWarningStyleClass = 'u-c--red';
      warningIconStyleClass = 'c-icon-info_red';
      companyForeignWorkersMaxSalary = this._data.companyForeignWorkersMaxSalary ? this._data.companyForeignWorkersMaxSalary.salary : 0;
      cityMinSalary = this._data.companyWorkersInfo && +this._data.companyWorkersInfo.min_salary;
      if (cityMinSalary > +companyForeignWorkersMaxSalary) {
        recommendedSalary = this._data.companyWorkersInfo.min_salary;
      } else {
        recommendedSalary = companyForeignWorkersMaxSalary;
      }
      recommendedSalaryElement.innerHTML = recommendedSalary;
      if (salary > +recommendedSalary) {
        salaryInput.classList.add(salaryInputStyleClass);
        salaryWarningElement.classList.add(salaryWarningStyleClass);
        return salaryTipIcon.classList.add(warningIconStyleClass);
      } else {
        salaryInput.classList.remove(salaryInputStyleClass);
        salaryWarningElement.classList.remove(salaryWarningStyleClass);
        return salaryTipIcon.classList.remove(warningIconStyleClass);
      }
    },
    toggleSwither: function(event) {
      if (this.$el.find('.js-set-state span').text() === app.tr('Recruitment open')) {
        this.$el.find('.js-set-state span').text(app.tr('There is no recruitment'));
      } else {
        this.$el.find('.js-set-state').text(app.tr('Recruitment open'));
      }
      return $(event.currentTarget).toggleClass('o-switcher-small--active');
    },
    saveVacancy: function(event) {
      var companyId, is_hiring, level, loader, salary;
      this.lockUI();
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Saving'));
      companyId = this.options.companyId;
      salary = document.querySelector('.js-salary').value;
      level = document.querySelector('.js-level').value;
      is_hiring = this.$el.find('.o-switcher-small').hasClass('o-switcher-small--active') ? true : false;
      return app.server.business.saveVacancy({
        companyId: companyId,
        salary: salary,
        level: level,
        is_hiring: is_hiring,
        success: (function(_this) {
          return function(data) {
            _this.unlockUI();
            loader.remove();
            if (is_hiring && app.gaEnabled) {
              return app.ga('send', 'event', 'business', 'hire', salary);
            }
          };
        })(this),
        error: function(data) {
          loader.remove();
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        }
      });
    },
    onDomRefresh: function() {
      var level, levelInput, salary, salaryInput;
      salaryInput = document.querySelector('.js-salary');
      levelInput = document.querySelector('.js-level');
      salary = salaryInput.value;
      level = levelInput.value;
      this.renderRecommendations();
      if (this._data.companyWorkersInfo != null) {
        if (!isNaN(salary)) {
          salary = parseFloat(this._data.companyWorkersInfo.employment.salary).toFixed(2);
          salaryInput.value = salary;
        }
        if (!isNaN(level)) {
          level = this._data.companyWorkersInfo.employment.level;
          levelInput.value = level;
        }
      }
      if ((this._data.companyWorkersInfo != null) && this._data.companyWorkersInfo.employment.is_hiring) {
        document.querySelector('.js-set-state span').innerHTML = app.tr('Recruitment open');
        document.querySelector('.o-switcher-small').classList.add('o-switcher-small--active');
      }
      levelInput.addEventListener('change', (function(_this) {
        return function() {
          level = parseFloat(levelInput.value);
          if (!isNaN(level)) {
            if (level < 1) {
              level = 1;
            }
            levelInput.value = level;
          } else {
            levelInput.value = 1;
          }
          return _this.renderRecommendations();
        };
      })(this));
      return salaryInput.addEventListener('change', (function(_this) {
        return function() {
          salary = parseFloat(salaryInput.value);
          if (!isNaN(salary)) {
            if (salary < 1) {
              salary = 1;
            }
            salaryInput.value = salary.toFixed(2);
          } else {
            salaryInput.value = "0.00";
          }
          return _this.renderRecommendations();
        };
      })(this));
    }
  });
});
