var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/business/storage.tmpl', 'views/components/currency', 'views/components/clusterize-helper', 'views/components/item-type-comparator'], function(app, ItemView, _, template, formatCurrency, ClusterizeHelper, itemTypeComparatorComponent) {
  var BusinessStorage;
  BusinessStorage = (function(superClass) {
    extend(BusinessStorage, superClass);

    function BusinessStorage() {
      this.showReports = bind(this.showReports, this);
      return BusinessStorage.__super__.constructor.apply(this, arguments);
    }

    BusinessStorage.prototype.template = template;

    BusinessStorage.prototype.data = {
      companyInfo: {
        pointer: 'business.companyInfo',
        companyId: function() {
          return this.options.companyId;
        }
      },
      effects: {
        pointer: 'business.company.effects',
        companyId: function() {
          return this.options.companyId;
        }
      },
      storage: {
        pointer: 'business.storage',
        companyId: function() {
          return this.options.companyId;
        }
      },
      inventory: {
        pointer: 'inventory.unequipped'
      },
      items: {
        pointer: 'tradeCenter.lotsCompany',
        companyId: function() {
          return this.options.companyId;
        }
      }
    };

    BusinessStorage.prototype.events = {
      'click .js-goto-employees': 'gotoEmployees',
      'click .js-goto-common': 'gotoCommon',
      'click .js-goto-selling': 'gotoSelling',
      'click .js-goto-buying': 'gotoBuying',
      'click .js-settings': 'settings',
      'click .js-inventory-item': 'transferItemToCompany',
      'click .js-storage-item': 'transferItemToUser',
      'click .js-expand-storage': 'expandStorage',
      'click .js-toggle-effects': 'showEffects',
      'click .js-reports-archive': 'showReports',
      'click .js-generate-report': 'generateReport'
    };

    BusinessStorage.prototype.initialize = function() {
      BusinessStorage.__super__.initialize.apply(this, arguments);
      return this.itemTypeComparatorFactory = itemTypeComparatorComponent.itemTypeComparatorFactory;
    };

    BusinessStorage.prototype.jsBack = function() {
      return app.screens.back(null, {
        needReload: false
      }, 'refresh:company', +this._data.options.companyId);
    };

    BusinessStorage.prototype.settings = function(event) {
      var companyId, menu;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      menu = new Backbone.Collection([
        {
          id: 1,
          title: app.tr('Rename company')
        }, {
          id: 2,
          title: app.tr('Transfer company to corporation')
        }
      ]);
      return app.screens.dialog('common/select-dialog-cb', {
        collection: menu,
        callback: (function(_this) {
          return function(itemId) {
            console.log('itemId', itemId);
            if (itemId === "1") {
              return app.screens.dialog('business/rename-company', {
                companyId: companyId
              });
            } else {
              return app.screens.dialog('business/transfer-company', {
                companyId: companyId
              });
            }
          };
        })(this)
      });
    };

    BusinessStorage.prototype.gotoEmployees = function(event) {
      var companyId;
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('business/employees', {
        companyId: companyId
      });
    };

    BusinessStorage.prototype.gotoCommon = function(event) {
      var companyId;
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('business/common', {
        companyId: companyId
      });
    };

    BusinessStorage.prototype.gotoSelling = function(event) {
      var companyId;
      companyId = event.currentTarget.getAttribute('data-company-id');
      if (this._data.storage.storage.length === 0 && this._data.items.exchanges.length === 0) {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("Your storage is empty")
        });
      } else {
        return app.screens.dialog('trade-center/trade-center-business-sell', {
          companyId: companyId
        });
      }
    };

    BusinessStorage.prototype.gotoBuying = function(event) {
      var companyId;
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('trade-center/trade-center-business', {
        companyId: companyId
      });
    };

    BusinessStorage.prototype.transferItemToCompany = function(event) {
      var companyId, itemId, itemTypeId, number;
      this.lockUI();
      itemTypeId = event.currentTarget.getAttribute('data-itemType-id');
      itemId = event.currentTarget.getAttribute('data-item-id');
      number = event.currentTarget.getAttribute('data-quantity');
      companyId = this.options.companyId;
      return app.screens.modal('business/move-item-to-company-popup', {
        itemId: itemId,
        number: number,
        callback: (function(_this) {
          return function(quantity) {
            return app.server.business.moveItemsToCompany({
              companyId: companyId,
              itemTypeId: itemTypeId,
              quantity: quantity,
              success: function(data) {
                return app.screens.back();
              },
              error: function(data) {
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    };

    BusinessStorage.prototype.transferItemToUser = function(event) {
      var companyId, itemTypeId, number;
      this.lockUI();
      itemTypeId = event.currentTarget.getAttribute('data-itemType-id');
      number = event.currentTarget.getAttribute('data-quantity');
      companyId = this.options.companyId;
      return app.screens.modal('business/move-item-to-user-popup', {
        itemTypeId: itemTypeId,
        number: number,
        callback: (function(_this) {
          return function(quantity) {
            return app.server.business.moveItemsToUser({
              companyId: companyId,
              itemTypeId: itemTypeId,
              quantity: quantity,
              success: function(data) {
                return app.screens.back();
              },
              error: function(data) {
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    };

    BusinessStorage.prototype.expandStorage = function(event) {
      var companyId;
      this.lockUI();
      companyId = this.options.companyId;
      if (this._data.storage.next_storage_level.max_prod != null) {
        return app.screens.modal('common/confirmation-popup', {
          msg: this._data.storage.next_storage_level.confirmation,
          callback: (function(_this) {
            return function() {
              return app.server.business.upStorageLevel({
                companyId: companyId,
                success: function(data) {
                  return app.screens.modal('common/success-popup', {
                    msg: data.setFlash[0].msg || app.tr("Warehouse increased")
                  });
                },
                error: function(data) {
                  return app.screens.modal('common/info-popup', {
                    msg: data.setFlash[0].msg || app.tr("Something went wrong")
                  });
                }
              });
            };
          })(this)
        });
      } else {
        return app.screens.modal('common/info-popup', {
          msg: this._data.storage.next_storage_level.confirmation || app.tr("Something went wrong")
        });
      }
    };

    BusinessStorage.prototype.serializeData = function() {
      var data, itemTypeComparator;
      data = BusinessStorage.__super__.serializeData.apply(this, arguments);
      if (data && (data.inventory != null) && data.inventory.storage.itemsArray.length !== 0) {
        data.inventory.storage.itemsArray.forEach(function(item, index) {
          if (data.inventory.storage.itemTypes != null) {
            return data.inventory.storage.itemsArray[index].ItemType = data.inventory.storage.itemTypes[item.UserItem.item_type_id];
          }
        });
        itemTypeComparator = this.itemTypeComparatorFactory(data.inventory.categorySortWeight);
        data.inventory.storage.itemsArray = data.inventory.storage.itemsArray.sort(itemTypeComparator);
        data.storage.storage = data.storage.storage.sort(itemTypeComparator);
      }
      return data;
    };

    BusinessStorage.prototype.onDomRefresh = function() {
      [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        return el.innerHTML = formatCurrency(el.innerHTML);
      });
      return ClusterizeHelper(this.$el, 30, true);
    };

    BusinessStorage.prototype.showEffects = function(event) {
      var companyId;
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('maria/city-effects', {
        companyId: companyId
      });
    };

    BusinessStorage.prototype.showEffects = function(event) {
      return $('.js-effects').toggle(400).removeClass('u-hide');
    };

    BusinessStorage.prototype.showReports = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('business/company-reports/archive', {
        companyId: companyId,
        companyName: this._data.companyInfo.company.name
      });
    };

    BusinessStorage.prototype.generateReport = function(event) {
      var companyId, msg, reportType;
      companyId = event.currentTarget.getAttribute('data-company-id');
      reportType = event.currentTarget.getAttribute('data-report-type');
      msg = app.tr('Are you sure you generate report?');
      return app.screens.modal('common/confirmation-popup', {
        msg: msg,
        callback: (function(_this) {
          return function() {
            _this.lockUI();
            return app.server.business.generateReport({
              companyId: companyId,
              reportType: reportType,
              success: function(data) {
                return app.screens.modal('business/company-reports/report', {
                  report: data.report
                });
              }
            });
          };
        })(this)
      });
    };

    BusinessStorage.prototype.onRender = function() {
      this.toggleView('.js-reports-switcher', '.js-reports-buttons');
      return BusinessStorage.__super__.onRender.call(this);
    };

    return BusinessStorage;

  })(ItemView);
  return BusinessStorage;
});
