define(['app', 'views/item-view', 'underscore', 'text!templates/business/withdraw.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    events: {
      'click .o-switcher': 'changeCurrency',
      'click .js-apply': 'takeFunds',
      'change .amount': 'changeAmount'
    },
    changeCurrency: function(event) {
      $(event.currentTarget).toggleClass('o-switcher--active');
      this.$el.find('.js-currency').toggleClass('o-icon--vdollar o-icon--vgold');
      return this.$el.find('.js-apply').toggleClass('c-btn--green c-btn--yellow');
    },
    changeAmount: function(event) {
      var amount;
      amount = this.$el.find('.amount').val();
      if (isNaN(amount)) {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("The amount may consist only of digits")
        });
      } else {
        return this.$el.find('.withdraw').html(amount * this.options.withdrawTax / 100);
      }
    },
    takeFunds: function(event) {
      var amount, companyId, currency, loader;
      this.lockUI();
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Withdrawing'));
      amount = this.$el.find('.amount').val();
      companyId = this.options.companyId;
      currency = this.$el.find('.o-switcher').hasClass('o-switcher--active') ? 'vgold' : 'vdollars';
      if (amount === "") {
        app.screens.modal('common/info-popup', {
          msg: app.tr("Specify the sum")
        });
        return loader.remove();
      } else if (isNaN(amount)) {
        app.screens.modal('common/info-popup', {
          msg: app.tr("The amount can only consist of digits")
        });
        return loader.remove();
      } else if (amount <= 0) {
        app.screens.modal('common/info-popup', {
          msg: app.tr("The amount must be greater than zero")
        });
        return loader.remove();
      } else {
        if (currency === "vdollars") {
          return app.server.business.takeFundsVd({
            companyId: companyId,
            amount: amount,
            success: (function(_this) {
              return function(data) {
                loader.remove();
                return _this.unlockUI();
              };
            })(this),
            error: (function(_this) {
              return function(data) {
                loader.remove();
                _this.unlockUI();
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                });
              };
            })(this)
          });
        } else {
          return app.server.business.takeFundsVg({
            companyId: companyId,
            amount: amount,
            success: (function(_this) {
              return function(data) {
                _this.unlockUI();
                return loader.remove();
              };
            })(this),
            error: (function(_this) {
              return function(data) {
                loader.remove();
                _this.unlockUI();
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                });
              };
            })(this)
          });
        }
      }
    }
  });
});
