
/*
	Possible values are positive and negative numbers
	0 - 9999 without formating
	10000 - 999999 K
	1000000 - 999999999 M
	1000000000 - 999999999999 B
	1000000000000 - 999999999999999 T

	Usage example
	=============

	Before:
	<div class="js-currency">2529870</div>
	[].forEach.call @el.querySelectorAll('.js-currency'), (el) ->
		el.innerHTML = formatCurrency(el.innerHTML)

	After:
	<div class="js-currency">25.3K</div>
 */
define([], function() {

  /**
  	 * Function to convert long currency values to their shorter formated representation
  	 * @return {string} Formated value
   */
  return function(value, separator, precision) {
    var abs, d, len, modifiers, numberLength, position;
    if (!value) {
      return false;
    }
    value = value.toString();
    separator = separator != null ? separator : ':';
    precision = precision != null ? precision : 0;
    len = value.length;
    position = value.indexOf(separator) + 1;
    if (position === 0) {
      value = +value;
      abs = Math.abs(value);
      d = Math.pow(10, precision);
      modifiers = (function() {
        switch (false) {
          case !(abs < 10000):
            return [1, ""];
          case !(abs >= 10000 && abs < 1000000):
            return [1000, "K"];
          case !(abs >= 1000000 && abs < 1000000000):
            return [1000 * 1000, "M"];
          case !(abs >= 1000000000 && abs < 1000000000000):
            return [1000 * 1000 * 1000, "B"];
          case !(abs >= 1000000000000):
            return [1000 * 1000 * 1000 * 1000, "T"];
          default:
            return [1, ""];
        }
      })();
      return ((parseInt((value / modifiers[0]) * d)) / d).toFixed(precision) + modifiers[1];
    } else {
      numberLength = len - position;
      if (numberLength <= 4) {
        return value.slice(position);
      }
      if (numberLength >= 5 && numberLength <= 6) {
        return value.slice(position, len - 3) + 'K';
      } else if (numberLength >= 7 && numberLength <= 9) {
        return value.slice(position, len - 6) + 'M';
      } else if (numberLength >= 10 && numberLength <= 12) {
        return value.slice(position, len - 9) + 'B';
      } else if (numberLength >= 13 && numberLength <= 15) {
        return value.slice(position, len - 12) + 'T';
      } else if (numberLength >= 16 && numberLength <= 18) {
        return value.slice(position, len - 15) + 'q';
      } else if (numberLength >= 19 && numberLength <= 21) {
        return value.slice(position, len - 18) + 'Q';
      } else if (numberLength >= 22 && numberLength <= 24) {
        return value.slice(position, len - 21) + 's';
      } else if (numberLength >= 25 && numberLength <= 27) {
        return value.slice(position, len - 24) + 'S';
      } else if (numberLength >= 28 && numberLength <= 29) {
        return value.slice(position, len - 27) + 'O';
      } else if (numberLength >= 30 && numberLength <= 32) {
        return value.slice(position, len - 29) + 'N';
      } else if (numberLength >= 33 && numberLength <= 35) {
        return value.slice(position, len - 32) + 'd';
      } else if (numberLength >= 36 && numberLength <= 38) {
        return value.slice(position, len - 35) + 'U';
      } else if (numberLength >= 39 && numberLength <= 41) {
        return value.slice(position, len - 38) + 'D';
      } else if (numberLength >= 42 && numberLength <= 44) {
        return value.slice(position, len - 41) + '!';
      } else if (numberLength >= 45 && numberLength <= 47) {
        return value.slice(position, len - 44) + '@';
      } else if (numberLength >= 48 && numberLength <= 50) {
        return value.slice(position, len - 47) + '#';
      } else if (numberLength >= 51) {
        return value.slice(position, len - 50) + '$';
      } else {
        return value.slice(position);
      }
    }
  };
});
