
/*
	ParlamentSwipe

	Functionality description
	=============
	1. detect touch on evement
	2. detect touch time and direction
	3. if horizontal direction more than vertical - then run delete func
	4. if vertical directon more than horizontal and touch time before move more than 1000ms touch start dragg

	Usage example
	=============

	Container:
	<div class="js-container"></div>
	$('.js-container').parlamentSwipe()
 */
define(['modernizr'], function(Modernizr) {
  var ParlamentSwipe;
  return ParlamentSwipe = (function() {
    function ParlamentSwipe(el, arg) {
      this.onRemoveElement = arg.onRemoveElement, this.onDrag = arg.onDrag, this.onScroll = arg.onScroll;
      this.el = el;
      this.$el = $(this.el);
      this.init();
    }


    /**
    		 * Function checks if touch events are supported by modernizr detection and initialize proper events
    		 * @param {obj} data - orbitary data can be passed here to extend method functionality
     */

    ParlamentSwipe.prototype.init = function(data) {
      this.swipeValue = 50;
      if (window.navigator.pointerEnabled != null) {
        this.start = 'pointerdown';
        this.move = 'pointermove';
        this.end = 'pointerup';
        this.cancel = 'pointerout';
      } else if (window.navigator.msPointerEnabled != null) {
        this.start = 'MSPointerDown';
        this.move = 'MSPointerMove';
        this.end = 'MSPointerUp';
        this.cancel = 'MSPointerOut';
      } else if (Modernizr.touch) {
        this.start = 'touchstart';
        this.move = 'touchmove';
        this.end = 'touchend';
        this.cancel = 'touchcancel';
      } else {
        this.start = 'mousedown';
        this.move = 'mousemove';
        this.end = 'mouseup';
        this.cancel = 'mouseout';
      }
      this.touchStart = this.touchStart.bind(this);
      this.touchMove = this.touchMove.bind(this);
      this.touchEnd = this.touchEnd.bind(this);
      this.el.addEventListener(this.start, this.touchStart, false);
      this.el.addEventListener(this.move, this.touchMove, false);
      this.el.addEventListener(this.end, this.touchEnd, false);
      this.el.addEventListener(this.cancel, this.touchEnd, false);
      this.transitionProperty = "translate";
      if (Modernizr.csstransforms3d) {
        return this.transitionProperty = 'translate3d';
      }
    };

    ParlamentSwipe.prototype.undelegate = function() {
      this.el.removeEventListener(this.start, this.touchStart, false);
      this.el.removeEventListener(this.move, this.touchMove, false);
      this.el.removeEventListener(this.end, this.touchEnd, false);
      return this.el.removeEventListener(this.cancel, this.touchEnd, false);
    };


    /**
    		 * Prevent default events and stopPropagation if needed
    		 * @param {event}
     */

    ParlamentSwipe.prototype.preventEvents = function(event) {
      event.preventDefault();
      return event.stopPropagation();
    };


    /**
    		 * When the touch begins we want to reset the direction. We also want to track the 
    		 * initial position of the touch as we'll be using that to calculate the delta as
    		 * their finger moves.
    		 * @param {event}
     */

    ParlamentSwipe.prototype.touchStart = function(event) {
      var ref, ref1;
      this.startTime = event.timeStamp;
      this.startX = (ref = event.clientX) != null ? ref : event.touches[0].pageX;
      this.startY = (ref1 = event.clientY) != null ? ref1 : event.touches[0].pageY;
      this.endX = this.startX;
      return this.endY = this.startY;
    };


    /**
    		 * touchMove
    		 * @param {event}
     */

    ParlamentSwipe.prototype.touchMove = function(event) {
      var xdelta, ydelta;
      this.touchTimeOut = event.timeStamp - this.startTime;
      this.endX = event.clientX || event.touches[0].pageX;
      this.endY = event.clientY || event.touches[0].pageY;
      xdelta = Math.abs(this.moveDistanceX());
      ydelta = Math.abs(this.moveDistanceY());
      if (this.touchTimeOut >= 500 && ydelta > xdelta) {
        return this.onDrag();
      } else if (this.touchTimeOut < 500 && Math.abs(ydelta > xdelta)) {
        return this.onScroll();
      } else if (ydelta < xdelta) {
        return this.transition();
      }
    };


    /**
    		 * touchEnd
    		 * @param {event}
     */

    ParlamentSwipe.prototype.touchEnd = function(event) {
      if (this.moveDistanceX() >= this.swipeValue) {
        return this.onRemoveElement();
      }
    };


    /**
    		 * touchCancel
    		 * @param {event}
     */

    ParlamentSwipe.prototype.touchCancel = function(event) {
      if (this.moveDistanceX() >= this.swipeValue) {
        return this.onRemoveElement();
      }
    };


    /**
    		 * Sortable transitions
    		 * @param {event}
     */

    ParlamentSwipe.prototype.sortable = function() {};


    /**
    		 * Transitions from min to max value
    		 * @param {event}
     */

    ParlamentSwipe.prototype.transition = function() {
      if (this.moveDistanceX() >= 0 && this.moveDistanceX() <= this.swipeValue) {
        if (this.transitionProperty === "translate") {
          return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(' + this.moveDistanceX() + 'px, 0)');
        } else {
          return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(' + this.moveDistanceX() + 'px, 0, 0)');
        }
      } else if (this.moveDistanceX() < 0) {
        return this.removeTransition();
      }
    };


    /**
    		 * Remove Transitions
    		 * @param {event}
     */

    ParlamentSwipe.prototype.removeTransition = function() {
      if (this.transitionProperty === "translate") {
        return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(0, 0)');
      } else {
        return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(0, 0, 0)');
      }
    };


    /**
    		 * Drag functionality
    		 * @param {event}
     */

    ParlamentSwipe.prototype.dragItem = function() {
      if (this.transitionProperty === "translate") {
        return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(0, 0)');
      } else {
        return this.$el.css(Modernizr.prefixed('transform'), this.transitionProperty + '(0, 0, 0)');
      }
    };


    /**
    		 * Calculate touch X distance
    		 * @param {event}
     */

    ParlamentSwipe.prototype.moveDistanceX = function() {
      return this.endX - this.startX;
    };


    /**
    		 * Calculate touch Y distance
    		 * @param {event}
     */

    ParlamentSwipe.prototype.moveDistanceY = function() {
      return this.endY - this.startY;
    };

    return ParlamentSwipe;

  })();
});
