var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/business/common.tmpl', 'views/components/currency', 'views/business/company-bots/company-bots'], function(app, ItemView, _, template, formatCurrency, CompanyBots) {
  var CorporationCommonView;
  CorporationCommonView = (function(superClass) {
    extend(CorporationCommonView, superClass);

    function CorporationCommonView() {
      this.showReports = bind(this.showReports, this);
      this.initialize = bind(this.initialize, this);
      return CorporationCommonView.__super__.constructor.apply(this, arguments);
    }

    CorporationCommonView.prototype.template = template;

    CorporationCommonView.prototype.data = {
      companyInfo: {
        pointer: 'business.companyInfo',
        companyId: function() {
          return this.options.companyId;
        }
      },
      info: 'user.info.short',
      effects: {
        pointer: 'business.company.effects',
        companyId: function() {
          return this.options.companyId;
        }
      }
    };

    CorporationCommonView.prototype.events = {
      'click .js-goto-employees': 'gotoEmployees',
      'click .js-goto-storage': 'gotoStorage',
      'click .js-add-funds': 'addFunds',
      'click .js-take-funds': 'takeFunds',
      'click .js-begin-work': 'beginWork',
      'click .js-settings': 'settings',
      'click .js-change-prod': 'changeProd',
      'click .c-btn-group__button': 'changeEnergy',
      'click .js-toggle-effects': 'showEffects',
      'click .js-reports-archive': 'showReports',
      'click .js-generate-report': 'generateReport',
      'click .js-reports-switcher': 'reportsToggle'
    };

    CorporationCommonView.prototype.jsBack = function() {
      return app.screens.back(null, {
        needReload: false
      }, 'refresh:company', +this._data.options.companyId);
    };

    CorporationCommonView.prototype.initialize = function(options1) {
      this.options = options1;
      this.options.activeEnergy = 50;
      return CorporationCommonView.__super__.initialize.call(this, this.options);
    };

    CorporationCommonView.prototype.reportsToggle = function() {
      return $('.js-reports-buttons').toggle().removeClass('u-hide');
    };

    CorporationCommonView.prototype.changeEnergy = function(event) {
      var activeStateClass, btn, buttons, el, i, len;
      btn = event.currentTarget;
      buttons = btn.parentNode.querySelectorAll('.c-btn-group__button');
      activeStateClass = 'c-btn-group__button--active';
      for (i = 0, len = buttons.length; i < len; i++) {
        el = buttons[i];
        if (el.classList) {
          el.classList.remove(activeStateClass);
        } else {
          el.activeStateClass = el.activeStateClass.replace(new RegExp('(^|\\b)' + activeStateClass.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
        }
      }
      if (btn.classList) {
        return btn.classList.add(activeStateClass);
      } else {
        return btn.activeStateClass += ' ' + activeStateClass;
      }
    };

    CorporationCommonView.prototype.addFunds = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('corporative_companies/add-funds', {
        companyId: companyId
      });
    };

    CorporationCommonView.prototype.takeFunds = function(event) {
      var companyId, withdrawTax;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      withdrawTax = event.currentTarget.getAttribute('data-withdraw-tax');
      return app.screens.dialog('corporative_companies/withdraw', {
        companyId: companyId,
        withdrawTax: withdrawTax
      });
    };

    CorporationCommonView.prototype.gotoEmployees = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('corporative_companies/employees', {
        companyId: companyId
      });
    };

    CorporationCommonView.prototype.gotoStorage = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('corporative_companies/storage', {
        companyId: companyId
      });
    };

    CorporationCommonView.prototype.beginWork = function(event) {
      var companyId, energy, loader;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      energy = parseInt(this.el.querySelector('.c-btn-group__button--active .js-energy-value').innerHTML);
      this.options.activeEnergy = energy;
      loader = this.setLoaderOnElement('.js-begin-work', app.tr('Work'));
      return app.server.work.beginWork({
        companyId: companyId,
        energy: energy,
        success: (function(_this) {
          return function(data) {
            var options;
            options = {
              data: data,
              repeat: {
                method: _this.beginWork,
                context: _this,
                event: event
              }
            };
            return _this.listenToOnce(app.vent, 'short-info:dataLoaded', function() {
              loader.remove();
              return app.screens.modal('job/job-done', options);
            });
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            loader.remove();
            _this.unlockUI();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    };

    CorporationCommonView.prototype.settings = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('business/settings', {
        companyId: companyId
      });
    };

    CorporationCommonView.prototype.changeProd = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('business/change-production', {
        companyId: companyId
      });
    };

    CorporationCommonView.prototype.balanceStatusState = function() {
      if (this._data.companyInfo != null) {
        if (this._data.companyInfo.company.current_production.production_status_title === 'Нет средств') {
          this.$('.js-currency-wrap').removeClass('u-bgc--grey-3');
          this.$('.js-currency-wrap').addClass('bgc-red-3');
          this.$('.js-currency-wrap').addClass('c-black');
          this.$('.js-currency-wrap i').removeClass('o-icon--vdollar');
          this.$('.js-currency-wrap i').addClass('o-icon--vdollar-black');
          return this.$('.js-status-text').removeClass('u-hide');
        }
      }
    };

    CorporationCommonView.prototype.onDomRefresh = function() {
      this.balanceStatusState();
      return [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        if (el.dataset.precision) {
          return el.innerHTML = formatCurrency(el.innerHTML, void 0, el.dataset.precision);
        } else {
          return el.innerHTML = formatCurrency(el.innerHTML);
        }
      });
    };

    CorporationCommonView.prototype.showEffects = function(event) {
      return $('.js-effects').toggle().removeClass('u-hide');
    };

    CorporationCommonView.prototype.showReports = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('business/company-reports/archive', {
        companyId: companyId,
        companyName: this._data.companyInfo.company.name
      });
    };

    CorporationCommonView.prototype.generateReport = function(event) {
      var companyId, msg, reportType;
      companyId = event.currentTarget.getAttribute('data-company-id');
      reportType = event.currentTarget.getAttribute('data-report-type');
      msg = app.tr('Are you sure you generate report?');
      return app.screens.modal('common/confirmation-popup', {
        msg: msg,
        callback: (function(_this) {
          return function() {
            _this.lockUI();
            return app.server.business.generateReport({
              companyId: companyId,
              reportType: reportType,
              success: function(data) {
                return app.screens.modal('business/company-reports/report', {
                  report: data.report
                });
              }
            });
          };
        })(this)
      });
    };

    CorporationCommonView.prototype.serializeData = function() {
      var companyBot, companyType, data, ref;
      data = CorporationCommonView.__super__.serializeData.call(this);
      if (data.companyInfo != null) {
        ref = data.companyInfo.companyBots;
        for (companyType in ref) {
          companyBot = ref[companyType];
          if (typeof companyBot === 'string') {
            data.companyInfo.companyBots[companyType] = JSON.parse(companyBot);
          }
        }
      }
      return data;
    };

    CorporationCommonView.prototype.onRender = function() {
      var companyBots, data;
      companyBots = new CompanyBots();
      data = this.mixinTemplateHelpers(this.serializeData({}));
      return companyBots.render('.js-company-bots', data);
    };

    return CorporationCommonView;

  })(ItemView);
  return CorporationCommonView;
});
