define(['app', 'views/item-view', 'underscore', 'text!templates/hospital.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    className: 'hospital-layout',
    template: template,
    data: {
      service: 'hospital.service'
    },
    events: {
      'click .js-heal-full-hp': 'healFullHP',
      'click .js-heal-trauma': 'healTrauma'
    },
    healFullHP: function(event) {
      var healingCost, loader;
      this.lockUI();
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Recover'));
      healingCost = this._data.service.heal_hp_cost.HealCost.heal_hp_price;
      return app.server.hospital.service.healFullHP({
        success: (function(_this) {
          return function(data) {
            if (app.gaEnabled) {
              app.ga('send', 'event', 'clinic', 'healing health', Number(healingCost));
            }
            return _this.listenToOnce(app.vent, 'hospital:dataLoaded', function() {
              loader.remove();
              return app.vent.trigger('hospital:full_healed', {
                data: data
              });
            });
          };
        })(this),
        error: function(data) {
          loader.remove();
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        }
      });
    },
    healTrauma: function(event) {
      var loader, ref, traumaHealCost, traumaId;
      this.lockUI();
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Recover'));
      traumaId = event.currentTarget.getAttribute('data-trauma-id');
      traumaHealCost = (ref = this._data.service.User.traumas[traumaId]) != null ? ref.healCost : void 0;
      return app.server.hospital.service.healTrauma({
        traumaId: traumaId,
        success: (function(_this) {
          return function(data) {
            if (app.gaEnabled) {
              app.ga('send', 'event', 'clinic', 'healing trauma', Number(traumaHealCost));
            }
            return _this.listenToOnce(app.vent, 'hospital:dataLoaded', function() {
              loader.remove();
              return app.vent.trigger('hospital:trauma_healed', {
                data: data
              });
            });
          };
        })(this),
        error: function(data) {
          loader.remove();
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        }
      });
    }
  });
});
