define(['app', 'views/item-view', 'underscore', 'text!templates/hospital/licenses.tmpl', 'views/components/currency'], function(app, ItemView, _, template, formatCurrency) {
  return ItemView.extend({
    className: 'hospital-layout',
    template: template,
    data: {
      licenses: 'work.licenses'
    },
    events: {
      'click .js-buy-license': 'buyLicense'
    },
    onDomRefresh: function() {
      return [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        return el.innerHTML = formatCurrency(el.innerHTML);
      });
    },
    buyLicense: function(event) {
      var currency, img, licenseId, name, price;
      this.lockUI();
      licenseId = event.currentTarget.getAttribute('data-license-id');
      name = event.currentTarget.getAttribute('data-name');
      price = event.currentTarget.getAttribute('data-price');
      currency = event.currentTarget.getAttribute('data-currency');
      img = event.currentTarget.getAttribute('data-img');
      return app.screens.modal('common/buy-popup', {
        name: name,
        maxCount: 1,
        price: price,
        currency: currency,
        img: img,
        callback: (function(_this) {
          return function(count) {
            return app.server.work.licenses.buyDoctorLicense({
              licenseId: licenseId,
              confirm: false,
              success: function(data) {
                var adjustEvent, key, ref;
                app.screens.back();
                if ((data.better_licence != null) && data.better_licence === true) {
                  return app.screens.modal('common/confirmation-popup', {
                    msg: app.tr('YOU HAVE A BETTER LICENSE'),
                    callback: (function(_this) {
                      return function(count) {
                        return app.server.work.licenses.buyDoctorLicense({
                          licenseId: licenseId,
                          confirm: true,
                          success: function(data) {
                            var ref;
                            if (app.gaEnabled) {
                              app.ga('send', 'event', 'clinic', 'healing licence', licenseId, Number(price));
                            }
                            return app.screens.modal('common/info-popup', {
                              msg: ((ref = data.setFlash) != null ? ref[0].msg : void 0) || app.tr("Insufficient level to purchase a license")
                            });
                          }
                        });
                      };
                    })(this)
                  });
                } else {
                  if (app.gaEnabled) {
                    app.ga('send', 'event', 'healing licence', "", price);
                  }
                  if (app.adjustEnabled) {
                    key = 'moj7c5';
                    adjustEvent = new app.adjust.AdjustEvent(key);
                    adjustEvent.addPartnerParameter('event_key', key);
                    app.adjust.Adjust.trackEvent(adjustEvent);
                  }
                  return app.screens.modal('common/info-popup', {
                    msg: ((ref = data.setFlash) != null ? ref[0].msg : void 0) || app.tr("Insufficient level to purchase a license")
                  });
                }
              },
              error: function(data) {
                app.screens.back();
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                });
              }
            });
          };
        })(this)
      });
    }
  });
});
