var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/inventory/popup.tmpl', 'text!templates/inventory/item.tmpl', 'moment'], function(app, ItemView, _, template, itemTemplate, moment) {
  var ItemPopupView;
  ItemPopupView = (function(superClass) {
    extend(ItemPopupView, superClass);

    function ItemPopupView() {
      this.onDataLoaded = bind(this.onDataLoaded, this);
      return ItemPopupView.__super__.constructor.apply(this, arguments);
    }

    ItemPopupView.prototype.template = template;

    ItemPopupView.prototype.itemTemplate = itemTemplate;

    ItemPopupView.prototype.required = ['itemId'];

    ItemPopupView.prototype.data = {
      item: {
        pointer: 'inventory.itemInfo',
        itemId: function() {
          return this.options.itemId;
        }
      },
      itemType: {
        pointer: 'inventory.itemTypeInfo',
        itemTypeId: function() {
          return this.options.itemTypeId;
        }
      }
    };

    ItemPopupView.prototype.events = {
      'click .js-put-on': 'putItemOn',
      'click .js-put-off': 'putItemOff',
      'click .js-use': 'useItem',
      'click .js-eat': 'eatItem',
      'click .js-throw': 'throwItem'
    };

    ItemPopupView.prototype.initialize = function(options) {
      ItemPopupView.__super__.initialize.call(this, options);
      return this.itemTemplate = _.template(this.itemTemplate);
    };

    ItemPopupView.prototype.preprocessData = function(data) {
      if (data.item.itemInfo.ItemType.type === 'receipt') {
        data.item.equippable = false;
        data.item.usable = true;
        data.item.eatable = false;
      } else {
        data.item.equippable = this.isItemEquippable(data.item);
        data.item.usable = this.isItemUsable(data.item);
        data.item.eatable = this.isItemEatable(data.item);
      }
      return data;
    };

    ItemPopupView.prototype.putItemOn = function(event) {
      var item, itemId, slotId;
      this.lockUI();
      item = this._data.item;
      itemId = item.currentItem.UserItem.id;
      slotId = this.chooseSlotToPutOn(this._data.item);
      return app.server.inventory.putOn({
        itemId: itemId,
        slotId: slotId,
        success: (function(_this) {
          return function(data) {
            app.vent.trigger('trade-center:item_put_on', {
              itemType: item.itemInfo.ItemType.type
            });
            app.vent.trigger('inventory:item_equipped', {
              data: item
            });
            return app.screens.back();
          };
        })(this)
      });
    };

    ItemPopupView.prototype.putItemOff = function(event) {
      var item, itemId;
      this.lockUI();
      item = this._data.item;
      itemId = item.currentItem.UserItem.id;
      return app.server.inventory.putOff({
        itemId: itemId,
        success: (function(_this) {
          return function(data) {
            app.vent.trigger('inventory:item_unequipped', {
              data: item
            });
            if (data && data.storage) {
              app.vent.trigger('inventory:inventory_changed', {
                data: data.storage
              });
            }
            return app.screens.back();
          };
        })(this)
      });
    };

    ItemPopupView.prototype.useItem = function(event) {
      var item, itemId, loader;
      this.lockUI();
      item = this._data.item;
      itemId = item.currentItem.UserItem.id;
      loader = this.setLoaderOnElement('.js-use', app.tr('Using item'));
      return app.server.inventory.useForm({
        itemId: itemId,
        success: (function(_this) {
          return function(data) {
            var msg;
            if (!data.error) {
              if ((_this._data.item.itemInfo.ItemType.special != null) && _this._data.item.itemInfo.ItemType.special > 0) {
                return app.server.inventory.useSpecial({
                  itemId: itemId,
                  success: function(data) {
                    loader.remove();
                    if ((_this.options.energyDrink != null) && _this.options.energyDrink) {
                      app.userEvents.trigger('energy:updated');
                      app.screens.close();
                    } else {
                      app.screens.close(function() {
                        return app.screens.modal('common/success-popup', {
                          msg: data.setFlash[0].msg || app.tr("Item received")
                        });
                      });
                    }
                    app.vent.trigger('inventory:special_item_used', {
                      data: item
                    });
                    if (data && data.storage) {
                      return app.vent.trigger('inventory:inventory_changed', {
                        data: data.storage
                      });
                    }
                  },
                  error: function(data) {
                    loader.remove();
                    return app.screens.back();
                  }
                });
              } else if (data.isBlueprint) {
                if (!data.companies || data.companies.length < 1) {
                  return app.screens.close(function() {
                    return app.screens.modal('common/info-popup', {
                      msg: app.tr("No companies to use blueprint")
                    });
                  });
                } else {
                  return app.screens.close(function() {
                    return app.screens.modal('inventory/use-blueprint', {
                      itemId: itemId,
                      productionItem: data.item,
                      companies: data.companies
                    });
                  });
                }
              } else {
                if (!data.empty) {
                  return app.screens.close(function() {
                    return app.screens.modal('inventory/select', {
                      itemId: itemId
                    });
                  });
                } else {
                  if (_this.needConfirm()) {
                    msg = '';
                    if (_this.options.isResetCert) {
                      msg = app.tr('Do you want reset your fight skills?');
                    }
                    console.log(999);
                    app.screens.modal('common/confirmation-popup', {
                      msg: msg,
                      callback: function() {
                        return _this.useInventory(itemId, item);
                      }
                    });
                    return loader.remove();
                  } else {
                    _this.useInventory(itemId, item, _this.options.msg);
                    return loader.remove();
                  }
                }
              }
            }
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            return app.screens.back();
          };
        })(this)
      });
    };

    ItemPopupView.prototype.needConfirm = function() {
      if (this.options.isResetCert) {
        return true;
      }
      return false;
    };

    ItemPopupView.prototype.useInventory = function(itemId, item, msg) {
      return app.server.inventory.use({
        itemId: itemId,
        cityId: -1,
        typeId: -1,
        success: (function(_this) {
          return function(data) {
            app.vent.trigger('inventory:item_used', {
              data: item
            });
            if (data && data.storage) {
              app.vent.trigger('inventory:inventory_changed', {
                data: data.storage
              });
            }
            return app.screens.close(function() {
              if (_this.options.isResetCert) {
                return app.screens.modal('military-base/combat-skills-popup');
              } else {
                return app.screens.modal('common/success-popup', {
                  msg: msg || data.setFlash[0].msg || app.tr("Item received")
                });
              }
            });
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            return app.screens.close(function() {
              return app.screens.modal('common/info-popup', {
                msg: data.setFlash[0].msg || app.tr("Something went wrong!")
              });
            });
          };
        })(this)
      });
    };

    ItemPopupView.prototype.eatItem = function(event) {
      var item, itemId;
      this.lockUI();
      item = this._data.item;
      itemId = item.currentItem.UserItem.id;
      if (this.options.msg && this.options.msg !== '') {
        return app.screens.close((function(_this) {
          return function() {
            return app.screens.modal('common/confirmation-popup', {
              msg: _this.options.msg,
              callback: function() {
                return _this.eatItemRequest(item, itemId, false);
              }
            });
          };
        })(this));
      } else {
        return this.eatItemRequest(item, itemId, true);
      }
    };

    ItemPopupView.prototype.eatItemRequest = function(item, itemId, needBack) {
      if (needBack == null) {
        needBack = true;
      }
      return app.server.inventory.eat({
        itemId: itemId,
        success: (function(_this) {
          return function(data) {
            app.vent.trigger('inventory:item_eaten', {
              data: item
            });
            if (data && data.storage) {
              app.vent.trigger('inventory:inventory_changed', {
                data: data.storage
              });
            }
            if (needBack) {
              return app.screens.back();
            }
          };
        })(this),
        error: (function(_this) {
          return function() {
            return app.screens.back();
          };
        })(this)
      });
    };

    ItemPopupView.prototype.throwItem = function(event) {
      this.lockUI();
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Do you want to throw this item away?'),
        backCallback: (function(_this) {
          return function() {
            return app.screens.back();
          };
        })(this),
        callback: (function(_this) {
          return function(count) {
            var itemId;
            itemId = _this._data.item.currentItem.UserItem.id;
            return app.server.inventory.remove({
              itemId: itemId,
              success: function(data) {
                app.vent.trigger('inventory:item_thrown', {
                  data: data
                });
                if (data && data.storage) {
                  app.vent.trigger('inventory:inventory_changed', {
                    data: data.storage
                  });
                }
                return app.screens.back();
              },
              error: function() {
                return app.screens.back();
              }
            });
          };
        })(this)
      });
    };

    ItemPopupView.prototype.chooseSlotToPutOn = function(item) {
      var i, j, len, len1, ref, ref1, slotId, slots, slotsOccupied;
      slots = (ref = this.classOrTypeToSlotId[item.itemInfo.ItemType.type]) != null ? ref : this.classOrTypeToSlotId[item.itemInfo.ItemType["class"]];
      slotsOccupied = {};
      ref1 = this.options.inventory;
      for (i = 0, len = ref1.length; i < len; i++) {
        item = ref1[i];
        slotsOccupied[item.UserItem.equipped_slot] = true;
      }
      for (j = 0, len1 = slots.length; j < len1; j++) {
        slotId = slots[j];
        if (slotsOccupied[slotId] == null) {
          return slotId;
        }
      }
      return slots[0];
    };

    ItemPopupView.prototype.isItemEquippable = function(item) {
      var ref;
      return !item.itemInfo.ItemType.can_use && (((ref = this.classOrTypeToSlotId[item.itemInfo.ItemType.type]) != null ? ref : this.classOrTypeToSlotId[item.itemInfo.ItemType["class"]]) != null);
    };

    ItemPopupView.prototype.isItemEatable = function(item) {
      return item.itemInfo.ItemType.type === 'food';
    };

    ItemPopupView.prototype.isItemUsable = function(item) {
      return item.itemInfo.ItemType.can_use;
    };

    ItemPopupView.prototype.onDataLoaded = function(data) {
      data.itemDescriptionString = this.itemTemplate(data.item.itemInfo);
      return ItemPopupView.__super__.onDataLoaded.call(this, data);
    };

    ItemPopupView.prototype.classOrTypeToSlotId = {
      'head_items': ['head_inventory'],
      'pants_items': ['pants_inventory'],
      'bracelet_items': ['bracelet_inventory'],
      'torso_items': ['torso_inventory'],
      'boots_items': ['boots_inventory'],
      'first_weapon_items': ['first_weapon_inventory', 'second_weapon_inventory'],
      'ring_items': ['first_ring_inventory', 'second_ring_inventory'],
      'ammunition': ['quick_inventory_1', 'quick_inventory_2', 'quick_inventory_3'],
      'special': ['quick_inventory_1', 'quick_inventory_2', 'quick_inventory_3'],
      'first_aid_kit': ['quick_inventory_1', 'quick_inventory_2', 'quick_inventory_3'],
      'flag': ['quick_inventory_1', 'quick_inventory_2', 'quick_inventory_3'],
      'bio_generator': ['quick_inventory_1', 'quick_inventory_2', 'quick_inventory_3']
    };

    return ItemPopupView;

  })(ItemView);
  return ItemPopupView;
});
