var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/maria/company-storage.tmpl', 'views/components/item-type-comparator'], function(app, ItemView, _, template, itemTypeComparatorComponent) {
  var CompanyStorage;
  CompanyStorage = (function(superClass) {
    extend(CompanyStorage, superClass);

    function CompanyStorage() {
      this.showReports = bind(this.showReports, this);
      return CompanyStorage.__super__.constructor.apply(this, arguments);
    }

    CompanyStorage.prototype.template = template;

    CompanyStorage.prototype.data = {
      companyInfo: {
        pointer: 'business.companyInfo',
        companyId: function() {
          return this.options.companyId;
        }
      },
      effects: {
        pointer: 'business.company.effects',
        companyId: function() {
          return this.options.companyId;
        }
      },
      storage: {
        pointer: 'business.storage',
        companyId: function() {
          return this.options.companyId;
        }
      },
      inventory: {
        pointer: 'cityHall.storage'
      }
    };

    CompanyStorage.prototype.events = {
      'click .js-goto-employees': 'gotoEmployees',
      'click .js-goto-common': 'gotoCommon',
      'click .js-goto-selling': 'gotoSelling',
      'click .js-goto-buying': 'gotoBuying',
      'click .js-settings': 'settings',
      'click .js-inventory-item': 'transferItemToCompany',
      'click .js-storage-item': 'transferItemToStorage',
      'click .js-expand-storage': 'expandStorage',
      'click .js-toggle-effects': 'showEffects',
      'click .js-reports-archive': 'showReports',
      'click .js-generate-report': 'generateReport'
    };

    CompanyStorage.prototype.initialize = function() {
      CompanyStorage.__super__.initialize.apply(this, arguments);
      return this.itemTypeComparatorFactory = itemTypeComparatorComponent.itemTypeComparatorFactory;
    };

    CompanyStorage.prototype.gotoEmployees = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('maria/employees', {
        companyId: companyId
      });
    };

    CompanyStorage.prototype.gotoCommon = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.tab('maria/company', {
        companyId: companyId
      });
    };

    CompanyStorage.prototype.gotoSelling = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      if (this._data.storage.storage.length === 0) {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("Your storage is empty")
        });
      } else {
        return app.screens.dialog('trade-center/sell-item-company', {
          companyId: companyId
        });
      }
    };

    CompanyStorage.prototype.gotoBuying = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('trade-center/trade-center-business', {
        companyId: companyId
      });
    };

    CompanyStorage.prototype.transferItemToCompany = function(event) {
      var companyId, itemTypeId, number;
      this.lockUI();
      itemTypeId = event.currentTarget.getAttribute('data-itemType-id');
      number = event.currentTarget.getAttribute('data-quantity');
      companyId = this.options.companyId;
      return app.screens.modal('business/move-item-from-city-storage-popup', {
        itemTypeId: itemTypeId,
        number: number,
        callback: (function(_this) {
          return function(quantity) {
            return app.server.business.moveItemsFromCityToCompany({
              companyId: companyId,
              itemTypeId: itemTypeId,
              quantity: quantity,
              success: function(data) {
                return app.screens.back();
              },
              error: function(data) {
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    };

    CompanyStorage.prototype.transferItemToStorage = function(event) {
      var companyId, itemTypeId, number;
      this.lockUI();
      itemTypeId = event.currentTarget.getAttribute('data-itemType-id');
      number = event.currentTarget.getAttribute('data-quantity');
      companyId = this.options.companyId;
      return app.screens.modal('business/move-item-to-city-storage-popup', {
        itemTypeId: itemTypeId,
        number: number,
        callback: (function(_this) {
          return function(quantity) {
            return app.server.business.moveItemsToCity({
              companyId: companyId,
              itemTypeId: itemTypeId,
              quantity: quantity,
              success: function(data) {
                return app.screens.back();
              },
              error: function(data) {
                return app.screens.modal('common/info-popup', {
                  msg: data.setFlash[0].msg || app.tr("Something went wrong")
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    };

    CompanyStorage.prototype.expandStorage = function(event) {
      var companyId;
      this.lockUI();
      companyId = this.options.companyId;
      if (this._data.storage.next_storage_level.max_prod != null) {
        return app.screens.modal('common/confirmation-popup', {
          msg: this._data.storage.next_storage_level.confirmation,
          callback: (function(_this) {
            return function() {
              return app.server.cityHall.company.upStorageLevel({
                companyId: companyId,
                success: function(data) {
                  return app.screens.modal('common/success-popup', {
                    msg: data.setFlash[0].msg || app.tr("Warehouse increased")
                  });
                },
                error: function(data) {
                  return app.screens.modal('common/info-popup', {
                    msg: data.setFlash[0].msg || app.tr("Something went wrong")
                  });
                }
              });
            };
          })(this)
        });
      } else {
        return app.screens.modal('common/info-popup', {
          msg: this._data.storage.next_storage_level.confirmation || app.tr("Something went wrong")
        });
      }
    };

    CompanyStorage.prototype.showEffects = function(event) {
      return $('.js-effects').toggle(400).removeClass('u-hide');
    };

    CompanyStorage.prototype.showReports = function(event) {
      var companyId;
      this.lockUI();
      companyId = event.currentTarget.getAttribute('data-company-id');
      return app.screens.dialog('business/company-reports/archive', {
        companyId: companyId,
        companyName: this._data.companyInfo.company.name
      });
    };

    CompanyStorage.prototype.generateReport = function(event) {
      var companyId, msg, reportType;
      companyId = event.currentTarget.getAttribute('data-company-id');
      reportType = event.currentTarget.getAttribute('data-report-type');
      msg = app.tr('Are you sure you generate report?');
      return app.screens.modal('common/confirmation-popup', {
        msg: msg,
        callback: (function(_this) {
          return function() {
            _this.lockUI();
            return app.server.business.generateReport({
              companyId: companyId,
              reportType: reportType,
              success: function(data) {
                return app.screens.modal('business/company-reports/report', {
                  report: data.report
                });
              }
            });
          };
        })(this)
      });
    };

    CompanyStorage.prototype.onRender = function() {
      this.toggleView('.js-reports-switcher', '.js-reports-buttons');
      return CompanyStorage.__super__.onRender.call(this);
    };

    CompanyStorage.prototype.serializeData = function() {
      var data, itemTypeComparator;
      data = CompanyStorage.__super__.serializeData.apply(this, arguments);
      if (data && (data.inventory != null) && data.inventory.storage.length !== 0) {
        itemTypeComparator = this.itemTypeComparatorFactory(data.inventory.categorySortWeight);
        data.inventory.storage = data.inventory.storage.sort(itemTypeComparator);
      }
      if (data && (data.storage != null) && data.storage.storage.length !== 0) {
        itemTypeComparator = this.itemTypeComparatorFactory(data.storage.categorySortWeight);
        data.storage.storage = data.storage.storage.sort(itemTypeComparator);
      }
      return data;
    };

    return CompanyStorage;

  })(ItemView);
  return CompanyStorage;
});
