var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/maria/initiative.tmpl'], function(app, ItemView, _, template) {
  var InitiativeView;
  InitiativeView = (function(superClass) {
    extend(InitiativeView, superClass);

    function InitiativeView() {
      this._triggerKeyboardHide = bind(this._triggerKeyboardHide, this);
      this._triggerKeyboardShow = bind(this._triggerKeyboardShow, this);
      this.initialize = bind(this.initialize, this);
      return InitiativeView.__super__.constructor.apply(this, arguments);
    }

    InitiativeView.prototype.template = template;

    InitiativeView.prototype.bill_status = {
      'initiative': app.tr('Initiative'),
      'active': app.tr('Under consideration'),
      'archive': app.tr('In archive')
    };

    InitiativeView.prototype.data = {
      bill: {
        pointer: 'cityHall.initiative',
        billId: function() {
          return this.options.billId;
        }
      },
      userStatus: 'user.status'
    };

    InitiativeView.prototype.events = {
      'click .js-add-comment': 'sendComment',
      'click .js-edit': 'editComment',
      'click .js-edit-bill': 'editBill',
      'click .js-vote-for': 'voteForBill',
      'click .js-vote-against': 'voteAgainstBill',
      'click .js-vote-initiative-for': 'voteForInitiative',
      'click .js-vote-initiative-against': 'voteAgainsInitiative',
      'click .js-vote-for-takeover': 'voteForTakeover',
      'click .js-vote-against-takeover': 'voteAgainstTakeover',
      'click .js-vote-for-itemBill': 'voteForItemBill',
      'click .js-vote-against-itemBill': 'voteAgainstItemBill',
      'click .js-evolution-vote-for': 'voteForEvolution',
      'click .js-evolution-vote-against': 'voteAgainstEvolution'
    };

    InitiativeView.prototype.initialize = function(options) {
      this.options = options;
      InitiativeView.__super__.initialize.call(this, this.options);
      this.listenTo(app, "keyboard:show", this._triggerKeyboardShow);
      return this.listenTo(app, "keyboard:hide", this._triggerKeyboardHide);
    };

    InitiativeView.prototype.preprocessData = function(data) {
      this.options.bill_status = this.bill_status[data.bill.Bill.state];
      return data;
    };

    InitiativeView.prototype._triggerKeyboardShow = function(keyboardHeight) {
      return this.$('.c-footer').css("bottom", keyboardHeight);
    };

    InitiativeView.prototype._triggerKeyboardHide = function() {
      return this.$('.c-footer').css("bottom", 0);
    };

    InitiativeView.prototype.sendComment = function(event) {
      var commentText;
      this.lockUI();
      commentText = this.$el.find('.js-text-comment').val();
      if (commentText === "") {
        return app.screens.modal('common/info-popup', {
          msg: app.tr("Comment cannot be empty")
        });
      } else {
        return app.server.cityHall.initiative.addComment({
          billId: this.options.billId,
          body: commentText,
          success: (function(_this) {
            return function(data) {
              return _this.reload();
            };
          })(this),
          error: function(data) {
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          }
        });
      }
    };

    InitiativeView.prototype.editComment = function(event) {
      var commentId;
      commentId = event.currentTarget.getAttribute('data-comment-id');
      return app.screens.dialog('maria/edit-comment', {
        billId: this.options.billId,
        commentId: commentId
      });
    };

    InitiativeView.prototype.editBill = function(event) {
      return app.screens.dialog('maria/edit-bill', {
        billId: this.options.billId
      });
    };

    InitiativeView.prototype.voteForBill = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteForBill({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteAgainstBill = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteAgainstBill({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteForInitiative = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteForInitiative({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteAgainstInitiative = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteAgainstInitiative({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteForTakeover = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.areasTakeover.vote({
        billId: billId,
        vote: 'positive',
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteAgainstTakeover = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-bill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.areasTakeover.vote({
        billId: billId,
        vote: 'negative',
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteForItemBill = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-itemBill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteForItemBill({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteAgainstItemBill = function(event) {
      var billId, loader;
      this.lockUI();
      billId = event.currentTarget.getAttribute('data-itemBill-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.voteAgainstItemBill({
        billId: billId,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.evolutionVote = function(event, vote) {
      var cityEvolutionId, loader;
      this.lockUI();
      cityEvolutionId = event.currentTarget.getAttribute('data-evolution-id');
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Voting'));
      return app.server.cityHall.evolutionVote({
        cityEvolutionId: cityEvolutionId,
        vote: vote,
        success: (function(_this) {
          return function(data) {
            loader.remove();
            return _this.reload();
          };
        })(this),
        error: function() {
          return loader.remove();
        }
      });
    };

    InitiativeView.prototype.voteForEvolution = function(event) {
      return this.evolutionVote(event, 1);
    };

    InitiativeView.prototype.voteAgainstEvolution = function(event) {
      return this.evolutionVote(event, 0);
    };

    return InitiativeView;

  })(ItemView);
  return InitiativeView;
});
