var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/military-base/gangs.tmpl'], function(app, ItemView, _, template) {
  var GangsView;
  GangsView = (function(superClass) {
    extend(GangsView, superClass);

    function GangsView() {
      return GangsView.__super__.constructor.apply(this, arguments);
    }

    GangsView.prototype.template = template;

    GangsView.prototype.className = 'military-layout dark-theme';

    GangsView.prototype.data = {
      info: 'military.info',
      gangs: {
        pointer: 'military.pve.gangs',
        bandId: function() {
          return Number(localStorage.getItem('mbLastBandId')) || 0;
        }
      }
    };

    GangsView.prototype.events = {
      'click .js-bands-select': 'onBandsSelectClick',
      'click .js-change-level': 'onChangeLevelClick',
      'click .js-change-rank': 'onChangeRankClick',
      'click .js-rank': 'selectRank',
      'click .js-fight': 'fight'
    };

    GangsView.prototype.bands = null;

    GangsView.prototype.rankInfo = null;

    GangsView.prototype.progressByBand = null;

    GangsView.prototype.selectedLowRank = false;

    GangsView.prototype.selectedLowComplexity = false;

    GangsView.prototype.initialize = function(options) {
      GangsView.__super__.initialize.call(this, options);
      this.options.selectedBandId = Number(localStorage.getItem('mbLastBandId')) || 0;
      this.options.selectedComplexity = 1;
      this.options.selectedRank = 1;
      this.bands = null;
      this.rankInfo = null;
      this.progressByBand = {};
      this.selectedLowRank = false;
      return this.selectedLowComplexity = false;
    };

    GangsView.prototype.preprocessData = function(data) {
      if (data && data.info) {
        this.rankInfo = data.info;
      }
      if (data && data.gangs && (!this.bands)) {
        this.bands = data.gangs.bands;
        if (!this.options.selectedBandId && this.bands.length) {
          this.options.selectedBandId = this.bands[0].id;
        }
        this.progressByBand[this.options.selectedBandId] = data.gangs.progress;
        this.options.selectedComplexity = data.gangs.progress.length;
        this.options.selectedRank = data.gangs.progress[this.options.selectedComplexity - 1].availableRank;
      }
      return data;
    };

    GangsView.prototype.serializeData = function() {
      var band, data, i, len, ref;
      data = GangsView.__super__.serializeData.call(this);
      if (data && data.gangs && data.gangs.bands) {
        data.band = null;
        ref = data.gangs.bands;
        for (i = 0, len = ref.length; i < len; i++) {
          band = ref[i];
          if (Number(band.id) === this.options.selectedBandId) {
            data.band = band;
            break;
          }
        }
        if (!data.band) {
          data.band = data.gangs.bands[0];
        }
        data.info = this.rankInfo;
        data.progress = this.progressByBand[this.options.selectedBandId];
        data.rank = data.progress[this.options.selectedComplexity - 1].ranks[this.options.selectedRank - 1];
      }
      return data;
    };

    GangsView.prototype.onChangeLevelClick = function(event) {
      var amount, newLvl;
      amount = event.currentTarget.dataset.type === 'inc' ? 1 : -1;
      newLvl = this.options.selectedComplexity + amount;
      this.setComplexity(newLvl);
      return this.render();
    };

    GangsView.prototype.onChangeRankClick = function(event) {
      var rank;
      rank = Number(event.currentTarget.dataset.rank);
      this.setRank(rank);
      return this.render();
    };

    GangsView.prototype.onBandsSelectClick = function(event) {
      return app.screens.dialog('military-base/gangs-list', {
        bands: this.bands,
        info: this.rankInfo,
        onSelect: (function(_this) {
          return function(bandId) {
            return _this.loadBandProgress(bandId);
          };
        })(this)
      });
    };

    GangsView.prototype.loadBandProgress = function(bandId) {
      if (bandId === this.options.selectedBandId) {
        return;
      }
      if (this.progressByBand[bandId]) {
        this.options.selectedBandId = bandId;
        return this.render();
      }
      app.showLoader();
      this.lockUI();
      return app.server.military.pve.gangProgress({
        bandId: bandId,
        success: (function(_this) {
          return function(data) {
            var ref;
            _this.unlockUI();
            app.hideLoader();
            if (data && data.progress) {
              _this.progressByBand[bandId] = data.progress;
              _this.setBandId(bandId);
              return _this.render();
            } else {
              return app.screens.modal('common/info-popup', {
                msg: (data != null ? (ref = data.setFlash[0]) != null ? ref.msg : void 0 : void 0) || app.tr("Something went wrong")
              });
            }
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            app.hideLoader();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    };

    GangsView.prototype.setBandId = function(bandId) {
      this.options.selectedBandId = bandId;
      localStorage.setItem('mbLastBandId', this.options.selectedBandId);
      return this.setComplexity(this.options.selectedComplexity);
    };

    GangsView.prototype.setComplexity = function(complexity) {
      var maxComplexity;
      maxComplexity = this.progressByBand[this.options.selectedBandId].length;
      if (maxComplexity < complexity) {
        this.options.selectedComplexity = maxComplexity;
      } else if (complexity < 1) {
        this.options.selectedComplexity = 1;
      } else {
        this.selectedLowComplexity = maxComplexity > complexity;
        this.options.selectedComplexity = complexity;
      }
      return this.setRank(this.options.selectedRank);
    };

    GangsView.prototype.setMaxComplexity = function() {
      this.options.selectedComplexity = this.progressByBand[this.options.selectedBandId].length;
      return this.setRank(this.options.selectedRank);
    };

    GangsView.prototype.setRank = function(rank) {
      var maxRank;
      maxRank = this.progressByBand[this.options.selectedBandId][this.options.selectedComplexity - 1].availableRank;
      if (maxRank < rank) {
        return this.options.selectedRank = maxRank;
      } else if (rank < 1) {
        return this.options.selectedRank = 1;
      } else {
        this.selectedLowRank = rank < maxRank;
        return this.options.selectedRank = rank;
      }
    };

    GangsView.prototype.setMaxRank = function() {
      return this.options.selectedRank = this.progressByBand[this.options.selectedBandId][this.options.selectedComplexity - 1].availableRank;
    };

    GangsView.prototype.updateSelectedAfterUserWin = function() {
      if (!this.selectedLowComplexity) {
        this.setMaxComplexity();
      }
      if (!this.selectedLowRank) {
        return this.setMaxRank();
      }
    };

    GangsView.prototype.fight = function(event) {
      var loader, oldData;
      this.lockUI();
      loader = this.setLoaderOnElement('.js-fight', app.tr('Start the fight'));
      oldData = this._data;
      return this.app.server.military.pve.fight({
        complexity: this.options.selectedComplexity,
        rank: this.options.selectedRank,
        bandId: this.options.selectedBandId,
        success: (function(_this) {
          return function(data) {
            var adjustEvent, gangLabel, openCombatPointsPopup, openInjuryPopup, openRankPopup, openResultPopup;
            _this.listenToOnce(app.vent, 'short-info:dataLoaded', function() {
              return loader.remove();
            });
            _this.app.vent.trigger('military-base:gangs_fought', {
              data: data
            });
            if (app.adjustEnabled && app.tutorial.isActive()) {
              adjustEvent = new app.adjust.AdjustEvent("2mulbt");
              adjustEvent.addPartnerParameter('event_key', '2mulbt');
              app.adjust.Adjust.trackEvent(adjustEvent);
            }
            if (app.gaEnabled) {
              gangLabel = _this.options.selectedBandId + "-" + _this.options.selectedComplexity + "-" + _this.options.selectedRank;
              app.ga('send', 'event', 'warbase', 'PVE', gangLabel, Number(data.result.vdEarned));
            }
            if (data.info) {
              _this.rankInfo = data.info;
            }
            if (data.userWin && data.progress) {
              _this.progressByBand[_this.options.selectedBandId] = data.progress;
              _this.updateSelectedAfterUserWin();
            }
            _this.render();
            app.shortInfo.changeUserParams({
              'health': data.result.healthLoss,
              'energy': data.result.energyLoss,
              'vd_balance': data.result.vdEarned
            });
            openInjuryPopup = function(onBack) {
              var options;
              if (data.result.traumas.length !== 0) {
                options = {
                  injured: {
                    light: 0,
                    medium: 0,
                    height: 0
                  },
                  traumasEffect: 0
                };
                if (onBack != null) {
                  options.onBack = onBack;
                }
                data.result.traumas.forEach(function(trauma) {
                  options.injured[trauma.name] += 1;
                  return options.traumasEffect += +trauma.result;
                });
                return app.screens.modal('common/injury-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            openRankPopup = function(onBack) {
              var options;
              if (data.result.rank != null) {
                options = {
                  rank: data.result.rank
                };
                if (onBack != null) {
                  options.onBack = onBack;
                }
                options.fightType = 'pve';
                return app.screens.modal('military-base/new-rank-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            openResultPopup = function(onBack) {
              var options;
              options = data;
              if (onBack != null) {
                options.onBack = onBack;
              }
              options.fightType = 'pve';
              options.repeat = {
                method: _this.fight,
                context: _this,
                event: event
              };
              return app.screens.modal('military-base/fight-result-popup', options);
            };
            openCombatPointsPopup = function(onBack) {
              var options;
              if (data.result.rank != null) {
                options = data;
                if (onBack != null) {
                  options.onBack = onBack;
                }
                options.fightType = 'pve';
                return app.screens.modal('military-base/combat-skills-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            return openResultPopup(function() {
              return openRankPopup(function() {
                return openCombatPointsPopup(function() {
                  return openInjuryPopup();
                });
              });
            });
          };
        })(this),
        error: (function(_this) {
          return function(err) {
            _this.unlockUI();
            loader.remove();
            if (err.setFlash) {
              return err.setFlash.forEach(function(msg) {
                return app.screens.modal('common/info-popup', {
                  msg: msg.msg
                });
              });
            }
          };
        })(this)
      });
    };


    /*
    			Center cloud
     */

    GangsView.prototype.centerCloud = function() {};

    GangsView.prototype.onDomRefresh = function() {
      return this.centerCloud();

      /*
      			if @_data?.gangs?
      				@availableComplexity = @_data.gangs.progress.length
      				@complexity = @complexity ? @availableComplexity
      				@availableRank = @_data.gangs.progress[@complexity - 1].availableRank
      				@rank = @rank ? @availableRank
      				gangName = @_data.gangs.progress[@complexity - 1].name
      
      				@setComplexity @complexity, @rank
      				@setGangName gangName
      				@setAvailableRank @availableRank
      				@setRank @rank
      				@setEnergy @complexity, @rank
       */
    };

    return GangsView;

  })(ItemView);
  return GangsView;
});
