var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/military-base/pvp-search-popup.tmpl'], function(app, ItemView, _, template) {
  var PvpSearchPopupView;
  PvpSearchPopupView = (function(superClass) {
    extend(PvpSearchPopupView, superClass);

    function PvpSearchPopupView() {
      this.customOnBack = bind(this.customOnBack, this);
      return PvpSearchPopupView.__super__.constructor.apply(this, arguments);
    }

    PvpSearchPopupView.prototype.template = template;

    PvpSearchPopupView.prototype.events = {
      'click .js-cancel-request': 'cancelRequest'
    };

    PvpSearchPopupView.prototype.customOnBack = function() {
      var ref;
      if (this.interval != null) {
        clearInterval(this.interval);
      }
      if (((ref = this.options) != null ? ref.onBack : void 0) != null) {
        return this.options.onBack();
      }
    };

    PvpSearchPopupView.prototype.cancelRequest = function() {
      var callback;
      this.lockUI();
      callback = (function(_this) {
        return function(data) {
          _this.unlockUI();
          _this.customOnBack();
          if ((data != null ? data.setFlash : void 0) != null) {
            return data.setFlash.forEach(function(msg) {
              return app.screens.modal('common/info-popup', {
                msg: msg.msg
              });
            });
          }
        };
      })(this);
      return app.server.military.pvp.fight.cancel({
        success: callback,
        error: callback
      });
    };

    PvpSearchPopupView.prototype.refreshTimer = function() {
      var delta;
      delta = 20000 - (Date.now() - this.start);
      return this.timerEl.innerHTML = Math.max(Math.round(delta / 1000), 0);
    };

    PvpSearchPopupView.prototype.onRender = function(event) {
      this.start = Date.now();
      return app.server.military.pvp.fight({
        success: (function(_this) {
          return function(data) {
            var label, openCombatPointsPopup, openInjuryPopup, openLiguePopup, openRankPopup, openResultPopup;
            if (app.gaEnabled) {
              if (parseInt(data.userId) === parseInt(data.enemyId)) {
                label = 'mirror';
              } else {
                label = 'real';
              }
              app.ga('send', 'event', 'warbase', 'PVP', label, parseInt(data.result.medalsEarned));
            }
            openInjuryPopup = function(onBack) {
              var options;
              if (data.result.traumas.length !== 0) {
                options = {
                  injured: {
                    light: 0,
                    medium: 0,
                    height: 0
                  },
                  traumasEffect: 0
                };
                if (onBack != null) {
                  options.onBack = onBack;
                }
                data.result.traumas.forEach(function(trauma) {
                  options.injured[trauma.name] += 1;
                  return options.traumasEffect += +trauma.result;
                });
                return app.screens.modal('common/injury-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            openRankPopup = function(onBack) {
              var options;
              if (data.result.rank != null) {
                options = {
                  rank: data.result.rank
                };
                if (onBack != null) {
                  options.onBack = onBack;
                }
                options.fightType = 'pvp';
                return app.screens.modal('military-base/new-rank-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            openResultPopup = function(onBack) {
              var options;
              options = data;
              if (onBack != null) {
                options.onBack = onBack;
              }
              options.fightType = 'pvp';
              options.repeat = {
                method: (function() {
                  return app.screens.modal('military-base/pvp-search-popup');
                }),
                context: _this,
                event: event
              };
              return app.screens.modal('military-base/fight-pvp-result-popup', options);
            };
            openLiguePopup = function(onBack) {
              var options, ref, ref1;
              if (((ref = data.result.old_ligue) != null ? ref.level : void 0) !== ((ref1 = data.result.new_ligue) != null ? ref1.level : void 0)) {
                options = data;
                if (onBack != null) {
                  options.onBack = onBack;
                }
                return app.screens.modal('military-base/ligue-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            openCombatPointsPopup = function(onBack) {
              var options;
              if (data.result.rank != null) {
                options = data;
                if (onBack != null) {
                  options.onBack = onBack;
                }
                options.fightType = 'pve';
                return app.screens.modal('military-base/combat-skills-popup', options);
              } else {
                if (onBack != null) {
                  return onBack();
                }
              }
            };
            return app.screens.back(function() {
              return openResultPopup(function() {
                return openRankPopup(function() {
                  return openCombatPointsPopup(function() {
                    return openInjuryPopup(_this.customOnBack);
                  });
                });
              });
            });
          };
        })(this),
        error: (function(_this) {
          return function(err) {
            _this.customOnBack();
            return app.screens.back(function() {
              if (err.setFlash) {
                return err.setFlash.forEach(function(msg) {
                  return app.screens.modal('common/info-popup', {
                    msg: msg.msg
                  });
                });
              }
            });
          };
        })(this)
      });
    };

    PvpSearchPopupView.prototype.onDomRefresh = function() {
      this.timerEl = this.el.querySelector('.js-timer');
      if (this.interval != null) {
        clearInterval(this.interval);
      }
      return this.interval = setInterval(this.refreshTimer.bind(this), 1000);
    };

    return PvpSearchPopupView;

  })(ItemView);
  return PvpSearchPopupView;
});
