define(['app', 'views/item-view', 'underscore', 'text!templates/parties/create-party.tmpl', 'jquery', 'views/components/file-uploader'], function(app, ItemView, _, template, $, fileUploader) {
  return ItemView.extend({
    template: template,
    className: 'no-bg-image',
    events: {
      'click .js-create-party': 'createParty',
      'click .js-select-image': 'selectAndUploadImage'
    },
    data: {
      price: 'party.create.price'
    },
    selectAndUploadImage: (function(_this) {
      return function(event) {
        var uploadPhoto, uploadUrl;
        uploadPhoto = function(imageURI) {
          var ft, options, to;
          options = new FileUploadOptions;
          options.fileKey = 'file';
          options.fileName = imageURI.substr(imageURI.lastIndexOf('/') + 1);
          options.mimeType = 'image/jpeg';
          options.chunkedMode = false;
          to = encodeURI(app.config.serverURLProtocol + '://' + app.config.serverURL + '/parties/upload.json');
          ft = new FileTransfer;
          return ft.upload(imageURI, to, function(r) {
            var json;
            json = JSON.parse(r.response);
            if ((json != null ? json.image : void 0) != null) {
              return $('#party-logo').attr('src', app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/partylogo/' + json.image);
            }
          }, function(error) {
            return app.screens.modal('common/info-popup', {
              msg: app.tr("Something went wrong")
            });
          }, options);
        };
        if (navigator.camera != null) {
          return navigator.camera.getPicture(uploadPhoto, (function(message) {
            return app.screens.modal('common/info-popup', {
              msg: app.tr("Image is not chosen")
            });
          }), {
            quality: 50,
            destinationType: navigator.camera.DestinationType.FILE_URI,
            sourceType: navigator.camera.PictureSourceType.PHOTOLIBRARY
          });
        } else {
          uploadUrl = encodeURI(app.config.serverURLProtocol + '://' + app.config.serverURL + '/parties/upload.json');
          return fileUploader('.js-file-select', uploadUrl, function(error, data) {
            var json;
            if (error) {
              return app.screens.modal('common/info-popup', {
                msg: app.tr("Something went wrong")
              });
            } else {
              json = JSON.parse(data);
              if ((json != null ? json.image : void 0) != null) {
                return $('#party-logo').attr('src', app.config.serverURLProtocol + '://' + app.config.serverURL + '/attachments/partylogo/' + json.image);
              }
            }
          });
        }
      };
    })(this),
    createParty: function(event) {
      var description, loader, logo, name, price;
      this.lockUI();
      name = this.$el.find('.name').val();
      description = this.$el.find('#description').val();
      logo = this.$el.find('#party-logo').attr('src');
      price = this.$('.js-currency').val();
      if (name === "") {
        app.screens.modal('common/info-popup', {
          msg: app.tr("Specify the name")
        });
        return;
      }
      if (description === "") {
        app.screens.modal('common/info-popup', {
          msg: app.tr("Specify the party's motto")
        });
        return;
      }
      if (logo === "" || logo === '../images/icons/icon-plus-2.png') {
        app.screens.modal('common/info-popup', {
          msg: app.tr("Upload the logo")
        });
        return;
      }
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Creating'));
      return app.server.party.create({
        name: name,
        description: description,
        logo: logo,
        success: (function(_this) {
          return function(data) {
            if (app.gaEnabled) {
              app.ga('send', 'event', 'party', 'vgoldbuy', 'createparty', price);
            }
            loader.remove();
            _this.unlockUI();
            return app.screens.back();
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            loader.remove();
            _this.unlockUI();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    }
  });
});
