define(['app', 'views/item-view', 'underscore', 'text!templates/parties/friendly-party.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    require: ['partyId'],
    data: {
      unfriendly: {
        pointer: 'party.unfriendly',
        partyId: function() {
          if (this.options.partyId != null) {
            return this.options.partyId;
          } else {
            return null;
          }
        },
        name: function() {
          if (this.options.name != null) {
            return this.options.name;
          } else {
            return null;
          }
        }
      }
    },
    events: {
      'click .js-add-friend': 'addFriend',
      'click .js-remove-friend': 'removeFriend',
      'change .c-input': 'partySearch'
    },
    addFriend: function(event) {
      var partyId;
      this.lockUI();
      partyId = event.currentTarget.getAttribute('data-party-id');
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Are you sure you want to give this party friendly status?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.friends.add({
              partyId: partyId,
              success: function(data) {
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    },
    removeFriend: function(event) {
      var partyId;
      this.lockUI();
      partyId = event.currentTarget.getAttribute('data-party-id');
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Do you want to exclude this party from the Friends?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.friends.remove({
              partyId: partyId,
              success: function(data) {
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                }, app.screens.back());
              }
            });
          };
        })(this)
      });
    },
    partySearch: function(event) {
      this.lockUI();
      event.preventDefault();
      this.options.name = this.$el.find('.c-input').val();
      return this.reload();
    }
  });
});
