define(['app', 'views/item-view', 'underscore', 'text!templates/parties/member-settings.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    require: ['memberId'],
    data: {
      info: {
        pointer: 'party.member',
        memberId: function() {
          return this.options.memberId;
        }
      }
    },
    events: {
      'click .js-set-master': 'setMaster',
      'click .js-remove-from-office': 'removeFromOffice',
      'click .js-add-to-parliament-list': 'addToPaliamentList',
      'click .js-remove-from-parliament-list': 'removeFromPaliamentList',
      'click .js-remove-from-members': 'removeFromMembers',
      'click .js-change-rights': 'changeRights'
    },
    setMaster: function(event) {
      var memberId, ref, ref1, ref2, username;
      this.lockUI();
      memberId = event.currentTarget.getAttribute('data-member-id');
      username = (ref = this._data.info) != null ? (ref1 = ref.member) != null ? (ref2 = ref1.User) != null ? ref2.username : void 0 : void 0 : void 0;
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Are you sure you would like to set a new manager?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.member.set.master({
              memberId: memberId,
              success: function(data) {
                if (app.gaEnabled) {
                  app.ga('send', 'event', "party", "promote to leader", username);
                }
                app.screens.back();
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                });
              }
            });
          };
        })(this)
      });
    },
    changeRights: function(event) {
      var memberId;
      this.lockUI();
      app.screens.back();
      memberId = event.currentTarget.getAttribute('data-member-id');
      return app.screens.dialog('parties/change-permissions', {
        memberId: memberId
      });
    },
    removeFromOffice: function(event) {
      var memberId;
      this.lockUI();
      memberId = event.currentTarget.getAttribute('data-member-id');
      return app.server.party.rights.remove.all({
        memberId: memberId,
        success: (function(_this) {
          return function(data) {
            app.screens.back();
            return app.screens.modal('common/success-popup', {
              msg: data.setFlash[0].msg
            });
          };
        })(this)
      });
    },
    removeFromMembers: function(event) {
      var memberId, ref, ref1, ref2, username;
      this.lockUI();
      username = (ref = this._data.info) != null ? (ref1 = ref.member) != null ? (ref2 = ref1.User) != null ? ref2.username : void 0 : void 0 : void 0;
      memberId = event.currentTarget.getAttribute('data-member-id');
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Do you want to exclude this character from the party?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.member.remove({
              memberId: memberId,
              success: function(data) {
                var $member;
                if (app.gaEnabled) {
                  app.ga('send', 'event', "party", "kick", username);
                }
                app.screens.close();
                $member = $.find('#js-member-' + memberId);
                if ($member) {
                  return $member[0].remove();
                }
              }
            });
          };
        })(this)
      });
    },
    removeFromPaliamentList: function(event) {
      var userId;
      this.lockUI();
      userId = event.currentTarget.getAttribute('data-user-id');
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Do you want to exclude this character from parliamentary list?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.member.parliament.remove({
              userId: userId,
              success: function(data) {
                app.screens.back();
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                });
              }
            });
          };
        })(this)
      });
    },
    addToPaliamentList: function(event) {
      var memberId, ref, ref1, ref2, username;
      this.lockUI();
      memberId = event.currentTarget.getAttribute('data-member-id');
      username = (ref = this._data.info) != null ? (ref1 = ref.member) != null ? (ref2 = ref1.User) != null ? ref2.username : void 0 : void 0 : void 0;
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Do you want to include this character in the parliamentary list?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.member.parliament.add({
              memberId: memberId,
              success: function(data) {
                if (app.gaEnabled) {
                  app.ga('send', 'event', "party", "delegate", username);
                }
                app.screens.back();
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                });
              }
            });
          };
        })(this)
      });
    }
  });
});
