define(['app', 'views/item-view', 'underscore', 'text!templates/parties/parlament.tmpl', 'views/components/parlament-swipe', 'sortable'], function(app, ItemView, _, template, ParlamentSwipe, Sortable) {
  return ItemView.extend({
    template: template,
    data: {
      list: 'party.parliamentary',
      request_count: 'party.requests.count'
    },
    events: {
      'click .js-remove-from-parliament-list': 'removeFromPaliamentList',
      'click .js-add-request': 'addRequest',
      'click .js-remove-request': 'removeRequest'
    },
    addRequest: function(event) {
      return app.server.party.parliament.add({
        success: (function(_this) {
          return function(data) {
            if (app.gaEnabled) {
              app.ga('send', 'event', 'party', 'vgoldbuy', 'parliament-request', data.price);
            }
            return app.screens.modal('common/success-popup', {
              msg: data.setFlash[0].msg
            });
          };
        })(this)
      });
    },
    removeRequest: function(event) {
      return app.server.party.parliament.remove({
        success: (function(_this) {
          return function(data) {
            return app.screens.modal('common/success-popup', {
              msg: data.setFlash[0].msg
            });
          };
        })(this)
      });
    },
    sortRequest: function(event) {
      var users;
      users = [];
      [].forEach.call(this.el.querySelectorAll('.c-card--parlament-users__item'), (function(_this) {
        return function(el, index) {
          users.push(parseInt(el.getAttribute('data-parliamentary-id')));
          return $(el).find('.js-user-order').text(++index);
        };
      })(this));
      return app.server.party.parliament.reorder({
        ids: users,
        success: (function(_this) {
          return function(data) {
            return app.screens.modal('common/success-popup', {
              msg: data.setFlash[0].msg
            });
          };
        })(this)
      });
    },
    removeFromPaliamentList: function($el) {
      var userId;
      this.lockUI();
      userId = $el.attr('data-user-id');
      app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Are you sure what you want to remove parlamentary?'),
        callback: (function(_this) {
          return function(count) {
            return app.server.party.member.parliament.remove({
              userId: userId,
              success: function(data) {
                return app.screens.modal('common/success-popup', {
                  msg: data.setFlash[0].msg
                });
              }
            });
          };
        })(this)
      });
      return this.reload();
    },
    enableSort: function() {
      this.sortableContainer = document.querySelector('.c-card--parlament-users');
      return this.sortable = Sortable.create(this.sortableContainer, {
        dropBubble: false,
        dragoverBubble: false,
        delay: 500,
        ignore: 'a, img',
        onSort: (function(_this) {
          return function() {
            return _this.sortRequest();
          };
        })(this)
      });
    },
    onDomRefresh: function(event) {
      this.enableSort();
      return [].forEach.call(this.el.querySelectorAll('.c-panel--parlament-user .c-swiped-area'), (function(_this) {
        return function(el) {
          return _this.ps = new ParlamentSwipe(el, {
            onRemoveElement: function() {
              return _this.removeFromPaliamentList($(el).closest('.c-card--parlament-users__item'));
            }
          });
        };
      })(this));
    }
  });
});
