define(['app', 'views/item-view', 'underscore', 'text!templates/port.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    data: {
      port: 'port'
    },
    events: {
      'click .js-fish': 'fish',
      'click .js-buy-ship': 'buyShip'
    },
    preprocessData: (function(_this) {
      return function(data) {
        if (data.port != null) {
          data.fisher = data.port.user.UserProfession;
        }
        return data;
      };
    })(this),
    fish: function(event) {
      var fishingEnergy, loader, ref, ref1;
      this.lockUI();
      loader = this.setLoaderOnElement('.js-fish', app.tr('To fish'));
      fishingEnergy = (ref = this._data.port) != null ? (ref1 = ref.fishing) != null ? ref1.energy : void 0 : void 0;
      return app.server.port.fish({
        success: (function(_this) {
          return function(data) {
            var options;
            if (app.gaEnabled) {
              app.ga('send', 'event', "port", "fishing", "", fishingEnergy);
            }
            _this.unlockUI();
            options = {
              data: data,
              repeat: {
                method: _this.fish,
                context: _this,
                event: event
              }
            };
            return _this.listenToOnce(app.vent, 'short-info:dataLoaded', function() {
              _this.app.screens.modal('port/fishing-popup', options);
              loader.remove();
              return _this.app.vent.trigger('port:fishing_completed', {
                data: data
              });
            });
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            loader.remove();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    },
    buyShip: function(event) {
      var loader;
      this.lockUI();
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Acquire'));
      return app.server.port.ship.buy({
        success: (function(_this) {
          return function(data) {
            var shipPrice;
            if (app.gaEnabled) {
              shipPrice = event.currentTarget.getAttribute('data-ship-price');
              app.ga('send', 'event', 'port', 'vgoldbuy', 'buyship', shipPrice);
            }
            loader.remove();
            return _this.unlockUI();
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            loader.remove();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    },

    /*
    			Center cloud
     */
    centerCloud: function() {},
    onDomRefresh: function() {
      return this.centerCloud();
    }
  });
});
