var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['underscore', 'views/item-view', 'text!templates/profile.tmpl', 'views/components/clusterize-helper', 'views/components/item-type-comparator', 'views/components/strings-helper'], function(_, ItemView, template, ClusterizeHelper, itemTypeComparatorComponent, StringsHelper) {
  var ProfileView;
  ProfileView = (function(superClass) {
    extend(ProfileView, superClass);

    function ProfileView() {
      this.updateInventory = bind(this.updateInventory, this);
      return ProfileView.__super__.constructor.apply(this, arguments);
    }

    ProfileView.prototype.template = template;

    ProfileView.prototype.data = {
      info: 'user.info.short',
      inventory: 'inventory'
    };

    ProfileView.prototype.events = {
      'click .js-item-action': 'itemAction',
      'click .js-buy-slots': 'buySlots',
      'click .c-icon-injury--red': 'injuryPopup',
      'click .c-icon-injury--green': 'energyPopup',
      'click .js-tactics': 'chooseTactics',
      'click .js-vgold-buy': 'onVgoldBuy',
      'click .js-button-energy': 'changeEnergy',
      'click .js-buy-vdollars': 'onVDBuyPlus'
    };

    ProfileView.prototype.changeEnergy = function(event) {
      var activeStateClass, btn, buttons, clonesCount, disabledStateClass, el, energyCost, i, len;
      btn = event.currentTarget;
      buttons = btn.parentNode.querySelectorAll('.square-btn');
      activeStateClass = 'square-btn--active';
      disabledStateClass = 'square-btn--disabled';
      if (!$(btn).hasClass(disabledStateClass)) {
        for (i = 0, len = buttons.length; i < len; i++) {
          el = buttons[i];
          if (el.classList) {
            el.classList.remove(activeStateClass);
          } else {
            el.activeStateClass = el.activeStateClass.replace(new RegExp('(^|\\b)' + activeStateClass.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
          }
        }
        if (btn.classList) {
          btn.classList.add(activeStateClass);
        } else {
          btn.activeStateClass += ' ' + activeStateClass;
        }
        clonesCount = parseInt(btn.innerHTML);
        energyCost = clonesCount * 20;
        $('.js-energy-cost').text(energyCost);
        return this.processItemsAmount(clonesCount);
      }
    };

    ProfileView.prototype.foodCategoryIds = [5, 6, 7, 8, 9, 30, 31, 32, 33];

    ProfileView.prototype.initialize = function() {
      ProfileView.__super__.initialize.apply(this, arguments);
      this.itemTypeComparatorFactory = itemTypeComparatorComponent.itemTypeComparatorFactory;
      this.listenTo(app.vent, 'inventory:inventory_changed', this.updateInventory);
      this.listenTo(app.vent, 'inventory:item_unequipped', (function(_this) {
        return function(event) {
          var ref, ref1;
          if (event.data && ((ref = event.data.itemInfo) != null ? (ref1 = ref.ItemType) != null ? ref1.prestige : void 0 : void 0)) {
            return _this.updatePrestige(-event.data.itemInfo.ItemType.prestige);
          }
        };
      })(this));
      this.listenTo(app.vent, 'inventory:item_equipped', (function(_this) {
        return function(event) {
          var ref, ref1;
          if (event.data && ((ref = event.data.itemInfo) != null ? (ref1 = ref.ItemType) != null ? ref1.prestige : void 0 : void 0)) {
            return _this.updatePrestige(event.data.itemInfo.ItemType.prestige);
          }
        };
      })(this));
      return this.listenTo(app.userEvents, "energy:updated", (function(_this) {
        return function(event) {
          return _this.invalidateCacheAndReload();
        };
      })(this));
    };

    ProfileView.prototype.updatePrestige = function(diff) {
      var initial, ref;
      if (((ref = this._data) != null ? ref.info : void 0) != null) {
        initial = Number(this._data.info.user.User.prestige) || 0;
        this._data.info.user.User.prestige = Math.max(initial + diff, 0);
        return this.render();
      }
    };

    ProfileView.prototype.onVgoldBuy = function() {
      return this.app.shop.gotoVGoldPurchase();
    };

    ProfileView.prototype.onVDBuyPlus = function() {
      return app.screens.dialog('trade-center/currencies');
    };

    ProfileView.prototype.updateInventory = function(event) {
      var ref, ref1, storage;
      if (!event || (event.data == null)) {
        return;
      }
      storage = event.data;
      if (this._data && (((ref = this._data) != null ? (ref1 = ref.inventory) != null ? ref1.storage : void 0 : void 0) != null)) {
        this._data.inventory.storage = storage;
        return this.render();
      }
    };

    ProfileView.prototype.injuryPopup = function(event) {
      var injured, ref, traumasEffect;
      if (((ref = this._data) != null ? ref.info : void 0) != null) {
        this.lockUI();
        injured = this._data.info.injured;
        traumasEffect = this._data.info.traumasEffect;
        return this.app.screens.modal('common/injury-popup', {
          injured: injured,
          traumasEffect: traumasEffect
        });
      }
    };

    ProfileView.prototype.energyPopup = function(event) {
      var items, recoveryTime, ref;
      if (((ref = this._data) != null ? ref.info : void 0) != null) {
        this.lockUI();
        recoveryTime = this._data.info.user.User.delta_recovery_energy;
        items = this._data.info.eatEffects;
        return this.app.screens.modal('common/energy-popup', {
          items: items,
          recoveryTime: recoveryTime
        });
      }
    };

    ProfileView.prototype.serializeData = function() {
      var data, itemTypeComparator;
      data = ProfileView.__super__.serializeData.apply(this, arguments);
      data.equippedItems = {};
      data.unequippedItems = [];
      if (data && (data.inventory != null) && data.inventory.storage.itemsArray.length !== 0) {
        data.inventory.storage.itemsArray.forEach(function(item) {
          if (data.inventory.storage.itemTypes != null) {
            item.ItemType = data.inventory.storage.itemTypes[item.UserItem.item_type_id];
          }
          if (item.UserItem.equipped === 1) {
            return data.equippedItems[item.UserItem.equipped_slot] = item;
          } else {
            return data.unequippedItems.push(item);
          }
        });
        itemTypeComparator = this.itemTypeComparatorFactory(data.inventory.categorySortWeight);
        data.unequippedItems = data.unequippedItems.sort(itemTypeComparator);
      }
      return data;
    };

    ProfileView.prototype.itemAction = function(event) {
      var currentItemCategory, eatEffect, energyDrinkId, i, inventory, itemCategory, itemId, itemType, itemTypeId, j, len, len1, ref, ref1;
      this.lockUI();
      itemId = event.currentTarget.getAttribute('data-item-id');
      itemTypeId = event.currentTarget.getAttribute('data-item-type-id');
      inventory = this._data.inventory;
      if (inventory.energy_drinks_ids != null) {
        ref = inventory.energy_drinks_ids;
        for (i = 0, len = ref.length; i < len; i++) {
          energyDrinkId = ref[i];
          if (parseInt(itemTypeId) === energyDrinkId) {
            return app.screens.modal('inventory/energy-popup', {
              itemId: itemId,
              itemTypeId: itemTypeId
            });
          }
        }
      }
      itemType = _.find(inventory.storage.itemsArray, function(el) {
        return el.ItemType.id.toString() === itemTypeId;
      });
      itemCategory = itemType.ItemType.category_id;
      if (this.foodCategoryIds.indexOf(itemCategory) !== -1 && this._data.info.eatEffects.length > 0) {
        ref1 = this._data.info.eatEffects;
        for (j = 0, len1 = ref1.length; j < len1; j++) {
          eatEffect = ref1[j];
          currentItemCategory = eatEffect.item[0].category_id;
          if (itemCategory === currentItemCategory) {
            return this.inventoryPopup(inventory, itemId, itemTypeId, true, app.tr('Warning food same category'));
          }
        }
      }
      return this.inventoryPopup(inventory, itemId, itemTypeId);
    };

    ProfileView.prototype.inventoryPopup = function(inventory, itemId, itemTypeId, needReload, msg) {
      if (needReload == null) {
        needReload = false;
      }
      return app.screens.modal('inventory/popup', {
        inventory: inventory.storage.itemsArray,
        needReload: needReload,
        itemId: itemId,
        itemTypeId: itemTypeId,
        isResetCert: parseInt(itemTypeId) === inventory.reset_certificate_id ? true : false,
        msg: msg ? msg : ''
      });
    };

    ProfileView.prototype.buySlots = function(event) {
      var size;
      this.lockUI();
      size = this._data.inventory.storage.inventorySize + this._data.inventory.expandInfo.size_step;
      return app.screens.modal('common/confirmation-popup', {
        msg: app.tr('Buy additional slots?'),
        callback: function() {
          return app.server.inventory.buySlots({
            size: size,
            success: function(data) {
              return app.screens.modal('common/success-popup', {
                msg: data.setFlash[0].msg || app.tr("Well done")
              });
            },
            error: function(data) {
              return app.screens.modal('common/info-popup', {
                msg: data.setFlash[0].msg || app.tr("Something went wrong")
              });
            }
          });
        }
      });
    };

    ProfileView.prototype.chooseTactics = function() {
      var clonesCount;
      clonesCount = parseInt(this.$el.find('.square-btn--active').text());
      this.lockUI();
      return app.screens.back((function(_this) {
        return function() {
          return app.screens.dialog('military-base/tactics', {
            areaId: _this.options.areaId,
            postId: _this.options.postId,
            clonesCount: clonesCount
          });
        };
      })(this));
    };

    ProfileView.prototype.processItemsAmount = function(clonesCount) {
      var $container, amountForAmmunitionPerClone, amountForGadgetsPerClone, item, msg, ref, results, slot, uneqippedClones;
      $container = $('.js-items-amount');
      amountForAmmunitionPerClone = 100;
      amountForGadgetsPerClone = 1;
      $('.c-item--warning').removeClass('c-item--warning');
      $('.js-items-amount').text("");
      ref = this._data.equippedItems;
      results = [];
      for (slot in ref) {
        item = ref[slot];
        uneqippedClones = 0;
        if (item.ItemType.type === 'ammunition') {
          uneqippedClones = (clonesCount * amountForAmmunitionPerClone - item.UserItem.quantity) / 100;
          if (uneqippedClones % 1 !== 0) {
            uneqippedClones = Math.floor(uneqippedClones) + 1;
          }
        } else if (item.ItemType.type === 'special' || item.ItemType.type === 'first_aid_kit') {
          uneqippedClones = clonesCount * amountForGadgetsPerClone - item.UserItem.quantity;
        } else if (item.ItemType.category_id === 16 || item.ItemType.category_id === 17 || item.ItemType.category_id === 18 || item.ItemType.category_id === 19 || item.ItemType.category_id === 20 || item.ItemType.category_id === 21 || item.ItemType.category_id === 22 || item.ItemType.category_id === 23) {
          uneqippedClones = clonesCount - item.UserItem.strength;
        }
        if (uneqippedClones > 0) {
          $("[data-slot-id='" + slot + "'] .c-item").addClass('c-item--warning');
          msg = StringsHelper.replace({
            '{{itemName}}': item.ItemType.name,
            '{{clonesCount}}': clonesCount - uneqippedClones
          }, app.tr("Youll have {{itemName}} only for {{clonesCount}} clones"));
          results.push($(".js-items-amount").append("<div>" + msg + "</div>"));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    ProfileView.prototype.onDomRefresh = function() {
      var clonesCount;
      $('.square-btn--disabled').css('background-color', '#57001D');
      $(app.screens.current.el.parentElement).removeClass('js-current-screen').addClass('js-current-screen');
      $('.js-current-screen .c-content-wrapper').scroll(function() {
        var position;
        position = $('.js-current-screen .c-content-wrapper').scrollTop() + $('.js-current-screen .c-content-wrapper').innerHeight() - 44;
        $(".c-footer--fixed.js-tactics").css("position", "absolute");
        $(".c-footer--fixed.js-tactics").css("top", position);
        return $(".c-footer--fixed.js-tactics").css("display", "block");
      });
      [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        var position, value;
        value = el.innerHTML;
        position = value.indexOf(':') + 1;
        return el.innerHTML = position === 0 ? value : value.slice(position);
      });
      clonesCount = parseInt(this.$el.find('.square-btn--active').text());
      return this.processItemsAmount(clonesCount);
    };

    return ProfileView;

  })(ItemView);
  return ProfileView;
});
