define(['app', 'views/item-view', 'underscore', 'text!templates/quests/quest.tmpl'], function(app, ItemView, _, template) {
  return ItemView.extend({
    template: template,
    _quest: null,
    data: {
      info: {
        pointer: 'quests.info',
        questId: function() {
          return this.options.questId;
        }
      }
    },
    events: {
      'click .js-take-quest': 'takeQuest',
      'click .js-collect-prize': 'collectPrize'
    },
    preprocessData: function(data) {
      if ((data != null ? data.info : void 0) != null) {
        this._quest = data.info.quest;
      }
      return data;
    },
    takeQuest: function(event) {
      app.showLoader();
      this.lockUI();
      return app.server.quests.takeQuest({
        questId: this.options.questId,
        success: (function(_this) {
          return function(data) {
            _this.unlockUI();
            app.hideLoader();
            if (_this._quest) {
              app.vent.trigger('quests:quest_taken', {
                data: {
                  id: _this._quest.Quest.id,
                  cid: _this._quest.Quest.cid,
                  title: _this._quest.Quest.title
                }
              });
            }
            app.screens.main('quests');
            return _this.reload();
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            app.hideLoader();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    },
    collectPrize: function(event) {
      app.showLoader();
      this.lockUI();
      return app.server.quests.completeQuest({
        questId: this.options.questId,
        success: (function(_this) {
          return function(data) {
            _this.unlockUI();
            app.hideLoader();
            app.vent.trigger('quests:quest_completed', {
              data: {
                id: _this._quest.Quest.id,
                cid: _this._quest.Quest.cid,
                title: _this._quest.Quest.title
              }
            });
            return app.screens.main('quests');
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            app.hideLoader();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    },
    sendGAView: function() {
      if (this.app.config.googleAnalyticsEnabled && this.app.ga) {
        return this.app.ga('send', 'pageview', {
          page: '/scr/' + this.options.id + '/' + this.options.questId,
          title: ''
        });
      }
    }
  });
});
