var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/settings.tmpl'], function(app, ItemView, _, template) {
  var SettingsView;
  SettingsView = (function(superClass) {
    extend(SettingsView, superClass);

    function SettingsView() {
      this._switchSocialToggle = bind(this._switchSocialToggle, this);
      this._linkAccount = bind(this._linkAccount, this);
      this.preprocessData = bind(this.preprocessData, this);
      return SettingsView.__super__.constructor.apply(this, arguments);
    }

    SettingsView.prototype.template = template;

    SettingsView.prototype.className = 'no-bg-image';

    SettingsView.prototype.data = {
      info: 'user.info.short',
      email: 'user.email',
      languages: 'user.languages',
      socialAccounts: 'social.getLinkedAccounts'
    };

    SettingsView.prototype.events = {
      'click .js-select-language': 'selectLanguage',
      'click .js-logout': 'logout',
      'click .js-contact': 'contact',
      'click .js-provider': 'linkAccount',
      'click .js-policy': 'policy',
      'click .js-rules': 'rules'
    };

    SettingsView.prototype.socialProviders = {
      'vk': {
        type: 'vk',
        'name': 'Vkontakte'
      },
      'fb': {
        type: 'fb',
        'name': 'Facebook'
      },
      'google': {
        type: 'google',
        'name': 'Google+'
      }
    };

    SettingsView.prototype.preprocessData = function(data) {
      var account, code, i, id, languages, len, ref, ref1, title;
      languages = [];
      id = 1;
      ref = data['languages']['languages'];
      for (code in ref) {
        title = ref[code];
        languages.push({
          code: code,
          title: title,
          id: id
        });
        id = id + 1;
      }
      this.languages = new Backbone.Collection(languages);
      this.accounts = {};
      ref1 = data.socialAccounts.accounts;
      for (i = 0, len = ref1.length; i < len; i++) {
        account = ref1[i];
        this.accounts[account.type] = account;
      }
      data.accounts = this.accounts;
      data.socialProviders = this.socialProviders;
      return data;
    };

    SettingsView.prototype.selectLanguage = function(event) {
      return app.screens.dialog('common/select-dialog-cb', {
        collection: this.languages,
        callback: (function(_this) {
          return function(itemId, callback) {
            var localization;
            app.showLoader();
            localization = _this.languages.get(itemId).get('code');
            _this.$('.js-localization').html(_this.languages.get(itemId).get("title"));
            return app.server.user.setLanguage({
              localization: localization,
              success: function(data) {
                if (app.translations.getLocale() !== data.locale) {
                  return app.translations.setLocaleAndFetch(localization, function() {
                    app.server.invalidateAllCache();
                    app.menu.view.render();
                    app.menu.view.reload();
                    app.shortInfo.view.reload();
                    return app.screens.back();
                  });
                } else {
                  return app.screens.back(function() {
                    return app.hideLoader();
                  });
                }
              },
              error: function() {
                return app.screens.back(function() {
                  return app.hideLoader();
                });
              }
            });
          };
        })(this)
      });
    };

    SettingsView.prototype.policy = function(event) {
      return app.screens.dialog("login/policy");
    };

    SettingsView.prototype.rules = function() {
      return app.screens.dialog("login/rules");
    };

    SettingsView.prototype.logout = function() {
      localStorage.removeItem('tutorial_skipped');
      return app.server.logout({
        success: function(data) {
          return app.screens.main('login');
        },
        error: function(data) {
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          }, app.screens.main('login'));
        }
      });
    };

    SettingsView.prototype.contact = function(event) {};

    SettingsView.prototype.linkAccount = function(event) {
      var provider, type;
      this.lockUI();
      type = event.currentTarget.getAttribute('data-type');
      if (!this.socialProviders.hasOwnProperty(type)) {
        app.screens.modal('common/info-popup', {
          msg: app.tr('Invalid social network')
        });
        return;
      }
      if (this.accounts.hasOwnProperty(type)) {
        this._switchSocialToggle(type);
        return;
      }
      provider = app.socialService.getProvider(type);
      if (!provider) {
        app.screens.modal('common/info-popup', {
          msg: app.tr('Invalid social network')
        });
        return;
      }
      return provider.reauth(true).then((function(_this) {
        return function(data) {
          return _this._linkAccount(data.user, provider);
        };
      })(this), (function(_this) {
        return function() {
          return app.screens.modal('common/info-popup', {
            msg: app.tr('Authentication failed')
          });
        };
      })(this));
    };

    SettingsView.prototype._linkAccount = function(userData, provider) {
      return this.app.server.social.linkAccount({
        provider: provider.getType(),
        socialUser: userData,
        token: provider.getCurrentToken(),
        success: (function(_this) {
          return function(data) {
            var adjustEvent, key;
            if (data && (data != null ? data.user_id : void 0)) {
              if (app.adjustEnabled) {
                key = 'nl6d3a';
                adjustEvent = new app.adjust.AdjustEvent(key);
                adjustEvent.addPartnerParameter('event_key', key);
                app.adjust.Adjust.trackEvent(adjustEvent);
              }
              _this.accounts[data.provider] = data;
              _this._switchSocialToggle(data.provider);
              return _this.reload();
            } else {
              return _this.app.screens.modal('common/info-popup', {
                msg: app.tr('Authentication failed')
              });
            }
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            if (data && data.error_code === 'ACCOUNT_LINKED') {
              _this.app.screens.modal('common/info-popup', {
                msg: data.error_message
              });
              return;
            }
            return _this.app.screens.modal('common/info-popup', {
              msg: app.tr('Authentication failed')
            });
          };
        })(this)
      });
    };

    SettingsView.prototype._switchSocialToggle = function(type) {
      var title, toggle;
      toggle = this.$('#js-social-' + type + '-toggle');
      title = this.$('#js-social-' + type + '-title');
      if (this.accounts[type] != null) {
        toggle.removeClass('o-icon--right-arrow').addClass('o-icon--check-green');
        return title.text(this.accounts[type].user_name);
      }
    };

    return SettingsView;

  })(ItemView);
  return SettingsView;
});
