var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/shop/vgold.tmpl'], function(app, ItemView, _, template) {
  var ShopVgoldView;
  ShopVgoldView = (function(superClass) {
    extend(ShopVgoldView, superClass);

    function ShopVgoldView() {
      this.onProviderChange = bind(this.onProviderChange, this);
      return ShopVgoldView.__super__.constructor.apply(this, arguments);
    }

    ShopVgoldView.prototype.template = template;

    ShopVgoldView.prototype.events = {
      'click .js-buy': 'onVGoldBuy',
      'click .js-change-provider': 'onProviderChange'
    };

    ShopVgoldView.prototype._currentProvider = null;

    ShopVgoldView.prototype._providers = {};

    ShopVgoldView.prototype.initialize = function(options) {
      ShopVgoldView.__super__.initialize.call(this, options);
      this._providers = app.paymentsService.getEnabledProviders();
      this._currentProvider = this._providers.length ? this._providers[0] : null;
      this._currencyList = this._currentProvider ? app.paymentsService.provider(this._currentProvider).loadCurrencyList() : [];
      return this.listenTo(app.vent, 'store:re-refreshed', (function(_this) {
        return function() {
          return _this.render();
        };
      })(this));
    };

    ShopVgoldView.prototype._storeError = function(err) {
      if (app.config.debug) {
        console.log(err);
      }
      this.unlockUI();
      return app.hideLoader();
    };

    ShopVgoldView.prototype.serializeData = function() {
      return {
        currentProvider: this._currentProvider,
        providers: this._providers,
        currencyList: this._currencyList
      };
    };

    ShopVgoldView.prototype.onProviderChange = function(event) {
      var provider;
      provider = event.currentTarget.getAttribute('data-provider');
      if (app.paymentsService.isEnabled(provider) && provider !== this._currentProvider) {
        this._currentProvider = provider;
        return this.render();
      }
    };

    ShopVgoldView.prototype.onVGoldBuy = function(event) {
      var adjustEvent, i, itemGAAlias, itemGACurrency, itemGAPrice, itemGAsku, itemId, itemVgold, j, len, len1, pageId, provider, ref, ref1, vGoldItem;
      pageId = this.options.id;
      provider = app.paymentsService.provider(this._currentProvider);
      itemId = event.currentTarget.getAttribute('data-item-id');
      itemVgold = provider.getCurrencyVgold(itemId);
      ref = this._currencyList;
      for (i = 0, len = ref.length; i < len; i++) {
        vGoldItem = ref[i];
        if (vGoldItem.id === parseInt(itemId) || parseInt(itemVgold) === parseInt(vGoldItem.vgold)) {
          itemGAPrice = vGoldItem.price_ga;
          itemGACurrency = vGoldItem.currency_ga;
          itemGAAlias = vGoldItem.alias;
        }
      }
      ref1 = this._currencyList;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        vGoldItem = ref1[j];
        if (vGoldItem.id === parseInt(itemId) || parseInt(itemVgold) === parseInt(vGoldItem.vgold)) {
          itemGAPrice = vGoldItem.price_ga;
          itemGACurrency = vGoldItem.currency_ga;
          itemGAAlias = vGoldItem.alias;
          itemGAsku = vGoldItem.consumable;
        }
      }
      if (app.gaEnabled) {
        app.ga('send', 'pageview', {
          page: '/scr/' + pageId + '/vgold-purchase/' + itemVgold,
          title: ''
        });
        app.ga('send', 'event', 'shop', 'vgoldbuybutton', itemId, Number(itemVgold));
      }
      if (app.adjustEnabled) {
        adjustEvent = new app.adjust.AdjustEvent("vgoldbuybutton");
        app.adjust.Adjust.trackEvent(adjustEvent);
      }
      app.showLoader();
      return provider.order(itemId).then(function(data) {
        var msg;
        app.hideLoader();
        app.shortInfo.refresh();
        if (data && data.status === 'uncompleted') {
          msg = app.tr('Purchase uncompleted. Please try to refresh purchases or contact us');
          msg += "\n (purchase id: " + data.transaction_id + ')';
          return app.screens.modal('common/info-popup', {
            msg: msg
          });
        } else {
          if (app.gaEnabled) {
            app.ga('send', 'pageview', {
              page: '/scr/' + pageId + '/vgold-success/' + itemVgold,
              title: ''
            });
            app.ga('send', 'event', 'shop', 'vgoldbuy', itemId, Number(itemVgold));
            app.ga('ecommerce:addTransaction', {
              'id': data.transaction_id,
              'revenue': itemGAPrice,
              'currency': itemGACurrency
            });
            app.ga('ecommerce:addItem', {
              'id': data.transaction_id,
              'name': itemGAAlias,
              'category': 'vgold',
              'price': itemGAPrice,
              'currency': itemGACurrency,
              'sku': itemGAsku
            });
          }
          if (app.adjustEnabled) {
            adjustEvent = new app.adjust.AdjustEvent("vgoldbuy");
            adjustEvent.setRevenue(itemGAPrice, itemGACurrency);
            app.adjust.Adjust.trackEvent(adjustEvent);
          }
          return app.screens.modal('common/success-popup', {
            msg: app.tr("Purchase completed"),
            currency: 'vgold',
            amount: data.amount
          });
        }
      }, function(data) {
        var ref2, ref3;
        app.hideLoader();
        if (data != null ? data.isVerified : void 0) {
          return app.screens.modal('shop/fail-popup', {
            msg: (data != null ? (ref2 = data.error) != null ? ref2.message : void 0 : void 0) || app.tr('Purchase failed')
          });
        } else {
          return app.screens.modal('common/info-popup', {
            msg: (data != null ? (ref3 = data.error) != null ? ref3.message : void 0 : void 0) || app.tr('Purchase failed')
          });
        }
      });
    };


    /*
    			Center cloud
     */

    ShopVgoldView.prototype.centerCloud = function() {};

    ShopVgoldView.prototype.onDomRefresh = function() {
      return this.centerCloud();
    };

    return ShopVgoldView;

  })(ItemView);
  return ShopVgoldView;
});
