var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['underscore', 'views/item-view', 'text!templates/short-info.tmpl', 'moment', 'jquery.countdown', 'views/components/currency'], function(_, ItemView, template, moment, countdown, formatCurrency) {
  var ShortInfoView;
  ShortInfoView = (function(superClass) {
    extend(ShortInfoView, superClass);

    function ShortInfoView() {
      this.initialize = bind(this.initialize, this);
      return ShortInfoView.__super__.constructor.apply(this, arguments);
    }

    ShortInfoView.prototype.template = template;

    ShortInfoView.prototype.events = {
      'click .c-person__injury--red': 'injuryPopup',
      'click .c-person__injury--green': 'energyPopup',
      'click .js-vgold-buy': 'onVgoldBuy',
      'click .js-energy-buy': 'onEnergyBuy',
      'click .js-health': 'onHealth'
    };

    ShortInfoView.prototype.data = {
      info: 'user.info.short'
    };

    ShortInfoView.prototype.initialize = function(options) {
      var menuSlide, menuSlidePos;
      this.options = options;
      this.listenTo(app.userEvents, "energy:updated", this.updateEnergyField);
      $('body').off('click', '.js-vgold-buy');
      menuSlide = 0;
      menuSlidePos = 0;
      $('body').on('touchstart', (function(_this) {
        return function(ev) {
          var wWidth;
          wWidth = $(window).width();
          menuSlidePos = ev.originalEvent.changedTouches[0].clientX;
          if (menuSlidePos < wWidth / 2) {
            return menuSlide = 1;
          } else if (menuSlidePos > wWidth / 2) {
            return menuSlide = -1;
          }
        };
      })(this)).on('touchend', (function(_this) {
        return function(ev) {
          var wWidth, x;
          x = ev.originalEvent.changedTouches[0].clientX;
          wWidth = $(window).width();
          if (menuSlide === 1 && (x - menuSlidePos > wWidth / 5)) {
            app.menu.open();
          } else if (menuSlide === -1 && (menuSlidePos - x > wWidth / 5)) {
            app.menu.close();
          }
          menuSlide = 0;
          return menuSlidePos = 0;
        };
      })(this));
      this.listenTo(app.userEvents, "health:updated", this.updateHealthField);
      return ShortInfoView.__super__.initialize.call(this, this.options);
    };

    ShortInfoView.prototype.preprocessData = function(data) {
      if (data.info.injured.height > 0 || data.info.injured.light > 0 || data.info.injured.medium > 0) {
        data.info.isInjured = true;
      }
      return data;
    };

    ShortInfoView.prototype.injuryPopup = function(event) {
      var injured, ref, traumasEffect;
      if (((ref = this._data) != null ? ref.info : void 0) != null) {
        this.lockUI();
        injured = this._data.info.injured;
        traumasEffect = this._data.info.traumasEffect;
        return this.app.screens.modal('common/injury-popup', {
          injured: injured,
          traumasEffect: traumasEffect
        });
      }
    };

    ShortInfoView.prototype.energyPopup = function(event) {
      var items, recoveryTime, ref;
      if (((ref = this._data) != null ? ref.info : void 0) != null) {
        this.lockUI();
        recoveryTime = this._data.info.user.User.delta_recovery_energy;
        items = this._data.info.eatEffects;
        return this.app.screens.modal('common/energy-popup', {
          items: items,
          recoveryTime: recoveryTime
        });
      }
    };

    ShortInfoView.prototype.onEnergyBuy = function(event) {
      return app.server.inventory.getEnergyDrink({
        success: function(data) {
          if ((data != null) && data.energy_drink !== 0) {
            return app.screens.modal('inventory/energy-popup', {
              itemTypeId: data.energy_drink.item_type_id,
              itemId: data.energy_drink.item_id
            });
          } else {
            return app.screens.dialog('shop');
          }
        },
        error: function(data) {
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong!")
          });
        }
      });
    };

    ShortInfoView.prototype.onHealth = function(event) {
      if (this._data.info.user.User.health < this._data.info.user.User.max_health || this._data.info.isInjured) {
        return app.screens.dialog('hospital');
      }
    };

    ShortInfoView.prototype.onDomRefresh = function() {
      return [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        return el.innerHTML = formatCurrency(el.innerHTML);
      });

      /* turn off tooltip
      			if @_data?.info?
      				a = moment()
      				b = moment(@_data.info.user.User.last_energy_update, 'seconds')
      				 * Formula - min(now - last_energy_update, delta_recovery_energy)
      				time = Math.min(a.diff(b), @_data.info.user.User.delta_recovery_energy)
      				 * Countdown date format
      				finalDate = moment(moment(moment()).add(time, 's')).toDate()
      				
      				self = @
      				@$el.find('.c-menu__tooltip').countdown(finalDate, (event) ->
      					$(@).text(
      						event.strftime('%M:%S')
      					)
      				).on('finish.countdown', (ev) ->
      					app.server.user.info.short.cache.invalidate()
      					app.menu.reload()
      					app.shortInfo.view.reload()
      				)
       */
    };

    ShortInfoView.prototype.energyTooltip = function() {
      return this.$el.find('.c-menu__tooltip').toggle();
    };

    ShortInfoView.prototype.updateEnergyField = function(newEnergy) {
      return this.$el.find('.js-energy').html(newEnergy);
    };

    ShortInfoView.prototype.updateHealthField = function(newHealth) {
      return this.$el.find('.js-health-update').html(newHealth);
    };

    ShortInfoView.prototype.onDataLoaded = function(data) {
      var city, levelChange, prevUserLevel, userLevel;
      city = data.info.cities[data.info.infos.city_id];
      userLevel = data.info.currentLevel;
      prevUserLevel = window.localStorage.getItem('user_level') || 0;
      levelChange = userLevel - prevUserLevel;
      if (levelChange > 0) {
        app.ga('set', 'metric1', levelChange);
        window.localStorage.setItem('user_level', levelChange);
      }
      return ShortInfoView.__super__.onDataLoaded.call(this, data);
    };

    ShortInfoView.prototype.onVgoldBuy = function() {
      return this.app.shop.gotoVGoldPurchase();
    };

    return ShortInfoView;

  })(ItemView);
  return ShortInfoView;
});
