var bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/stadium/choose-trainer.tmpl', 'chosen', 'text!templates/stadium/blocks/choose-trainer-list.tmpl', 'views/components/lazy-loading', 'views/common/lazy-loading-view', 'views/components/strings-helper'], function(app, ItemView, _, template, Chosen, listTemplate, LazyLoading, CommonLazyView, Strings) {
  var ChooseTrainerView;
  ChooseTrainerView = (function(superClass) {
    extend(ChooseTrainerView, superClass);

    function ChooseTrainerView() {
      this._loadData = bind(this._loadData, this);
      this._loadDataSearche = bind(this._loadDataSearche, this);
      this.sortChange = bind(this.sortChange, this);
      this.onRender = bind(this.onRender, this);
      this.initialize = bind(this.initialize, this);
      return ChooseTrainerView.__super__.constructor.apply(this, arguments);
    }

    ChooseTrainerView.prototype.template = template;

    ChooseTrainerView.prototype.listTemplate = _.template(listTemplate);

    ChooseTrainerView.prototype.pointer = 'trainers';

    ChooseTrainerView.prototype.data = {
      trainers: {
        pointer: 'stadium.trainersByParam',
        page: 1,
        sortField: 'price',
        order: 'asc'
      }
    };

    ChooseTrainerView.prototype.events = {
      'click .js-choose-program': 'chooseProgram',
      'click .js-next-page': 'nextPage',
      'click .js-prev-page': 'prevPage',
      'change .js-name': 'onNameChange'
    };

    ChooseTrainerView.prototype.initialize = function(options) {
      var order, ref, sortField;
      this.options = options;
      if (localStorage.trainers_sort != null) {
        ref = JSON.parse(localStorage.trainers_sort), sortField = ref.sortField, order = ref.order;
        this.data.trainers.sortField = sortField;
        this.data.trainers.order = order;
      }
      this.data.trainers.page = 1;
      this.data.trainers.sortField = 'price';
      this.data.trainers.order = 'asc';
      this.data.trainers.coefficient = this.options.coefficient != null ? this.options.coefficient : null;
      this.data.trainers.name = this.options.name != null ? this.options.name : null;
      return ChooseTrainerView.__super__.initialize.call(this, this.options);
    };

    ChooseTrainerView.prototype._fixChosenWidth = function() {};

    ChooseTrainerView.prototype.onRender = function() {
      $('.js-sort-select').on('change', this.sortChange);
      $('.js-sort-select').val(this.data.trainers.sortField + '-' + this.data.trainers.order);
      this._fixChosenWidth();
      if (this.options.name != null) {
        return $('.js-name').val(this.options.name);
      }
    };

    ChooseTrainerView.prototype.onNameChange = function(event) {
      var name;
      name = event.currentTarget.value;
      name = Strings.filterUsername(name);
      event.currentTarget.value = name;
      if (name.length > 2 || !name) {
        this._name = name;
        return this._reloadMainList(name, 1);
      }
    };

    ChooseTrainerView.prototype._reloadMainList = function(name, page) {
      var container, self;
      container = this.el.querySelector('#js-main-list-content');
      self = this;
      self.lockUI();
      self._showListLoader(container);
      return $.when(self._clearAllLists(), self._loadDataSearche(name, page)).then(function(ign, data) {
        self.unlockUI();
        self._hideListLoader(container);
        return self._renderList(container, data, data.page, data.hasNext);
      });
    };

    ChooseTrainerView.prototype.sortChange = function(event, params) {
      var value;
      value = $(event.target).val();
      if (value === 'price-asc') {
        this.data.trainers.sortField = 'price';
        this.data.trainers.order = 'asc';
      } else if (value === 'price-desc') {
        this.data.trainers.sortField = 'price';
        this.data.trainers.order = 'desc';
      }
      localStorage.setItem('trainers_sort', JSON.stringify({
        sortField: this.data.trainers.sortField,
        order: this.data.trainers.order
      }));
      this.data.trainers.page = 1;
      this.activeRequest = false;
      this.hasNext = true;
      return this.loadData();
    };

    ChooseTrainerView.prototype.nextPage = function() {
      this.data.trainers.page++;
      return this.loadData();
    };

    ChooseTrainerView.prototype.prevPage = function() {
      this.data.trainers.page--;
      return this.loadData();
    };

    ChooseTrainerView.prototype.chooseProgram = function(event) {
      var coefficient, currency, licenseId, price, programId;
      programId = event.currentTarget.getAttribute('data-program-id');
      licenseId = event.currentTarget.getAttribute('data-license-id');
      price = event.currentTarget.getAttribute('data-price');
      currency = event.currentTarget.getAttribute('data-currency');
      coefficient = event.currentTarget.getAttribute('data-coefficient');
      app.screens.back(function() {
        return app.screens.back;
      });
      return app.vent.trigger('stadium:trainer_chosen', {
        programId: programId,
        licenseId: licenseId,
        price: price,
        currency: currency,
        coefficient: coefficient
      });
    };

    ChooseTrainerView.prototype._loadDataSearche = function(name, page) {
      var deferred, self;
      deferred = $.Deferred();
      self = this;
      app.server.stadium.trainersByParam({
        page: page || 1,
        sortField: this.data.trainers.sortField,
        order: this.data.trainers.order,
        coefficient: this.options.coefficient,
        name: name,
        success: (function(_this) {
          return function(data) {
            var resultData;
            resultData = data;
            resultData.page = page;
            resultData.hasNext = data.has_next || false;
            _this.hasNext = data.paging.CoachProgram.nextPage || false;
            _this.data.trainers.page++;
            return deferred.resolve(resultData);
          };
        })(this),
        error: function() {
          return deferred.resolve({
            users: [],
            page: 1,
            hasNext: false
          });
        }
      });
      return deferred;
    };

    ChooseTrainerView.prototype._loadData = function(page) {
      var deferred, self;
      deferred = $.Deferred();
      self = this;
      app.server.stadium.trainersByParam({
        page: page || 1,
        sortField: this.data.trainers.sortField,
        order: this.data.trainers.order,
        coefficient: this.options.coefficient,
        success: (function(_this) {
          return function(data) {
            var resultData;
            resultData = data;
            resultData.page = page;
            resultData.hasNext = data.has_next || false;
            _this.hasNext = data.paging.CoachProgram.nextPage || false;
            _this.data.trainers.page++;
            return deferred.resolve(resultData);
          };
        })(this),
        error: function() {
          return deferred.resolve({
            users: [],
            page: 1,
            hasNext: false
          });
        }
      });
      return deferred;
    };

    return ChooseTrainerView;

  })(CommonLazyView);
  return ChooseTrainerView;
});
