define(['app', 'views/item-view', 'underscore', 'text!templates/trade-center/currency-lot-settings.tmpl', 'views/components/currency', 'views/common/precision-check', 'views/components/strings-helper'], function(app, ItemView, _, template, formatCurrency, precisionCheck, StringsHelper) {
  return ItemView.extend({
    template: template,
    events: {
      'click .js-save-lot': 'changeLot',
      'change .js-items-price': 'priceChangeHandler'
    },
    elements: {},
    changeLot: function(event) {
      var exchangeId, price;
      this.lockUI();
      exchangeId = this._data.model['id'];
      price = parseFloat(this.elements.priceInput.value);
      if (!precisionCheck(price)) {
        return;
      }
      return app.server.tradeCenter.editCurrencyLot({
        exchangeId: exchangeId,
        price: price.toFixed(2),
        success: function(data) {
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Lot edited")
          }, app.screens.back());
        },
        error: function(data) {
          return app.screens.modal('common/info-popup', {
            msg: data.setFlash[0].msg || app.tr("Something went wrong")
          });
        }
      });
    },
    onRender: function() {
      app.chat.updateMsgCounter();
      return this.$('.js-items-price').change((function(_this) {
        return function(event) {
          var price;
          price = parseFloat($(event.currentTarget).val());
          if (!isNaN(price)) {
            return $(event.currentTarget).val(price.toFixed(2));
          } else {
            return $(event.currentTarget).val(parseFloat(_this.model.get("price")).toFixed(2));
          }
        };
      })(this));
    },
    onDomRefresh: function() {
      [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        if (el.dataset.precision) {
          return el.innerHTML = formatCurrency(el.innerHTML, void 0, el.dataset.precision);
        } else {
          return el.innerHTML = formatCurrency(el.innerHTML);
        }
      });
      this.elements.amountInput = this.el.querySelector('.js-items-amount');
      this.elements.priceInput = this.el.querySelector('.js-items-price');
      this.elements.helpElement = this.el.querySelector('#js-help');
      this.updateHelpText();
      return app.chat.updateMsgCounter();
    },
    priceChangeHandler: function(event) {
      var price;
      price = parseFloat(event.currentTarget.value);
      if (!isNaN(price)) {
        event.currentTarget.value = price.toFixed(2);
      } else {
        event.currentTarget.value = "0.00";
      }
      return this.updateHelpText();
    },
    updateHelpText: function() {
      var amount, currencyIcon, price, text, vDollarIcon, vGoldIcon;
      amount = parseFloat(this.elements.amountInput.value);
      price = parseFloat(this.elements.priceInput.value);
      if (!(amount > 0 && price > 0)) {
        this.elements.helpElement.innerHTML = "";
        return;
      }
      vDollarIcon = '<i class="o-icon--vdollar u-mr-5"></i>';
      vGoldIcon = '<i class="o-icon--vgold u-mr-5"></i>';
      if (this._data.model.currency_type === 2) {
        currencyIcon = vDollarIcon;
      } else {
        currencyIcon = vGoldIcon;
      }
      text = StringsHelper.replace({
        '{{amount}}': amount,
        '{{price}}': price,
        '{{vDollarIcon}}': vDollarIcon,
        '{{vGoldIcon}}': vGoldIcon,
        '{{currencyIcon}}': currencyIcon
      }, app.tr('You sell total {{amount}} {{currencyIcon}} for price of {{price}} {{vDollarIcon}} for 1 {{vGoldIcon}}'));
      return this.elements.helpElement.innerHTML = text;
    }
  });
});
