var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'text!templates/trade-center/sell-currency.tmpl', 'views/components/currency', 'views/common/precision-check', 'views/components/strings-helper'], function(app, ItemView, _, template, formatCurrency, precisionCheck, StringsHelper) {
  var SellItemView;
  SellItemView = (function(superClass) {
    extend(SellItemView, superClass);

    function SellItemView() {
      return SellItemView.__super__.constructor.apply(this, arguments);
    }

    SellItemView.prototype.template = template;

    SellItemView.prototype.events = {
      'click .js-create-lot': 'createLot',
      'click .js-choose-all-items': 'chooseAllItems',
      'change .js-items-amount': 'amountChangeHandler',
      'change .js-items-price': 'priceChangeHandler',
      'keyup .js-items-amount': 'calcPrice',
      'keyup .js-items-price': 'calcPrice'
    };

    SellItemView.prototype.elements = {};

    SellItemView.prototype.onDomRefresh = function() {
      [].forEach.call(this.el.querySelectorAll('.js-currency'), function(el) {
        if (el.dataset.precision) {
          return el.innerHTML = formatCurrency(el.innerHTML, void 0, el.dataset.precision);
        } else {
          return el.innerHTML = formatCurrency(el.innerHTML);
        }
      });
      this.elements.amountInput = this.el.querySelector('.js-items-amount');
      this.elements.priceInput = this.el.querySelector('.js-items-price');
      return this.elements.helpElement = this.el.querySelector('#js-help');
    };

    SellItemView.prototype.createLot = function(event) {
      var amount, loader, price;
      this.lockUI();
      amount = this.elements.amountInput.value ? parseFloat(this.elements.amountInput.value) : 0;
      price = this.elements.priceInput.value ? parseFloat(this.elements.priceInput.value) : 0;
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Selling'));
      if (precisionCheck(price)) {
        return app.server.tradeCenter.createCurrencyLot({
          amount: amount.toFixed(2),
          price: price.toFixed(2),
          type: this.options.currency,
          success: (function(_this) {
            return function(data) {
              _this.unlockUI();
              loader.remove();
              return app.screens.back();
            };
          })(this),
          error: (function(_this) {
            return function(data) {
              var options;
              loader.remove();
              _this.unlockUI();
              options = {
                msg: data.setFlash[0].msg || app.tr("Something went wrong"),
                onBack: function() {}
              };
              return app.screens.modal('common/info-popup', options);
            };
          })(this)
        });
      }
    };

    SellItemView.prototype.amountChangeHandler = function(event) {
      var amount, userHave;
      amount = parseFloat(event.currentTarget.value);
      if (amount <= 0) {
        amount = 0.01;
      }
      $(event.currentTarget).val(amount);
      userHave = parseFloat(this.options.available);
      if (isNaN(amount)) {
        event.currentTarget.value = "0.00";
      } else if (amount > userHave) {
        event.currentTarget.value = userHave.toFixed(2);
      } else {
        event.currentTarget.value = amount.toFixed(2);
      }
      return this.updateHelpText();
    };

    SellItemView.prototype.priceChangeHandler = function(event) {
      var price;
      price = parseFloat(event.currentTarget.value);
      if (!isNaN(price)) {
        event.currentTarget.value = price.toFixed(2);
      } else {
        event.currentTarget.value = "0.00";
      }
      return this.updateHelpText();
    };

    SellItemView.prototype.updateHelpText = function() {
      var amount, currencyIcon, price, text, vDollarIcon, vGoldIcon;
      amount = parseFloat(this.elements.amountInput.value);
      price = parseFloat(this.elements.priceInput.value);
      if (!(amount > 0 && price > 0)) {
        this.elements.helpElement.innerHTML = "";
        return;
      }
      vDollarIcon = '<i class="o-icon--vdollar u-mr-5"></i>';
      vGoldIcon = '<i class="o-icon--vgold u-mr-5"></i>';
      if (this.options.currency === 'vd') {
        currencyIcon = vDollarIcon;
      } else {
        currencyIcon = vGoldIcon;
      }
      text = StringsHelper.replace({
        '{{amount}}': amount,
        '{{price}}': price,
        '{{vDollarIcon}}': vDollarIcon,
        '{{vGoldIcon}}': vGoldIcon,
        '{{currencyIcon}}': currencyIcon
      }, app.tr('You sell total {{amount}} {{currencyIcon}} for price of {{price}} {{vDollarIcon}} for 1 {{vGoldIcon}}'));
      return this.elements.helpElement.innerHTML = text;
    };

    SellItemView.prototype.chooseAllItems = function() {
      this.$('.js-items-amount').val(this._data.options.available);
      return this.$('.js-items-amount').trigger('change');
    };

    SellItemView.prototype.calcPrice = function() {
      var currencyIcon, formattedLotPrice, itemsCount, lotPrice, pricePerItem, userHave, vDollarIcon, vGoldIcon;
      itemsCount = parseFloat(this.$el.find('.js-items-amount').val());
      pricePerItem = parseFloat(this.$el.find('.js-items-price').val());
      userHave = parseInt(this.options.itemAmount);
      vDollarIcon = '<i class="o-icon--vdollar u-ml-5 u-mr-5"></i>';
      vGoldIcon = '<i class="o-icon--vgold u-ml-5 u-mr-5"></i>';
      if (this.options.currency === 'vd') {
        currencyIcon = vGoldIcon;
      } else {
        currencyIcon = vDollarIcon;
      }
      if (isNaN(itemsCount)) {
        itemsCount = 0;
      }
      if (itemsCount > userHave) {
        itemsCount = userHave;
      }
      if (isNaN(pricePerItem)) {
        pricePerItem = 0;
      }
      lotPrice = parseFloat(itemsCount * pricePerItem);
      formattedLotPrice = this.templateHelpers().formatNumber(lotPrice);
      return this.$el.find('.js-lot-price').html(formattedLotPrice + currencyIcon);
    };

    return SellItemView;

  })(ItemView);
  return SellItemView;
});
