var extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

define(['app', 'views/item-view', 'underscore', 'views/components/item-type-comparator', 'text!templates/trade-center/sell-item.tmpl', 'text!templates/trade-center/blocks/min-max-price.tmpl'], function(app, ItemView, _, itemTypeComparatorComponent, template, minMaxTemplate) {
  var SellItemCorporationView;
  SellItemCorporationView = (function(superClass) {
    extend(SellItemCorporationView, superClass);

    function SellItemCorporationView() {
      return SellItemCorporationView.__super__.constructor.apply(this, arguments);
    }

    SellItemCorporationView.prototype.template = template;

    SellItemCorporationView.prototype.minMaxTemplate = _.template(minMaxTemplate);

    SellItemCorporationView.prototype.data = {
      items: {
        pointer: 'tradeCenter.corporationItemsToSell',
        corporationId: function() {
          return this.options.corporationId;
        }
      }
    };

    SellItemCorporationView.prototype.events = {
      'click .o-switcher': 'changeCurrency',
      'click .js-create-lot': 'createLot',
      'click .js-select': 'selectItem',
      'click .js-choose-all-items': 'chooseAllItems',
      'keyup .js-items-amount': function() {
        this.calcCommission();
        return this.calcPrice();
      },
      'keyup .js-items-price': function() {
        this.calcCommission();
        return this.calcPrice();
      }
    };

    SellItemCorporationView.prototype.initialize = function(options) {
      SellItemCorporationView.__super__.initialize.call(this, options);
      return this.itemTypeComparatorFactory = itemTypeComparatorComponent.itemTypeComparatorFactory;
    };

    SellItemCorporationView.prototype.preprocessData = function(data) {
      var i, item, itemTypeComparator, len, ref;
      ref = data.items.corporationItemsArray;
      for (i = 0, len = ref.length; i < len; i++) {
        item = ref[i];
        item.id = item.ItemType.id;
        item.title = item.ItemType.name + " (" + item.CorporationItem.quantity + ")";
      }
      itemTypeComparator = this.itemTypeComparatorFactory(data.items.categorySortWeight || {});
      data.items.corporationItemsArray = data.items.corporationItemsArray.sort(itemTypeComparator);
      this.corporationItemsCollection = new Backbone.Collection(data['items']['corporationItemsArray']);
      return data;
    };

    SellItemCorporationView.prototype.calcCommission = function() {
      var commissionPercent, commissionValue, companyHave, formattedCommissionValue, item, itemId, itemsCount, pricePerItem;
      itemsCount = parseInt(this.$el.find('.js-items-amount').val());
      pricePerItem = parseFloat(this.$el.find('.js-items-price').val());
      itemId = parseInt(this.$('.js-select').attr("data-id"));
      item = this.corporationItemsCollection.get(itemId);
      if (!item) {
        this.$el.find('.js-commission').html(0);
        return;
      }
      companyHave = item.get('CorporationItem')['quantity'];
      commissionPercent = this._data.items.commission;
      if (isNaN(itemsCount)) {
        itemsCount = 0;
      }
      if (itemsCount > companyHave) {
        itemsCount = companyHave;
      }
      this.$el.find('.js-items-amount').val(itemsCount);
      if (isNaN(pricePerItem)) {
        pricePerItem = 0;
        this.$el.find('.js-items-price').val('');
      }
      commissionValue = parseFloat(itemsCount * pricePerItem * commissionPercent / 100);
      formattedCommissionValue = this.templateHelpers().formatNumber(commissionValue);
      return this.$el.find('.js-commission').html(formattedCommissionValue);
    };

    SellItemCorporationView.prototype.calcPrice = function() {
      var companyHave, formattedLotPrice, item, itemId, itemsCount, lotPrice, pricePerItem;
      itemsCount = parseInt(this.$el.find('.js-items-amount').val());
      pricePerItem = parseFloat(this.$el.find('.js-items-price').val());
      itemId = parseInt(this.$('.js-select').attr("data-id"));
      item = this.corporationItemsCollection.get(itemId);
      if (!item) {
        this.$el.find('.js-lot-price').html(0);
        return;
      }
      companyHave = item.get('CorporationItem')['quantity'];
      if (isNaN(itemsCount)) {
        itemsCount = 0;
      }
      if (itemsCount > companyHave) {
        itemsCount = companyHave;
      }
      if (isNaN(pricePerItem)) {
        pricePerItem = 0;
      }
      lotPrice = parseFloat(itemsCount * pricePerItem);
      formattedLotPrice = this.templateHelpers().formatNumber(lotPrice);
      return this.$el.find('.js-lot-price').html(formattedLotPrice);
    };

    SellItemCorporationView.prototype.changeCurrency = function(event) {
      $(event.currentTarget).toggleClass('o-switcher--active');
      if (this.$el.find('.o-switcher').hasClass('o-switcher--active')) {
        this.$('.js-commission-currency').addClass('o-icon--vgold');
        return this.$('.js-commission-currency').removeClass('o-icon--vdollar');
      } else {
        this.$('.js-commission-currency').addClass('o-icon--vdollar');
        return this.$('.js-commission-currency').removeClass('o-icon--vgold');
      }
    };

    SellItemCorporationView.prototype.selectItem = function(event) {
      this.lockUI();
      return app.screens.dialog('common/select-dialog', {
        collection: this.corporationItemsCollection,
        callback: (function(_this) {
          return function(itemId) {
            var fragment, lots;
            event.currentTarget.setAttribute('data-id', itemId);
            _this.$('.js-select .js-item-name').html(_this.corporationItemsCollection.get(itemId).get("title"));
            fragment = document.createElement('DIV');
            lots = _this.serializeData();
            lots.elements = _this.corporationItemsCollection.get(itemId).get('lots');
            fragment.innerHTML = Marionette.Renderer.render(_this.minMaxTemplate, lots);
            $('.js-min-max').html(fragment.innerHTML);
            _this.calcCommission();
            return _this.calcPrice();
          };
        })(this)
      });
    };

    SellItemCorporationView.prototype.createLot = function(event) {
      var amount, currency, itemTypeId, loader, price;
      this.lockUI();
      itemTypeId = this.$('.js-select').attr("data-id");
      amount = this.$('.js-items-amount').val();
      if (!amount) {
        amount = 0;
      }
      price = this.$('.js-items-price').val();
      if (!price) {
        price = 0;
      }
      currency = this.$el.find('.o-switcher').hasClass('o-switcher--active') ? 'vgold' : 'vdollars';
      loader = this.setLoaderOnElement(event.currentTarget, app.tr('Selling'));
      return app.server.tradeCenter.createLotCorporation({
        corporationId: this.options.corporationId,
        itemTypeId: itemTypeId,
        amount: amount,
        price: price,
        currency: currency,
        success: (function(_this) {
          return function(data) {
            _this.unlockUI();
            loader.remove();
            return app.screens.back();
          };
        })(this),
        error: (function(_this) {
          return function(data) {
            _this.unlockUI();
            loader.remove();
            return app.screens.modal('common/info-popup', {
              msg: data.setFlash[0].msg || app.tr("Something went wrong")
            });
          };
        })(this)
      });
    };

    SellItemCorporationView.prototype.chooseAllItems = function() {
      var companyHave, itemId;
      itemId = parseInt(this.$('.js-select').attr("data-id"));
      if (itemId) {
        companyHave = this.corporationItemsCollection.get(itemId).get('CorporationItem')['quantity'];
      }
      this.$('.js-items-amount').val(companyHave);
      return this.calcCommission();
    };

    return SellItemCorporationView;

  })(ItemView);
  return SellItemCorporationView;
});
